/***************************************************************************
                          mattpar  -  description
                             -------------------
    begin                : Wed Aug 18 2004
    copyright            : (C) 2004 by gmp
    email                : peppe@newpeppe
  ***************************************************************************
  *                                                                         *
  *   NA48  simulation program.                                             *
  *                                                                         *
  ***************************************************************************/

#include "parm.h"
#include "mattpar.h"

using namespace std;


MattPar::~MattPar()
{
}

MattPar::MattPar(const char *nnome,const char *sstatus, int zZ, double aA, double iIntl,double dDedx,double xX0, double rRo)
{
    static MattPar *last = 0;
    static int totelem = 0;
    totelem++;
    id    = totelem;

    nome=nnome;
    status=sstatus;
    Z =   zZ;  //numero atomico
    A =   aA;  //numero atomico
    if(Z>0&&A>0) ZsuA=Z/A;
    else ZsuA=0.421;
    Intl= iIntl;  //interc length g/cm2
    Dedx= dDedx;  // de/dx Mev/g/cm2
    X0 =  xX0;  // rad length g/cm2
    Ro =  rRo;  // densita' g/cm3
    X0l=  Ro?X0/Ro:0.0;

//  linking
    if (last == 0) Elemento= this;
    else              last->next = this;
    last = this;
    this->next = 0;	// end linking

}



//------------------------------------------------------------
//                  S h o w m a t e r i a
//------------------------------------------------------------

void MattPar::
Show()
{
    MattPar *mp;
    mp = this;
    Gout<<"\n\n ------------------------------------------------";
    Gout<<"\n              M A T E R I A L I      list  \n";
    Gout<<"\n  id  nome  Stat  Z     A        Int      de/dx            X0           ro ";
    Gout<<"\n                               g/cm2    Mev/g/cm2   g/cm2       cm     g/cm3\n";
    while (mp != 0)
    {

//     Gout.precision(3);
        Gout<<setfill(' ');
        Gout.setf(ios::right,ios::adjustfield);
        Gout.setf(ios::floatfield);
        Gout<<setprecision(6)
        <<setw(3)<<mp->id<<" "
        <<setw(5)<<mp->nome<<" "
        <<setw(6)<<mp->status<<" "
        <<setw(3)<<mp->Z<<" "
        <<setw(8)<<mp->A<<" "
        <<setw(8)<<mp->Intl<<" "
        <<setw(8)<<mp->Dedx<<" "
        <<setw(9)<<mp->X0<<" "
        <<setw(9)<<mp->X0l<<" "
        <<setw(8)<<mp->Ro<<endl;
        mp = mp->next;
    }
    Gout<<"\n              M A T E R I A L I      end ";
    Gout<<"\n ------------------------------------------------"<<endl;
}

//------------------------------------------------------------
//                  P O I N T E R  to ...

//------------------------------------------------------------
MattPar * MattPar::
PntElem (string mater)
{
    MattPar *mp;
    mp = Elemento;
    while (mp != 0)  //Check  the list
    {
        if (mater.find(mp->nome) == 0) return mp;
        mp = mp->next;
    }
    Gout<<"\n Errore: Material - "<<mater<<" - not found or not yet defined... exit."<<std::endl;
    exit(0);
}

// ===============================================================
//    D A T A B A SE  D E I  M A T E R I A L I

//  Lista dei materiali per apparati sperimentali
//  Ricordare di aggiornare secondo le necessità.
//   nome  stato   Z    A      Intl    de/dx   X0     ro
void  Crea_materiali_db()
{
 MattPar *ptr;
//                    nome       stato   Z    A        Intl    de/dx    X0       ro
    ptr = new MattPar ("Fe"   ,"solid" , 26  ,55.85    ,132.1 , 1.451 , 13.84 ,  7.87      );
    ptr = new MattPar ("H2lq" ,"liq"   ,  1  , 1.01    ,52.0  , 4.103 , 63.04 ,  0.071     );  // idrogeno liquido
    ptr = new MattPar ("H2gs" ,"gas"   ,  1  , 1.01    ,50.8  , 4.103 , 61.28 ,  0.000084  );  // idrogeno gas 1 bar
    ptr = new MattPar ("He"   ,"gas"   ,  2  , 4.0     ,65.1  , 1.937 , 94.32 ,  0.000166  );  // elio gas 1 bar
    ptr = new MattPar ("Ne"   ,"gas"   ,  10 , 20.18   ,99.0  , 1.724 , 28.94 ,  0.000839  );    
    ptr = new MattPar ("H2O"  ,"liquid",  10 , 18.     ,83.3  , 1.992 , 36.08 ,  1.0     );
    ptr = new MattPar ("W"    ,"solid" ,  74 , 183.85  ,191.9 , 1.145 ,  6.76 ,  19.3      );
    ptr = new MattPar ("Pb"   ,"solid" ,  82 , 207.2   ,199.6 , 1.122 ,  6.54 ,  11.35     );
    ptr = new MattPar ("Cu"   ,"solid" ,  29 , 63.55   ,134.9 , 1.403 , 12.86 ,  8.96      );
    ptr = new MattPar ("Al"   ,"solid" ,  13 , 26.98   ,106.4 , 1.615 , 24.01 ,  2.70      );
    ptr = new MattPar ("Kr"   ,"liquid",  36 , 83.98   ,222.0 , 1.5   , 17.74 ,  3.7       );
    ptr = new MattPar ("Ckv"  ,"solid" ,  6  , 12.01   ,85.8  , 1.74  , 42.70 ,  19.40     );
    ptr = new MattPar ("Air"  ,"gas"   ,  7.8, 15.16   ,90.1  , 1.815 , 36.62 ,  0.001205  );  //aria controllare 7.8????
    ptr = new MattPar ("Str"  ,"straw" ,  0  ,  0.0    , 0.0  , 0.0   , 39.11 ,  0.040077  );  //definire ?????????

    ptr = new MattPar ("Ar"   ,"gas"   ,  18 , 39.948  ,117.2 , 1.519 , 19.55 ,  1.396     );
    ptr = new MattPar ("Gmw"  ,"gas"   ,  0  ,  0.0    , 0.0  , 1.5   ,  30.0 ,  0.006     );  //definire ?????????
    ptr = new MattPar ("Si"   ,"solid" ,  14 , 28.0855 ,106.0 , 1.664 , 21.82 ,  2.33      );
    ptr = new MattPar ("Scint","solid" ,  10 , 100.0   ,2.0   , 1.664 , 21.82 ,  1.33      );  //scintillatore plastico
    ptr = new MattPar ("Ldg"  ,"solid" ,  5. , 10.     ,100.0 , 2.0   , 7.87  ,  6.22      );
    ptr->Set_ZsuA(.42101);
    ptr = new MattPar ("none","none"   ,  0  ,  0.0    , 0.0  , 0.0   ,  0.0  ,  0.0       );
    ptr = new MattPar ("null" ,"none"  ,  0  ,  0.0    , 0.0  , 0.0   ,  0.0  ,  0.0       );
    ptr = new MattPar ("vuoto","vuoto" ,  0  ,  0.0    , 0.0  , 0.0   ,  0.0  ,  0.0       );
}

