/***************************************************************************
                          carte.cpp  -  description
                             -------------------
    begin                : Sat Dec 9 2000
    copyright            : (C) 2000 by Giuseppe M Pierazzini
    email                : pierazzini@unipi.it
 ***************************************************************************
 *                                                                         *
 *   NA48  simulation program.           New version    3.02.2005                               *
 *                                                                         *
 ***************************************************************************/
#include <parm.h>
#include <part_db.h>

//using namespace std;

/** \file  database.cpp

 Questo procedura genera il database delle particelle più comuni in uso nella simulazione e la lista dei decadimenti elementari con i ralativi rate.
<ul>
<li> Ogni particella è descritta da: una massa, una carica, una larghezza (se diversa da zero) e una vita media in cm (ctau) se diverso da zero;
     ctau=-1 per particellla  stabile, ctau=0  per risonanza o decadimento immediato.
<li>La procedura controlla che i retes di una stessa particella  siano normalizzati a 1.
<li>Vedi anche la classe Part_db in part_db.h e part_db.cpp
<li>I dati sono ricavati da Review of Particle Physics.
</ul>
<br>
*@author Giuseppe Pierazzini
*/


//--------------------------
void Part_db::Controlla_rates()
{
    Part_db *ptr=this;

    if (ptr==0)return;

    if (ptr->ncan==0)
    {
        ptr=ptr->next;
        if (ptr!=0) ptr->Controlla_rates();
        return;
    }

    int Nc=ptr->ncan;
//     Gout<<"\n  decadimento per  "<< ptr->nome << " in calali "<<Nc;
    if (Nc>1)
    {
        double totr=0;
        for (int i=0;i<Nc;i++)
        {
            totr+=ptr->Canale[i].rate;

// 	  Gout<<"\n "<< ptr->nome << " canale "<<i<<" rate "<< ptr->Canale[i].rate
// 	            <<" totr "<<totr;


        }
        if (totr<=0.) {
            Gout<<"\n Errore in particle data_base for "<< ptr->nome
            <<" \n tot Rate  "<<totr
            << "  è errato.. Exit! ....Controlla";
            exit(0);
        }
        if (totr<1.)
        {
            double norma=1./totr;
            for (int i=0;i<Nc;i++)
            {
                ptr->Canale[i].rate*=norma;
            }
            Gout<<"\n Rinormalizzata ad 1. la decay _chain della particella "<<ptr->nome;
        }
    }
    ptr=ptr->next;
    if (ptr!=0) ptr->Controlla_rates();
    return;

}
//-------------------

void Crea_part_db()
{

    Part_db *ptr;
    // define the particle data base
    //             Nome   vis     ch   massa      width    ctau(cm)
    ptr=new Part_db ( "gam",   1,  0,      0.0,      0.0,    -1.0 );
    Gam  =ptr->Get_Nome();
    ptr_Gam=ptr;
    ptr=new Part_db ( "nu",    0,  0,      0.0,      0.0,    -1.0 );
    Neutrino=ptr->Get_Nome();
    ptr_Neutrino=ptr;
    ptr=new Part_db ( "elec",  1, -1,  0.00051,      0.0,    -1.0 );
    Elec =ptr->Get_Nome();
    ptr_Elec=ptr;
    ptr=new Part_db ( "elep",  1,  1,  0.00051,      0.0,    -1.0 );
    Elep =ptr->Get_Nome();
    ptr_Elep=ptr;
    ptr=new Part_db ( "mum",   1, -1,  0.10565,      0.0, 65865.4 );
    Mum  =ptr->Get_Nome();
    ptr_Mum=ptr;
    ptr=new Part_db ( "mup",   1,  1,  0.10565,      0.0, 65865.4 );
    Mup  =ptr->Get_Nome();
    ptr_Mup=ptr;
    ptr=new Part_db ( "pim",   1, -1,  0.13957018,   0.0,  780.45 );
    Pim  =ptr->Get_Nome();
    ptr_Pim=ptr;
    ptr=new Part_db ( "pip",   1,  1,  0.13957018,   0.0,  780.45 );
    Pip  =ptr->Get_Nome();
    ptr_Pip=ptr;
    ptr=new Part_db ( "pi0",   0,  0,  0.1349766,    0.0,     0.0 );
    Pi0  =ptr->Get_Nome();
    ptr_Pi0=ptr;
    ptr=new Part_db ( "Km",    1, -1,  0.493677,     0.0,   371.3 );
    Kapm =ptr->Get_Nome();
    ptr_Kapm=ptr;
    ptr=new Part_db ( "Kp",    1,  1,  0.493677,     0.0,   371.3 );
    Kapp =ptr->Get_Nome();
    ptr_Kapp=ptr;
    ptr=new Part_db ( "Kl",    1,  0,  0.497648,     0.0,  1534.0 );
    Kapl =ptr->Get_Nome();
    ptr_Kapl=ptr;
    ptr=new Part_db ( "Ks",    1,  0,  0.497648,     0.0,  2.6842 );
    Kaps =ptr->Get_Nome();
    ptr_Kaps=ptr;
    ptr=new Part_db ( "K0",    1,  0,  0.497648,     0.0,     0.0 );
    Kap0 =ptr->Get_Nome();
    ptr_Kap0=ptr;
    ptr=new Part_db ( "a_K0",  1,  0,   0.4977,       0.,   0. );
    A_Kap0=ptr->Get_Nome();
    ptr=new Part_db ( "eta",   0,  0,  0.54880,      0.0,     0.0 );
    Eta  =ptr->Get_Nome();
    ptr_Eta=ptr;
//
    ptr=new Part_db ( "Ro",    1,  0,  0.77000,    0.153,     0.0 );
    ptr=new Part_db ( "W",     1,  0,  0.78300,    0.008,     0.0 );
    ptr=new Part_db ( "p"  ,   1,  1,  0.93827,      0.0,    -1.0 );
    Protone =ptr->Get_Nome();
    ptr_Protone=ptr;
    ptr=new Part_db ( "a_p",   1, -1,   0.93827,       0.,  -1. );
    A_Protone=ptr->Get_Nome();
    ptr=new Part_db ( "n"  ,   1,  0,  0.93957,      0.0,    -1.0 );
    Neutrone=ptr->Get_Nome();
    ptr_Neutrone=ptr;
    ptr=new Part_db ( "a_n",   1,  0,   0.93957,       0.,  -1. );
    A_Neutrone=ptr->Get_Nome();
    ptr=new Part_db ( "fy",    1,  0,  1.01940,  0.00441,     0.0 );
    ptr=new Part_db ( "a_lam", 1,  0,   1.1156,       0.,   7.89 );
    A_Lambda=ptr->Get_Nome();
    ptr_A_Lambda=ptr;
    ptr=new Part_db ( "lam",   1,  0,   1.1156,       0.,   7.89 );
    Lambda=ptr->Get_Nome();
    ptr_Lambda=ptr;
    
    
// atomi complessi per interazione p-gas.. vedui anche più sotto     
// attenzione sono messi non visibili........?????
  /*  
    ptr=new Part_db ( "uus",   0,  1,   1.1894,       0.,   0. );
    Uus=ptr->Get_Nome();
    ptr_Uus=ptr;
    ptr=new Part_db ( "a_uus", 0, -1,   1.1894,       0.,   0. );
    A_Uus=ptr->Get_Nome();
    ptr_A_Uus=ptr;
    ptr=new Part_db ( "uds",   0,  0,   1.1925,       0.,   0. );
    Uds=ptr->Get_Nome();
    ptr_Uds=ptr;
    ptr=new Part_db ( "a_uds", 0,  0,   1.1925,       0.,   0. );
    A_Uds=ptr->Get_Nome();
    ptr_A_Uds=ptr;
    ptr=new Part_db ( "dds",   0, -1,   1.1974,       0.,   0. );
    Dds=ptr->Get_Nome();
    ptr_Dds=ptr;
    ptr=new Part_db ( "a_dds", 0,  1,   1.1974,       0.,   0. );
    A_Dds=ptr->Get_Nome();
    ptr_A_Dds=ptr;
    ptr=new Part_db ( "uss",   0,  0,   1.3149,       0.,   0. );
    Uss=ptr->Get_Nome();
    ptr_Uss=ptr;
    ptr=new Part_db ( "a_uss", 0,  0,   1.3149,       0.,   0. );
    A_Uss=ptr->Get_Nome();
    ptr_A_Uss=ptr;
    ptr=new Part_db ( "dss",   0, -1,   1.3213,       0.,   0. );
    Dss=ptr->Get_Nome();
    ptr_Dss=ptr;
    ptr=new Part_db ( "a_dss", 0,  1,   1.3213,       0.,   0. );
    A_Dss=ptr->Get_Nome();
    ptr_A_Dss=ptr;
    ptr=new Part_db ( "sss",   0, -1,   1.6724,       0.,   0. );
    Sss=ptr->Get_Nome();
    ptr_Sss=ptr;
    ptr=new Part_db ( "a_sss",   0, -1,   1.6724,     0.,   0. );
    A_Sss=ptr->Get_Nome();
    ptr_A_Sss=ptr;

// lista degli ioni usati paer le interazioni su Gas...vedi reagas.h e reags.cpp
    ptr=new Part_db ( "AT33",  1,  1,   1.8756,       0.,  -1. );   // AT33= ptr->Get_Nome()
    ptr=new Part_db ( "AT34",  1,  2,   2.8084,       0.,  -1. );   // AT34= ptr->Get_Nome()
    ptr=new Part_db ( "AT35",  1,  1,   2.8089,       0.,  -1. );   // AT35= ptr->Get_Nome()
    ptr=new Part_db ( "AT36",  1,  2,   3.7274,       0.,  -1. );   // AT36= ptr->Get_Nome()
    ptr=new Part_db ( "AT37",  1,  3,   5.6015,       0.,  -1. );   // AT37= ptr->Get_Nome()
    ptr=new Part_db ( "AT38",  1,  2,   5.6055,       0.,  -1. );   // AT38= ptr->Get_Nome()
    ptr=new Part_db ( "AT39",  1,  3,   6.5338,       0.,  -1. );   // AT39= ptr->Get_Nome()
    ptr=new Part_db ( "AT40",  1,  4,   6.5342,       0.,  -1. );   // AT40= ptr->Get_Nome()
    ptr=new Part_db ( "AT41",  1,  3,   7.4714,       0.,  -1. );   //AT41= ptr->Get_Nome()
    ptr=new Part_db ( "AT42",  1,  5,   7.4723,       0.,  -1. );   // AT42= ptr->Get_Nome()
    ptr=new Part_db ( "AT43",  1,  2,   7.4825,       0.,  -1. );   // AT43= ptr->Get_Nome()
    ptr=new Part_db ( "AT44",  1,  4,   8.3928,       0.,  -1. );   // AT44= ptr->Get_Nome()
    ptr=new Part_db ( "AT45",  1,  3,   8.4069,       0.,  -1. );   // AT45= ptr->Get_Nome()
    ptr=new Part_db ( "AT46",  1,  6,   8.4093,       0.,  -1. );   //AT46= ptr->Get_Nome()
    ptr=new Part_db ( "AT47",  1,  5,   9.3244,       0.,  -1. );   //AT47= ptr->Get_Nome()
    ptr=new Part_db ( "AT48",  1,  4,   9.3255,       0.,  -1. );   // AT48= ptr->Get_Nome()
    ptr=new Part_db ( "AT49",  1,  6,   9.3276,       0.,  -1. );   // AT49= ptr->Get_Nome()
    ptr=new Part_db ( "AT50",  1,  5,  10.2526,       0.,  -1. );   // AT50= ptr->Get_Nome()
    ptr=new Part_db ( "AT51",  1,  6,  10.2540,       0.,  -1. );   // AT51= ptr->Get_Nome()
    ptr=new Part_db ( "AT52",  1,  4,  10.2646,       0.,  -1. );   // AT52= ptr->Get_Nome()
    ptr=new Part_db ( "AT53",  1,  3,  10.2857,       0.,  -1. );   // AT53= ptr->Get_Nome()
    ptr=new Part_db ( "AT54",  1,  8,  10.2882,       0.,  -1. );   // AT54= ptr->Get_Nome()
    ptr=new Part_db ( "AT55",  1,  6,  11.1749,       0.,  -1. );   // AT55= ptr->Get_Nome()
    ptr=new Part_db ( "AT56",  1,  5,  11.1887,       0.,  -1. );   //AT56= ptr->Get_Nome()
    ptr=new Part_db ( "AT57",  1,  7,  11.1917,       0.,  -1. );   // AT57= ptr->Get_Nome()
    ptr=new Part_db ( "AT58",  1,  4,  11.2010,       0.,  -1. );   // AT58= ptr->Get_Nome()
    ptr=new Part_db ( "AT59",  1,  6,  12.1095,       0.,  -1. );   // AT59= ptr->Get_Nome()
    ptr=new Part_db ( "AT60",  1,  7,  12.1112,       0.,  -1. );   // AT60= ptr->Get_Nome()
    ptr=new Part_db ( "AT61",  1,  5,  12.1234,       0.,  -1. );   // AT61= ptr->Get_Nome()
    ptr=new Part_db ( "AT62",  1,  8,  12.1285,       0.,  -1. );   // AT62= ptr->Get_Nome()
    ptr=new Part_db ( "AT63",  1,  7,  13.0402,       0.,  -1. );   // AT63= ptr->Get_Nome()
    ptr=new Part_db ( "AT64",  1,  6,  13.0409,       0.,  -1. );   // AT64= ptr->Get_Nome()
    ptr=new Part_db ( "AT65",  1,  8,  13.0448,       0.,  -1. );   //AT65= ptr->Get_Nome()
    ptr=new Part_db ( "AT66",  1,  7,  13.9689,       0.,  -1. );   //AT66= ptr->Get_Nome()
    ptr=new Part_db ( "AT67",  1,  8,  13.9712,       0.,  -1. );   //AT67= ptr->Get_Nome()
    ptr=new Part_db ( "Boo!",  0,  0,      0.0,      0.0,     0.0 );
    Booh =ptr->Get_Nome();
    ptr_boo=ptr;
*/

// REASINV_H ---  Sinv Particella dummy con massa ee o pp = sqrt(S);    
    ptr=new Part_db ( "sinv",   0,  0,      0.0,      0.0,     0.0 );
    Sinv =ptr->Get_Nome();   
    


//  dummy particles
    ptr=new Part_db ( "res",   0,  0,      0.0,      0.0,     0.0 );
    Res  =ptr->Get_Nome();
    ptr=new Part_db ( "res2",  0,  0,      0.0,      0.0,     0.0 );
    Res2 =ptr->Get_Nome();
    ptr=new Part_db ( "res3",  0,  0,      0.0,      0.0,     0.0 );
    Res3 =ptr->Get_Nome();
    ptr=new Part_db ( "res4",  0,  0,      0.0,      0.0,     0.0 );
    Res4 =ptr->Get_Nome();
    ptr=new Part_db ( "res5",  0,  0,      0.0,      0.0,     0.0 );
    Res5 =ptr->Get_Nome();
    
    ptr=new Part_db ( "vmas2",   0,  0,      0.0,      0.0,     0.0 );
    Vmas2  =ptr->Get_Nome();
    ptr=new Part_db ( "vmas3",  0,  0,      0.0,      0.0,     0.0 );
    Vmas3 =ptr->Get_Nome();
    
    
    ptr=new Part_db ( "dum",   0,  0,      0.0,      0.0,     0.0 );
    Dum  =ptr->Get_Nome();
    ptr=new Part_db ( "dum2",  0,  0,      0.0,      0.0,     0.0 );
    Dum2 =ptr->Get_Nome();
    ptr=new Part_db ( "dlz",   0,  0,      0.0,      0.0,     0.0 );
    Dlz  =ptr->Get_Nome();
   
    ptr=new Part_db ( "cms",   0,  0,      0.0,      0.0,     0.0 );
    Cms  =ptr->Get_Nome();

    
    
    
//   ************  lisat decadimenti elementari   *****
    
    
    ptr_Mum->Canale=new Decadimento[1];
    ptr_Mum->Canale[0].Define(1.00,Elec,Neutrino,Neutrino);
    ptr_Mum->ncan=1;

    ptr_Mup->Canale=new Decadimento[1];
    ptr_Mup->Canale[0].Define(1.00,Elep,Neutrino,Neutrino);
    ptr_Mup->ncan=1;

    ptr_Pim->Canale= new Decadimento[1];
    ptr_Pim->Canale[0].Define(1.00,Mum,Neutrino);
    ptr_Pim->ncan=1;

    ptr_Pip->Canale= new Decadimento[1];
    ptr_Pip->Canale[0].Define(1.00,Mup,Neutrino);
    ptr_Pip->ncan=1;

    ptr_Pi0->Canale= new Decadimento[1];
    ptr_Pi0->Canale[0].Define(1.00,Gam,Gam);
    ptr_Pi0->ncan=1;

    ptr_Kapm->Canale=new Decadimento[1];
    ptr_Kapm->Canale[0].Define(1.00,Mum,Neutrino);
    ptr_Kapm->ncan=1;

    ptr_Kapp->Canale=new Decadimento[1];
    ptr_Kapp->Canale[0].Define(1.00,Mup,Neutrino);
    ptr_Kapp->ncan=1;

    ptr_Kaps->Canale=new Decadimento[2];
    ptr_Kaps->Canale[0].Define(0.6920,Pip,Pim);
    ptr_Kaps->Canale[1].Define(0.3069,Pi0,Pi0);
    ptr_Kaps->ncan=2;

    ptr_Kapl->Canale=new Decadimento[6];
    ptr_Kapl->Canale[0].Define(0.2027,Pip,Elec,Neutrino);
    ptr_Kapl->Canale[1].Define(0.2027,Pim,Elep,Neutrino);
    ptr_Kapl->Canale[2].Define(0.1352,Pip,Mum,Neutrino);
    ptr_Kapl->Canale[3].Define(0.1352,Pim,Mup,Neutrino);
    ptr_Kapl->Canale[4].Define(0.1952,Pi0,Pi0,Pi0);
    ptr_Kapl->Canale[5].Define(0.1254,Pip,Pim,Pi0);
    
    ptr_Kapl->ncan=6;

    ptr_Lambda->Canale=new Decadimento[2];
    ptr_Lambda->Canale[0].Define(0.639,Protone,Pim);
    ptr_Lambda->Canale[1].Define(0.358,Neutrone,Pi0);
    ptr_Lambda->ncan=2;

    ptr_A_Lambda->Canale=new Decadimento[2];
    ptr_A_Lambda->Canale[0].Define(0.639,A_Protone,Pip);
    ptr_A_Lambda->Canale[1].Define(0.358,A_Neutrone,Pi0);
    ptr_A_Lambda->ncan=2;

    
    //Atomi complessi --attivare solo se necessario-- per esempio urto p-gas 
/*    
    ptr_Uus->Canale=new Decadimento[2];
    ptr_Uus->Canale[0].Define(0.5157,Protone,Pi0);
    ptr_Uus->Canale[1].Define(0.4843,Neutrone,Pip);
    ptr_Uus->ncan=2;

    ptr_A_Uus->Canale=new Decadimento[2];
    ptr_A_Uus->Canale[0].Define(0.5157,A_Protone,Pi0);
    ptr_A_Uus->Canale[1].Define(0.4843,A_Neutrone,Pim);
    ptr_A_Uus->ncan=2;

    ptr_Uds->Canale=new Decadimento[1];
    ptr_Uds->Canale[0].Define(1.0,Lambda,Gam);
    ptr_Uds->ncan=1;

    ptr_A_Uds->Canale=new Decadimento[1];
    ptr_A_Uds->Canale[0].Define(1.0,A_Lambda,Gam);
    ptr_A_Uds->ncan=1;

    ptr_Dds->Canale=new Decadimento[1];
    ptr_Dds->Canale[0].Define(1.0,Neutrone,Pim);
    ptr_Dds->ncan=1;

    ptr_A_Dds->Canale=new Decadimento[1];
    ptr_A_Dds->Canale[0].Define(1.0,A_Neutrone,Pip);
    ptr_A_Dds->ncan=1;

    ptr_Uss->Canale=new Decadimento[1];
    ptr_Uss->Canale[0].Define(1.0,Lambda,Pi0);
    ptr_Uss->ncan=1;

    ptr_A_Uss->Canale=new Decadimento[1];
    ptr_A_Uss->Canale[0].Define(1.0,A_Lambda,Pi0);
    ptr_A_Uss->ncan=1;

    ptr_Dss->Canale=new Decadimento[1];
    ptr_Dss->Canale[0].Define(1.0,Lambda,Pim);
    ptr_Dss->ncan=1;
*/


// controllo della decay_chain

    Gout<<"\n ==== Particle Data_base and Chaining done  si ha da fa "<<std::endl;
    Matter->Controlla_rates();
    Gout<<"\n ==== Particle Data_base and Chaining done ======"<<std::endl;
}











