/***************************************************************************
                          DevChanti.h  -  description
                             -------------------
    begin                : Sat Jul 2011
    copyright            : (C) 2001 by Giuseppe Pierazzini
    email                : peppe@unipi.it
 ***************************************************************************/

#ifndef DEVCHANTI_H
#define DEVCHANTI_H

#include "devrtrt.h"


/** NA62 device
 The Charged ANTI (CHANTI) detector is required in order to reduce critical background induced by
inelastic interactions of the beam with the collimator and the Gigatracker (GTK) stations as well as to
tag beam halo muons in the region immediately close to the beam. The most critical events are the
ones in which the inelastic interaction takes place in the last GTK station (GTK-3). In such cases, pions
or other particles produced in the interaction, if emitted at low angle, can reach the straw tracker
and mimic a K decay in the fiducial region. If no other track is detected, these events can appear like
a signal event, i.e. one single π+ in the final state. A GEANT4 simulation has shown that kaon
inelastic interactions with GTK-3 happens in about 1/103 cases, so that the combined rejection
factors of the analysis cuts and the CHANTI veto must lead to a remaining inefficiency of 10-8.

It is composed of six stations, placed inside
the vacuum tube respectively at 27-77-177-377-777-1577 mm distance from the GTK-3 
The rectangular hole inside each station is 50 mm in y and 90 mm in X due to the rectangular shape
of the beam. Outer square side length is 300 mm. For particles hitting the GTK-3 center the CHANTI
covers hermetically the angular region between 34 mrad and 1.38 rad wrt the beam axis, for
particles hitting one of the GTK-3 corners the coverage is hermetic between 50 mrad and 1.16 rad.

See CHANTI Detector Requirements publication
*/


class DevChanti : public DevRtRt  {
public: 
	DevChanti();
	~DevChanti();
virtual int SimulaDev();

};

#if defined(MAIN)
DevChanti *Chanti=0;
#else
extern DevChanti *Chanti;
#endif








#endif
