/***************************************************************************
                          cammino.cpp  -  description
                             -------------------
    begin                : 02.09.2011
    copyright            : (C) 2001 by Giuseppe Pierazzini
    email                : pierazzini@unipi.it
  ***************************************************************************
  *                                                                         *
  *   NA62.... M.C. simulation program.                                     *
  *                                                                         *
  ***************************************************************************/


#define CEPSI 1.0e-2

//        Tirrenia   GmP 21.08.2011
#include <ctype.h>
#include <parm.h>
#include <evento.h>
#include <device.h>
#include <reazione.h>

using namespace GmpConst;
using namespace std;
int Dbg_tracing=0;
//=====================================

//==================Get_camm ==================

/**

Procedura per calcolare il cammino libero di una particella prima di colpire la superficie, esterna o interna di un device. Ritorna il puntatore al device,  il cammino libero e definisce due flags di posizione: uno relativo alla posizione di provenienza ed uno alla posizione attuale.<br>
Vedi la tabella in cui gli indici mostrano la posizione della particella prima di iniziare la tracciatura
(prevpos) e la posizione attuale (nowpos) rispetto al device restituito.
<table>
<tr><td> prevpos </td><td>   nowpos </td><td>             definizione </td></tr>
<tr><td>    3    </td><td>     1    </td><td> da  posizione non definita (appena nata) ora esterna al dev</td></tr>
<tr><td>    1    </td><td>     1    </td><td> da  posizione esterna    ancora esterna</td></tr>
<tr><td>    3    </td><td>     0    </td><td> da  posizione non def    ora interna.</td></tr>
<tr><td>    1    </td><td>     0    </td><td> da  posizione esterna    ora interna</td></tr>
<tr><td>    0    </td><td>     0    </td><td> da  posizione interna ancora interna</td></tr>
<tr><td>    0    </td><td>    -1    </td><td> da  posizione interna ora nel buco</td></tr>
<tr><td>   -1    </td><td>    -1    </td><td> da  posizione nel buco ancora nel buco</td></tr>
</table>

In particolare il significato dell'indice:
<ul>
<li> 3 = posizione non definita; è il caso di una particella appena nata
<li> 1 = esterna al rivelatore o device
<li> 0 = interna al rivelatore
<li> -1 = internamente al "buco" del rivelatore; per esempio nel buco dove passa la pipe!
</ul>
*/

Device *Device::Get_camm ( Particella *pr )
{
    //  double tratto,path;  ...in device.h
    //  int posok,pos;
    //  double Last_distq=1.e+12;
    Device *Dvk,*Dev;

    Dvk=0;
    Dev=this;    // device di partenza per la loop
    int posok=3;
    double cammino=0.0;


    Dbg_tracing=0;   //  set to 1  to start debugging 
    Dbg_dev_start=0;
    Dbg_dev_stop=100;
    
//  DEBUG tracing
//  if(Eventi_Generati==2029) Dbg_tracing=1;
//  if (Eventi_Generati==52) Dbg_tracing=1;
//  Gout<<"\n Get_cam inizio "<<pr->Get_Nome();
    
    
 //  double path_ora =Zona;    
    //correzione gmp 14.04.2012
    double path_ora =Zona-pr->X.z;
    
    double path_for_debug=path_ora;

    double pathdafare=pr->path_max-pr->path_done;
    if (pathdafare<path_ora ) path_ora =pathdafare;

    while ( Dev!=0 )
    {

        if (Dbg_tracing==1&&Dev!=0&&Dev==Devloop) {

            Dev->pr_dev=pr;
            Dev->X_dev=Dev->Lab2cDev ( pr->X ); // trasla al centro dev e ruota
            Dev->X_dev.Norma();
            Dev->V_dev=Dev->Lab2Dev ( pr->Vers );
            pos_corrente=Dev->Posizione();
            Gout<<"\n";
            Dev->Spia_Camm("Inizio loop",pr,-100.0,path_ora ,"   ");
        }


        if ( Dev->accedi==99) break;  //no more dev to check

        if ( Dev->accedi>0|| Dev->escludi>0||Dev->rivela<0 )  // visibiltà
        {

            if ( Dev == Devloop ) {
                Devloop = Devloop->next; //  aggiorno Devloop
                if (Dbg_tracing==1&& Devloop!=0) Dev->Spia_Camm("ReStartloop",pr,cammino,path_ora ," devescluso! \n");
            }
            Dev=Dev->next;
            continue;
        }



        //--------------------=================----------------
        Dev->pr_dev=pr;
        Dev->X_dev=Dev->Lab2cDev ( pr->X ); // trasla al centro dev e ruota
        Dev->V_dev=Dev->Lab2Dev ( pr->Vers ); // routa il versore.... att. aggiornato!
        Dev->X_dev.Norma();






        double Dimdev=Dev->Devsize;
        if ( Dev->X_dev.norma>Dimdev )  // test grossolano ma veloce  per dev esterno
        {
            double dcs=Dev->X_dev%Dev->V_dev;   // dist. dal centro dev proiet. sul cammino
            if ( dcs>=0.0 )  // se > 0 la part si allontana....dev superato!
            {
                Dev->accedi=11;
                if ( Dev == Devloop )
                {
                    Devloop = Devloop->next; //  aggiorno Devloop
                    if (Dbg_tracing==1&& Devloop!=0)
                        Dev->Spia_Camm("Changloop..",pr,cammino,path_ora ," Accedi 11");
                }
                Dev=Dev->next;
                continue;
            }

            if (pr->charg==0.0&&
                    (Dev->X_dev&Dev->V_dev).Norma()>Dimdev  ) // va bene solo per neutri
            {
                Dev->accedi=12;   //no! se lo scattering o un campo cambia la direzione di volo!
                Dev=Dev->next;
                continue;
            }

            if ( Dev->X_dev.norma> Dimdev+pr->path_max-pr->path_done)
            {  //non ci si arriva mai???
                Dev->accedi=13;
                if (Dbg_tracing==1&& Devloop!=0)
                    Dev->Spia_Camm("Non ci arr.",pr,cammino,path_ora ," Accedi 13");
                Dev=Dev->next;
                continue;
            }

//             if ( Dev->X_dev.norma> 50.+Zona/3.)
//             {
//
//               if (Dbg_tracing==1&& Devloop!=0)
//                     Dev->Spia_Camm("Lontanza   ",pr,cammino,path_ora ," skip ");
//
//                 Dev=Dev->next;
//                 continue;
//             }


        }
        // check position carefullly.....



        int posto=Dev->pos_corrente= Dev->Posizione();

//    if ( pr->idm==14 ) Dev->SpiaPr ( " Posiz ", 49,49 ,-99.);



//--------------------------

        if ( posto==1 )              //Esterno
        {


            cammino=Dev->CamEster();



            if (Dbg_tracing==1&& Devloop!=0)
            {
                if (cammino<path_for_debug)
                    Dev->Spia_Camm(" Pos 1     ",pr,cammino,path_ora ," pos 1");
                if (cammino>0.0)path_for_debug=cammino;
            }
//-----------------------------------------
            if (cammino==-2.) {

                if (Dbg_tracing==1&& Devloop!=0)
                    Dev->Spia_Camm("Fuori tiro ",pr,cammino,path_ora ,"  ");


                if (Dev->accedi==14&& Dev == Devloop )
                {
                    Devloop = Devloop->next; //  aggiorno Devloop
                    if (Dbg_tracing==1&& Devloop!=0)
                        Dev->Spia_Camm("Changloop-2",pr,cammino,path_ora ," acc=14");
                }

                Dev=Dev->next;
                posok=posto;
                continue;
            }
//-----------------------------------------
            if (cammino >path_ora )
            {  // sono già più lontano del primo dev trovato
                // oppure ho superato il punto di decadimento
                Dev=Dev->next;
                posok=posto;
                continue;
            }



//-----------------------------------------
            if (cammino >Dev->X_dev.norma+Devsize )
            {  // vero per device molto distanti e magari non  in linea....salto

                if (Dbg_tracing==1&& Devloop!=0)
                    Dev->Spia_Camm("Norma_out ",pr,cammino,path_ora ," distante ");
                Dev=Dev->next;
                posok=posto;
                continue;


            }
//-----------------------------------------
            if (cammino==-1. )          // Dev not accessible or just crossed
            {
                Dev->accedi = 14;

                if (Dbg_tracing==1&& Devloop!=0)
                    Dev->Spia_Camm("Fuori/pass ",pr,cammino,path_ora ," acc=14");

                if ( Dev == Devloop )
                {
                    Devloop = Devloop->next; //  aggiorno Devloop
                    if (Dbg_tracing==1&& Devloop!=0)
                        Dev->Spia_Camm("Changloop-1",pr,cammino,path_ora ," acc=14");
                }
                Dev=Dev->next;
                posok=posto;
                continue;
            }
//--------------------------  qui ok ------------

            if (cammino <path_ora  )    // viaggia verso il dev...
            {
                path_ora =cammino;

// attenzione path interni: posok <= 0 sono all'interno di un dev quindi non ridefinisco Dvk....
                if ( posok>0 )
                {
                    Dvk=Dev;
                    posok=posto;
                }
            }
        }
//--------------------------
        else if ( posto==0 )               //Interno ..
        {
            // calcolo la dist dalle parete di uscita. o da un dev  interno
            cammino =Dev->CamInter();


            if (Dbg_tracing==1&& Devloop!=0)
            {
                Dev->Spia_Camm(" Pos 0     ",pr,cammino,path_ora ," pos 0");
            }

            if (cammino > path_ora &&Dbg_tracing==1&& Devloop!=0)
                Dev->Spia_Camm("Error 0 >>>",pr,cammino,path_ora ," pos 0");

            path_ora =cammino;  // corretto

            Dvk=Dev;
            posok=posto;
//  NOTA:!!!    Nell'interno di un device non ci sono per definizione altri device
//          quindi qui si può terminare la loop
            break;

//--------------------------
        }
        else if ( posto<0 )   //nel buco...  anche qui attenzione ai dev interni ......
        {
            cammino =Dev->CamBuco();


            if (Dbg_tracing==1&& Devloop!=0)
            {
                Dev->Spia_Camm(" Pos <0   ",pr,cammino,path_ora ," pos <0");
            }

//nel buco...  anche qui attenzione ai dev interni ...... se non ci sono altri dev (strutture)
// coinvolte si termina loop....
// NOTA: se un device ha un'altro interno, va segnalato.....si possono annidare più dev....


// se la particella decade il path_ora può essere minore del cammino calcolato all'interno del dev
// e quindi si lascia path_ora, altrimenti si ridefinisce path_ora

            if (cammino <path_ora  ) path_ora =cammino;

            Dvk=Dev;
            posok=posto;
            if (Dev->strutture_interne==0) break;  //esco se non ci sono strutture interne
//            }

        }

        Dev=Dev->next;

    } // end loop

    //**************************
    //set the index of loop_end

    Dev=Devloopend;
    while (Dev!=0)
    {
        if (Dev->accedi==99);
        else  if ( Dev->accedi>0|| Dev->escludi>0|| Dev->rivela<0 ) Dev->accedi=99;
        else {
            Devloopend=Dev;
            break;
        }
        Dev=Dev->up;
    }
//
//=======================

    if ( Dvk!=0 )
    {


        Dvk->pr_dev=pr;   // ricordo l'indirizzo della particella in tracing..
        Dvk->camm= path_ora +CEPSI;
        Dvk->prevpos=Dvk->nowpos;
        Dvk->nowpos=posok;


// DEBUG PRINTING
        if (Dbg_tracing==1)      Dvk->Spia_Camm("Select--->",pr,Dvk->camm,path_ora ," select ");
    }
    else if (Dbg_tracing==1) Gout<<"\n Cammino: no selection done.... go head! Remains cm "<<path_ora<< " to do...";



    return Dvk;
}

//========================================    E N D C A M ======================================

//===============================    S P I A    =========================================
void Device::Spia_Camm ( string Titin, Particella *pr, double cam_pre,double cam_ora,string Titout )
{
    if ( idev<Dbg_dev_start|| idev>Dbg_dev_stop ) return;
    Gout.setf ( ios::fixed,ios::floatfield );
    Gout<<setprecision ( 1 );
    Gout<<"\n Ev "<<setw ( 6 ) <<Eventi_Generati<<" "<<setw(12)<<Titin;


    Gout<<" [" <<setw (3) << Devloop->idev<<"->("<<setw (8)<<nome<<" "<<setw (3)<<idev
    <<")<-"<<setw (3)<<Devloopend->idev<<"] "

    <<" Cfz "<<setw(8)<<Cface.z
    <<" I "<<strutture_interne;
    Gout <<" "<<setw (4) <<pr->nome
    <<" X("<<setw ( 6 ) <<pr->X.x<<" "<<setw ( 6 ) <<pr->X.y<<" "<<setw ( 8 ) <<pr->X.z
    <<" path_da_fare "<<setw ( 8 )<<pr->path_max-pr->path_done

    <<" vz "<<setprecision ( 3 )<<setw ( 8 )<<V_dev.z<<setprecision ( 1 )

    <<") OK "<<setw ( 2)<<accedi<<setw ( 2 ) <<escludi<<setw ( 2) <<rivela
    <<" p "<<setw ( 2 ) <<pos_corrente
    <<" "<<setw ( 2 ) <<prevpos
    <<" "<<setw ( 2 ) <<nowpos
    <<" pth[ "<<setw ( 9 ) <<cam_pre<<setw ( 9 ) <<cam_ora
    <<Titout;

    Gout<<flush;
}
//================================  U T I L I T Y     C A M ===============================



double  Device::Cam_su_Rt( gvet X_indev, gvet V_indev,gvet Sizdev)
{
// cammino per colpire dall'esterno un dev (o buco) parallelepipedo di dimensioni  Sizdev;

//     double t=0.;
//     gvet Xnew;

    double cammino=-2.;
    double t=0.;

    // calcolo il cammino verso le sei superfici. Per dist>0
    // scelgo il cammino maggiore.
    // camm resta negativo se il dev non puo' essere colpito.

    /*****************  standar version ****/
    gvet Xnew;


    // piani z
    if ( fabs(X_indev.z)>Sizdev.z)
    {
        t=-1.;
        if (V_indev.z>0.0) t=-(Sizdev.z + X_indev.z)/V_indev.z;
        else if ( V_indev.z<-0.0)t=(Sizdev.z - X_indev.z)/V_indev.z;
        if (t>0.0&&t>cammino)cammino=t;
    }
    // piani x
    if ( fabs(X_indev.x)>Sizdev.x)
    {
        t=-1.;
        if (V_indev.x>0.0) t=-(Sizdev.x + X_indev.x)/V_indev.x;
        else if ( V_indev.x<-0.0)t=(Sizdev.x - X_indev.x)/V_indev.x;
        if (t>0.0&&t>cammino)cammino=t;
    }
// piani y
    if ( fabs(X_indev.y)>Sizdev.y)
    {
        t=-1.;
        if (V_indev.y>0.0) t=-(Sizdev.y + X_indev.y)/V_indev.y;
        else if (V_indev.y<-0.0)t=(Sizdev.y - X_indev.y)/V_indev.y;
        if (t>0.0&&t>cammino)cammino=t;
    }


    if (cammino>0.0) {  // test se arrivato sul dev
        gvet Xnew=X_indev+V_indev*cammino;
        if (fabs(Xnew.x)>Sizdev.x+0.001||fabs(Xnew.y)>Sizdev.y+0.001||fabs(Xnew.z)>Sizdev.z+0.001)
        {
            cammino=-2.;
        }
    }
    if (cammino<0.&&X_indev.norma<2.*Devsize) return -1.0; // se sono vicino ...chiudo il dev
    if (cammino>0.0)cammino+=0.01;
    return cammino;

}

//=====================================

double Device::Cam_da_Rt(gvet X_indev, gvet V_indev,gvet Sizdev)
{


    double cammino=1000000; //massima size 10 mila mt!!!
    double t=0.;

    // calcolo il cammino verso le sei superfici. Per dist>0
    // scelgo il cammino maggiore.
    
    /*****************  standar version ****/
    gvet Xnew;


    // piani z

    {
        t=-1.;
        if (V_indev.z<.0) t=-(Sizdev.z + X_indev.z)/V_indev.z;
        else if ( V_indev.z>-0.0)t=(Sizdev.z - X_indev.z)/V_indev.z;
        if (t>0.0&&t<cammino)cammino=t;

    }
    // piani x

    {
        t=-1.;
        if (V_indev.x<0.0) t=-(Sizdev.x + X_indev.x)/V_indev.x;
        else if ( V_indev.x>-0.0)t=(Sizdev.x - X_indev.x)/V_indev.x;
        if (t>0.0&&t<cammino)cammino=t;
    }
// piani y

    {
        t=-1.;
        if (V_indev.y<0.0) t=-(Sizdev.y + X_indev.y)/V_indev.y;
        else if (V_indev.y>-0.0)t=(Sizdev.y - X_indev.y)/V_indev.y;
        if (t>0.0&&t<cammino)cammino=t;
    }
    if (cammino<0.0)
//qui mai!!!! Dall'interno non posso uscire senza colpire il dev!!!
        Gout<<"\n *** Errore Cam_da Rt at    Ev "<<Eventi_Generati
        <<" "<<nome<<" "<<pr_dev->Get_Nome()<< " Cammino " <<cammino<<" correggere!!!! ";




    return cammino+0.01;

}
//===========================================

double Device::Cam_su_Cln(gvet X_indev,gvet V_indev,gvet Sizdev)
{// Revised 27.08.2011  gmp OK
// calcolo il cammino positivo verso la  superfice del cilindro.
    double t,cammino=-2.;
    double raggio=Sizdev.x;

// *****   controllo se colpisce una delle due  basi che sono sempre  paralleli al piano z=0

// piani z


//-----------------------------
    if ( fabs(X_indev.z)>Sizdev.z)
    {
        t=-1.;
        if (V_indev.z>.0) t=-(Sizdev.z + X_indev.z)/V_indev.z;
        else if ( V_indev.z<-0.0)t=(Sizdev.z - X_indev.z)/V_indev.z;
        if (t>0.0&&t>cammino)cammino=t;
    }

//      if(devtype==TypDevRich) cout <<"\n su cln0 "<< nome <<" "<<cammino;

    // se non colpisce nessun piano zeta
    if (cammino<0.)
    {
        if (X_indev.norma<2.*Devsize) return -1.; // se sono vicino chiudo il dev Perchè....
        return -2.0;
    }


// qui cammino >0
// colpisco la superficie laterale dall'esterno?
    if (X_indev.XYNorma()<=raggio) return cammino; //colpirebbe la superficie laterale dall'interno


//-----------------------------

    gvet VV(V_indev.x,V_indev.y,0.0) ;
    double sintet=VV.norma;
    if (sintet<0.001)return cammino; // volo parallelo all'asse...  esco
    VV=VV.Verso();
    gvet Xp(X_indev.x,X_indev.y,0.0);
    double  b=VV%Xp;

    double dist_min= fabs(VV.x*X_indev.y-VV.y*X_indev.x);
    if (dist_min> raggio) return -2.; //passo troppo lontano dall'asse del cilindro

// bene... allora trovo il punto di impatto con la superficie....
//   ora verifico se colpisco la superficie laterale nei limiti di zeta...
//  revised  27.08.2011 gmp

    double corda=sqrt(raggio*raggio-dist_min*dist_min); // in effetti è  la semicorda !!!

    t= (-corda-b )/sintet; // Scelgo la soluzione più vicina

//     if(t>0.0)
//     cout << "\n cln  "<<nome<<" "<<pr_dev->Get_Nome()<<" rg"
//          <<setw(8)<<raggio<<" xyn "<<setw(8)<< X_indev.XYNorma()<< " b "
//          <<setw(8)<<b<<" "<<setw(8)<<corda<< " soluzioni "
//          <<setw(8)<< t<<" e "<<setw(8)<<(corda-b )/sintet;

//

    if ( t>=0.0 )  //vera per definizione... ma .....
    {
        // check limiti in zeta
        double z=X_indev.z+V_indev.z*t;
        if ( fabs(z)<=Sizdev.z && t>cammino)
        {
            cammino=t;

            /*     cout<<"\nInside  "<<setprecision(3)<<setw(8)<<nome<< " "<<pr_dev->Get_Nome()<< " b "<<setw(9)<<b
              <<" dist "<<setw(9)<<dist_min
             <<" corda "<<setw(9)<<corda<< " t  "<<setw(9)<<t<<" z "<<setw(9)<<z<< " cammino  "<<setw(9)<<cammino;*/


        }
    }

// si può uscire con cammino=-1.0/-2. se il dev non viene colpito.
//if(devtype==TypDevRich)cout<<"\n su cln1 "<< nome <<" "<<cammino;
    return cammino+0.01;
}
//===========================================================

double Device::Cam_da_Cln(gvet X_indev,gvet V_indev,gvet Sizdev)
{// Revised 27.08.2011  gmp ok
    double cammino=1000000; // max 10 mila mt!!!!
    double t=-1.;
// calcolo il cammino verso le  superfici esterne
// quella cilindrica e le basi del cilindro.
    // piani z

    {
        if (V_indev.z<0.0) t=-(Sizdev.z + X_indev.z)/V_indev.z;
        else if ( V_indev.z>-0.00)t=(Sizdev.z - X_indev.z)/V_indev.z;
        if (t>0.0&&t<cammino)cammino=t;
    }

// superficie laterale....

//-----------------------------
    double raggio=Sizdev.x;
    gvet VV(V_indev.x,V_indev.y,0.0) ;
    double sintet=VV.norma;
    if (sintet<0.001)return cammino; // volo parallelo all'asse z...  esco
    VV=VV.Verso();
    gvet Xp(X_indev.x,X_indev.y,0.0);
    double  b=VV%Xp;

    double dist_min= fabs(VV.x*X_indev.y-VV.y*X_indev.x);


    if (dist_min> raggio) return cammino; //passo troppo lontano dall'asse del cilindro errore!!!

// bene... allora trovo il punto di impatto con la superficie....
//   ora verifico se colpisco la superficie laterale nei limiti di zeta...
//  revised  27.08.2011 gmp

    double corda=sqrt(raggio*raggio-dist_min*dist_min); // in effetti è  la semicorda !!!

    t= (corda-b )/sintet; // Scelgo la soluzione positiva

//

    if ( t>=0.0 )  //vera per definizione... ma .....
    {

        // check limiti in zeta
        double z=X_indev.z+V_indev.z*t;
        if ( fabs(z)<=Sizdev.z && t<cammino)
        {
            cammino=t;
        }
        return cammino;
    }
     double energia=pr_dev->P.e;
//qui mai!!!! Dall'interno non posso uscire senza colpire il dev!!!
    Gout<<"\n *** Errore Cam_da_Cln at   Ev "<<Eventi_Generati
    <<" "<<nome<<" "<<pr_dev->Get_Nome()<< " en "<<energia<< " Cammino " <<cammino<<" correggere!!!! ";
    pr_dev->X.print("X");
    pr_dev->P.print("P");
    Xp.print("xp");
    
    return -1.0;
}

//===========================================================
//
double  Device::Cam_su_Sfera( gvet X_indev, gvet V_indev,double raggio)
{
    double cammino=-2.0;
    double  dist_min=0.0, b=0.;


    b=X_indev%V_indev;






    gvet Dist=X_indev&V_indev;
    dist_min= Dist.Norma();





//        X_indev.print("Xin");
//        Gout<<" Dminall "<<dist_min<<" b "<<b;


    if (b>0.0) {// non si va verso la sfera, ci si allontana
        if (pr_dev->charg==0.0 ||X_indev.norma<3.*raggio)return -1. ;
        return -2.0;
    }






    if (dist_min>raggio)
    {// non si colpisce la sfera
        if (X_indev.norma< 2.*Devsize) return -1.; // se sono vicino chiudo il dev Perchè....
        return -2.0;
    }


    double corda=sqrt(raggio*raggio - Dist.normaq);   // metà corda della traiettoria nella sfera.
    cammino = -corda-b ;
    if ( cammino<0.0 )  { // per sicurezza...qui mai!!!! t deve essere positivo per definizione
        Gout<<"\n *** Errore Cam_su Sfera at   Ev "<<Eventi_Generati
        <<" "<<nome<<" "<<pr_dev->Get_Nome()<< " Cammino " <<cammino<<" correggere!!!! ";
        X_indev.print("X");
        Gout<<" corda "<<corda<<" b " <<b<<"  cammino "<< cammino;
        V_indev.print("V");
    }

    return cammino+0.01;
}
//===========================================================
//
double  Device::Cam_da_Sfera( gvet X_indev, gvet V_indev,double raggio)
{
    double cammino=-1.0;
    double  dist_min=0.0, b=0., corda=0.;


    b=X_indev%V_indev;
    gvet Dist=X_indev&V_indev;
    dist_min= Dist.Norma();

    //-------------
    if (dist_min>raggio) {//strano errore... qui per def dovrebbe essere  dist_min < raggio
        Gout<<"\n *** Errore 1 Cam_da_Sfera at   Ev "<<Eventi_Generati<< " Dist_min " <<dist_min
        <<" raggio "<< raggio

        <<" correggere!!!! ";
        cammino=-1.0;
        return cammino;
    }
    //-----------------
    corda=sqrt(raggio*raggio - Dist.normaq);   // metà corda della traiettoria nella sfera.
    cammino= corda-b;

    if ( cammino<0.0 )   // per sicurezza...qui mai!!!! t deve essere positivo per definizione
        Gout<<"\n *** Errore 2 Cam_da_Sfera  at   Ev "<<Eventi_Generati<< " Cammino " <<cammino<<" correggere!!!";




    return cammino;

}

