/***************************************************************************
                          device.cpp  -  description
                             -------------------
    begin                : Sat May 26 2001
    copyright            : (C) 2001 by Giuseppe Pierazzini
    email                : peppe@unipi.it
  ***************************************************************************
  *                                                                         *
  *   NA48  simulation program.                                             *
  *                                                                         *
  ***************************************************************************/
// procedura Get_cam in sviluppo  dal  25.08.2011

#define CEPSI 1.0e-6
#include "flyoh.h"

//        Tirrenia   GmP 4.09.
#include <ctype.h>

#define ESTERNO  1
#define INTERNO  0
#define INBUCO   -1
#define EPSR     1.0E-200

using namespace GmpConst;
using namespace std;

static int tot_device = 0;
void Scrivi_Dati ( int );
/**
Class base per i rivelatori. Il costruttore Device() legge i file ( di solito di tipo .app) degli apparati con le definizioni geometriche e dei materiali dei rivelatori.
*/
// classe Base
Device::Device()
{
        working_buffer=MemoryBuffer;
        Make_working_buffer ( working_buffer );
        Make_measur_buffer ( working_buffer );
        // create e new device
        tot_device++;
        idev = tot_device;
        if ( idev<200 ) Rivelatore[idev]=this; // se occorre più grande aumentare le dimensioni in device.h
        devtype=TypDevice;  //=0
        if ( last_device== 0 ) Apparato=this;
        else             last_device->next = this;   // linking
        up=last_device;
        last_device = this;
        this->next = 0;   // end linking

        // default orientation and zero setting
        Nx.setvn ( 1.,0.,0. );
        Ny.setvn ( 0.,1.,0. );
        Nz.setvn ( 0.,0.,1. );
        Tot_seen=0;
        for ( int i=0; i<40; i++ ) Part_hit[i]=0;
        nhit =ghcir= gnhit=nvede = -1;
        Ray_abs=0;
        ghit_last=mhit=0;
        strutture_interne=0;
        nowpos=prevpos=6;
        wrout=accedi=escludi=rivela=Zint=Zbuc=extrawork=0;
        registra=1;
        campo=-1;
        reckik=-1. ; // reconstruction Kik non defined
        mattint="";
        mattbuc="";
        maxBrems=maxPair=0;    //if >1 enable bremsstrhalung  and pair production
        Brem_done=Pair_done=0;
        eDev=t_dead_mem =time_now=Ro =Dedx =0.0;
        Roint=Robuc=Dedxint=Dedxbuc=0.0;
        Aint=Abuc=lradi = lradb=lrad= 0.0;
        fun=Nodef;       // set default function at None...

// legge i dati di definizione del dev
        Leggi_schede_dev();

//=============================================
// Dimensioni e Posizioni delle superfici del device:
// Lout(dx,dy,dz) semidimensioni del dev  nelle tre dimensioni. Se è cilindrico dx=dy=R
// poi vedi casi particolari nei costruttori delle classi dev.
// Lin(dx,dy,dz) semidimensioni del buco interno nelle tre dimesioni; poi come sopra.
// Centro(xc,yc,zc) è il centro del dev definito  nelle coordinate del lab (definito nelle data_cards)
// Centrin (xin,yin,zin) è il centro del buco interno riferito alle coordinate del dev
//       coincide con il centro se non è esplicitamente indicato.
// Cface (xcf,ycf,zcf) è il centro della superfice upstream in coordinate del lab
//       può essere fissato dall'utente al posto di Centro se il dev ha l'asse z parallelo all'asse z del label
//       altrimenti si deve dare il centro e l'angolo di rotazione rotx,roty .....

// definizione di Cface.
        gvet Cupstream ( 0.,0.,-Lout.z );
        Cface=Devc2Lab ( Cupstream );


}


//---------------------------------
Device::~Device()
{}
//--------------------------------------------
// making buffer procedures
void  Device::Make_working_buffer ( int dim )
{
        if ( dim>MemoryBuffer ) {
                delete[] Hits;
                delete[] G_Hits;
                delete[] M_Hits;
                Gout<<"\n In device ** "<<nome <<" **  working_buffer adjusted to ==> "<<dim;
        }

        circular_buffer=dim;
        if ( Pipeline>0 )  circular_buffer=4*dim;
        Hits  = new Bufdev[dim];
        G_Hits= new Bufdev[circular_buffer];
        M_Hits= new Bufdev[dim];

}
void  Device::Make_measur_buffer ( int dim )
{



        if ( dim>MemoryBuffer ) {
// nel primo dev non di delete.....
//if(idev==2) Gout<<"\n old pointer  "<< index<<" "<< fdx[0]<<" "<<&index<<" "<< &fdx;


                delete[] index;
                delete[] ido;
                delete[] mid;
                delete[] mdead;
                delete[] devstory;
                delete[] mxd;
                delete[] myd;
                delete[] mzd;
                delete[] pxd;
                delete[] pyd;
                delete[] pzd;
                delete[] pxl;
                delete[] pyl;
                delete[] pzl;
                delete[] pel;
                delete[] mdd;
                delete[] me;
                delete[] mxl;
                delete[] myl;
                delete[] mzl;
                delete[] gpx;
                delete[] gpy;
                delete[] gpz;
                delete[] ge;
                delete[] dtemp;
                delete[] structinfo;

                Gout<<"\n In device ** "<<nome <<" **  measur_buffer adjusted to ==> "<<dim;

        }

        index=new int[dim];
        ido=new int[dim];
        mid =new int[dim];
        mdead=new int[dim];
        devstory= new unsigned long long[dim];
        mxd=new float[dim];
        myd=new float[dim];
        mzd=new float[dim];
        mdd=new float[dim];
        me=new float[dim];
        pxd=new float[dim];
        pyd=new float[dim];
        pzd=new float[dim];
        pxl=new float[dim];
        pyl=new float[dim];
        pzl=new float[dim];
        pel=new float[dim];
        mxl=new float[dim];
        myl=new float[dim];
        mzl=new float[dim];
        gpx=new float[dim];
        gpy=new float[dim];
        gpz=new float[dim];
        ge=new float[dim];
        dtemp=new double[dim];
        structinfo=new int[dim];


}

/** Reset della posizione della particella rispetto al device.  */
void Device::Reset_Pos()
{
        // start from the tree top
        Device *pdev=Apparato;
        while ( pdev!=0 ) {
                nowpos=prevpos=6;
                pdev=pdev->next;
        }
}

//***********************  R E S E T ****************************

/** Reset dei device all'inizio e nella loop di generazione eventi */
void Reset_Devices()
{
// attenzione dove si mette questo reset... per la relazine tempoarle
        // start from the tree top...
        Device *pdev=Apparato;
        while ( pdev!=0 ) {
                pdev->Reset_Dev();
                pdev=pdev->next;
        }
}
/** Reset del device: chiamata da Reset_Devices(). */
void Device::Reset_Dev()
{
// attenzione dove si mette questo reset... si suppone all'inizio evento
// quando si conosce il tempo di generazione del nouvo evento ....

        nowpos=prevpos=6;  //posizione non definita
        accedi=0;
        rivela=0;
        eDev=0.0;
        Brem_done=Pair_done=0;
        nhit=-1;
        mhit = 0;

        if ( Pipeline==0 ) {
                gnhit = -1;  // hits generati
        }
}

//====================   s c r i v i   c o l p i  ================
/**
ScvrColpi stores the coordinates of the particles in the buffer of the detector [Hits]. For the study of the overlapping events, when the flag  pipeline is "on", the procedure copies data even in the circular buffer G_Hits!


*/
void  Device::ScrvColpi ( Particella *pr )
{
        /**
        RIVELA:  <br>
        <ul>
        <li>-1  = pr non seen
        <li> 0  = pr seen
        <li> 1  = pr all energy memorized  but tracing goes on
        <li> 2  = pr seen and dump and  stop tracing
        <li> 3  = pr seen but not registered
        <li> 4  = pr dump but not registered
        </ul>
        </P>
        */

//  cout<<"\n Ev Scrv "<<Eventi_Generati<< " dev "<<nome<< " pr "<<pr->nome<<" colpo "<<colpo<<" visto "<<visto;

//if(idev==52||idev==53){ pr->X.print("X");Gout<<" Ev "<<Eventi_Generati<<" id "<<idev
//  <<pr->nome <<" mom "<<pr->P.mom <<" in " <<nome;
//}


        if ( nhit> ( working_buffer-2 ) ) { //verifica la size del buffer!
                Gout<<"\n Ev "<< Eventi_Generati<<"  In dev "<<nome<<" too many particles "<< nhit<< "  get out! "<<endl;
                Scrivi_Dati ( 2 );
                flyoend();
                exit ( 0 );
        }


// aggiorna il contatore di dev per le particelle che lo attraversano
        Part_hit[Idsee]++;
        Tot_seen++;

        pr->devhit=idev;  //memorizzo l'ultimo indice del dev colpito.

        // salva in formato codificato i devices colpiti dalla particell
        // il bit acceso corrisponde al device colpito
        // attenzione memorizza solo i primi 64 device.
        pr->Update_Devstory ( idev );



        if ( rivela>2 ) return;   // non registered
// attenzione si evita anche  di registrare  gli hits per i dev che non sono esplicitamente
// usati nella analisi successiva. registra=(0,1)==(no,si)


        if ( registra==0 ) return;

//      if(nhit>-1&&pr==Hits[nhit].Pr){
//     Gout<<"\n Ev "<<Eventi_Generati<< " Scrivi colpi ";
//     pr->X.print("X");Gout<<" " <<nome;
//     pr->P.print("P");
//     qvet Pdev=Lab2Dev(pr->P);
//     Pdev.print("Pdev");
//     }

// write only the first time hit the dev...


        if ( nhit>-1&&pr==Hits[nhit].Pr&&prevpos==nowpos ) return;


        nhit++;       // remember nhit starts from -1 that corrisponds to no  hits...

        Bufdev  *bf = &Hits[nhit];
        bf->Pr=             pr;
        bf->tempo=          evento_.Gen.tempo;
        bf->idead=          pr->Ifato;
        bf->devstory=       pr->devstory;
        bf->last_path=      pr->last_path;
        bf->pnome =         pr->nome;
        bf->id=             pr->id;
        bf->ido=            pr->ido;
        bf->idm=            pr->idm;
        bf->Vers=           pr->Vers;
        bf->Xlab=           pr->X;
        bf->P=              pr->P;
        bf->Pgen=           pr->Gp;
        bf->Xdev=           Lab2cDev ( pr->X );
        bf->structinfo= Get_Structinfo();





//  ----------------------
        if ( rivela>0 ) bf->e_rivela=pr->P.e;
        else bf->e_rivela=0.0;   //it is done anyway if multiplescattering if required...

//  ------------- pipeline test -------
        if ( Pipeline==0 ) return;

// pipeline is on! copies the hits in the circular buffer

        gnhit++;       // hits index up to now...
        ghcir=gnhit%circular_buffer;  //attention... Circular buffer
        G_Hits[ghcir]=bf[0];
        G_Hits[ ( ghcir+1 ) %circular_buffer].tempo=-10;  //set to -10 next hit not yet arrived....

//    Gout<<"\n Eve_gen "<<evento_.Gen.Event<<" dev "<<nome<< " hit "<<gnhit<<" "<<ghcir<<" tempo "<< G_Hits[ghcir].tempo
//     <<" " << G_Hits[(ghcir+1)%circular_buffer].tempo;
        ;

};



/**
 La procedura è attivata se la flag Pipeline  is on, per recuperare l'evento successivo dal Buffer circolare G_Hits e copiarlo in M.Hits. <br>
L'evento viene poi elaborato da procedure di analisi e sottoposto a programmi di plotting (root/paw).
*/
void  Device::GetWindowHits ( double trigger_time )
{



        double dtm, adtm;
        int pipe=0;
        nhit=-1;
        mhit=0;


        if ( gnhit<0 ) return;  // gnhit: total hit seen up to now memorized in the circular buffer

        double tmorto=t_dead_mem;  // get dead time of the device
        if ( tmorto>0.0 ) tmorto=t_dead_mem +0.001;
        else tmorto=0.001;   //if zero set to 0.001 ns

        int ghit=ghit_last; // get the last hit
        ghit_last=0;

// Gout<<"\n In Dev " <<nome << " for  trig_time  " <<trigger_time;
// cout<<"\n In Dev " <<nome << " for  trig_time  " <<trigger_time;

        int ih=0;
        while ( ih<circular_buffer ) {
                ghit=ghit%circular_buffer;
                if ( G_Hits[ghit].tempo<-9. ) break; // test on last hit to break the loop...
                dtm = trigger_time -G_Hits[ghit].tempo;  // difference between trigger_time and hit_time...
                adtm=fabs ( dtm );

                if ( adtm<=tmorto ) { //Test if the hit is inside the dead_time....
                        pipe=0;
                        nhit++;
                        if ( adtm>0.001 ) {
                                pipe=1;
                                evento_.Gen.Pipev=1;
                        }
                        M_Hits[nhit]=G_Hits[ghit];   //copy from circular buffer into M_Hits...
                        M_Hits[nhit].Pileup=pipe;
                        if ( ghit_last==0 ) ghit_last=ghit;

                } else if ( dtm>0.0 ) ghit_last=ghit;
                else if ( dtm<0.0 ) break;

//      Gout<<" Lasthit "<<setw(5)<<ghit_last<<"  pipe "<<pipe ;

                if ( nhit> ( working_buffer-2 ) ) {
                        Gout<<"\n Ev "<< evento_.Gen.Event<<"  Too many hits in dev " << nome<< "  see GetWindowHits()... exit" <<endl;
                        exit ( 0 );
                }
                ghit++;
                ih++;
        }
        mhit=nhit+1;
        //   if(mhit>0)Gout<<"\n Ev "<< Eventi_Generati<<"  In dev "<<nome<<"  Collected hits "<<mhit;
}


//***************************************************
//======== fill default dev coordinate type(2) with type(1).====
/** Se il Pipeline non è attivo si caricano i dati da G_Hits in M_Hits in successione sincrona con la generazione. */

int Device::Fill_Default()
{
        mhit=nhit+1;
//  Gout<<"\n Default copy Ev "<< evento_.Gen.Event<<" dev "<<nome<< " hits "<< mhit;

        for ( int i=0; i<mhit; i++ ) {
                M_Hits[i] =Hits[i];
        }
        return mhit;
}

//===============================================================
int Device::Get_DatiOut()
{
        /**
        Copia  i dati ricostruiti (flags, coordinate e parametri cinematici), da M.Hits
        nelle variabili per il file Root di uscita. <br>
        <ul>
        <li> int hit_recorded, mid[30]
        <li> double mxd[30],myd[30],mzd[30],me[30],... etcc.
        </ul>
        che sono i buffer utili alla fine evento per riempire gli istogrammi in root.
        Vedi la procedura Outnt().<br><br>
        Attenzione questa routine deve essere chiamata dopo DataSmear() e SimulaDev().<br>
        Se SimulaDev() è dummy... mhit deve essere comunque definito correttamente; Vedi  Fill_Default().

        */
        hit_recorded=mhit;

        emDev=0.;
        if ( mhit>0 ) {
// test on the total hits
                if ( mhit>working_buffer ) {
                        Gout<<"\n Ev "<< Eventi_Generati<<" <Measur_buffer> too low  in "<<nome<< " Detected hits: "<<mhit
                            << " cut to  "<<working_buffer;
                        mhit=working_buffer;
                }

                hit_recorded=mhit;
                for ( int hts=0; hts<hit_recorded; hts++ ) {
                        index[hts]=M_Hits[hts].id;
                        ido[hts]=M_Hits[hts].ido;
                        mid[hts]=M_Hits[hts].idm;
                        mdead[hts]=M_Hits[hts].idead;
                        structinfo[hts]=M_Hits[hts].structinfo;
                        devstory[hts]=M_Hits[hts].devstory;
                        M_Hits[hts].Xdev.putv ( mxd[hts],myd[hts],mzd[hts] );
                        mdd[hts]= M_Hits[hts].Xdev.XYNorma();
                        me[hts]=M_Hits[hts].e_rivela;
                        M_Hits[hts].Xlab.putv ( mxl[hts],myl[hts],mzl[hts] );
                        M_Hits[hts].P.putq ( pxl[hts],pyl[hts],pzl[hts],pel[hts] );
                        // impulso come generato
                        M_Hits[hts].Pgen.putq ( gpx[hts],gpy[hts],gpz[hts],ge[hts] );

                        gvet Puls = M_Hits[hts].P;
                        Puls=Lab2Dev ( Puls );
                        Puls.putv ( pxd[hts],pyd[hts],pzd[hts] );
                        dtemp[hts]=M_Hits[hts].tempo-evento_.Gen.tempo;
                        emDev+=me[hts];

                }
        }

        return hit_recorded;
}

//*********************************************************

//------------------------------------------
void  Device::Get_Materiali()
{
        MattPar *Elem;
        // nel interno del dev
        //   Gout<<"\n Elemeto interno "<< mattint;
        if ( mattint.size() !=0 ) {
                Elem= Elemento->PntElem ( mattint );
                Zint   =Elem->Get_Z();   // carica atomica
                Aint   =Elem->Get_A();   // numero atomico
                liint  =Elem->Get_Intl();// lungh. interazione cm
                Roint  =Elem->Get_Ro();  // densita' g/cm3
                Dedxint=Elem->Get_Dedxl();// perdita energia Gev/cm
                lradi  =Elem->Get_Radl();//  rad in cm
        }

        if ( mattbuc.size() !=0 ) {
                Elem= Elemento->PntElem ( mattbuc );
                Zbuc   =Elem->Get_Z();   // carica atomica
                Abuc   =Elem->Get_A();   // numero atomico
                libuc  =Elem->Get_Intl();// lungh. interazione cm
                Robuc  =Elem->Get_Ro();  // densita' g/cm3
                Dedxbuc=Elem->Get_Dedxl();// perdita energia Gev/cm
                lradb  =Elem->Get_Radl();//  rad in cm
        }
}
//=============================================================================
//------------------------------------------



void  Device::Get_Pointers()
{
        /**
        Definisce alcuni pointer di device ritenuti utili nell'eseguire più velocemente
        la simulazione.<br>
        Attenzione questi pointers sono di tipo device e quindi puntano alle variabili definite in
        device.h  e non conoscono i parametri definiti nelle classi derivate, mentre  possono chiamare le procedure
        virtuali di ciascuna classe derivata e via questi i parmetri delle classe derivate.
        */



        // to be updated according to the device names....
        // get dev pointers
        Gout<<"\n Device::Get_pointers() : get the device pointers to speed up the simulation..."
            <<"\n The user must adjust the device labels occording to names as defined in  .app file \n";


        collo=   Apparato->Get_Pointer ( "Coll" );
        xch02=   Apparato->Get_Pointer ( "XCH02" );
        spia0 = Apparato->Get_Pointer ( "spia0" );
        spia3 = Apparato->Get_Pointer ( "spia3" );
        vtappo = Apparato->Get_Pointer ( "Vtappo" );

        dectubo= Apparato->Get_Pointer ( "dectubo" );
        kabs1= ( Devkabes* )  Apparato->Get_Pointer ( "Kbs1" );
        kabs2= ( Devkabes* )  Apparato->Get_Pointer ( "Kbs2" );
        kabs3= ( Devkabes* )  Apparato->Get_Pointer ( "Kbs3" );

        giga1= ( DevGtk* ) Apparato->Get_Pointer ( "Gtk1" );
        giga2= ( DevGtk* ) Apparato->Get_Pointer ( "Gtk2" );
        giga3= ( DevGtk* ) Apparato->Get_Pointer ( "Gtk3" );
        cedar= ( DevCedar* ) Apparato->Get_Pointer ( "cedar" );
        mgb1 =   Apparato->Get_Pointer ( "MCB80" );   // magnet pointers of acromat 2
        mgb2 =   Apparato->Get_Pointer ( "MCB84" );
        mgb3 =   Apparato->Get_Pointer ( "MCB95" );
        mgb4 =   Apparato->Get_Pointer ( "MCB97" );
        mgb5 =   Apparato->Get_Pointer ( "MDX99" );

        Chanti = ( DevChanti* ) Apparato->Get_Pointer ( "chanti" );

        // get anticounter pointers...

        anti[0]= ( DevLav* ) Apparato->Get_Pointer ( "Ant1" );
        anti[1]= ( DevLav* ) Apparato->Get_Pointer ( "Ant2" );
        anti[2]= ( DevLav* ) Apparato->Get_Pointer ( "Ant3" );
        anti[3]= ( DevLav* ) Apparato->Get_Pointer ( "Ant4" );
        anti[4]= ( DevLav* ) Apparato->Get_Pointer ( "Ant5" );
        anti[5]= ( DevLav* ) Apparato->Get_Pointer ( "Ant6" );
        anti[6]= ( DevLav* ) Apparato->Get_Pointer ( "Ant7" );
        anti[7]= ( DevLav* ) Apparato->Get_Pointer ( "Ant8" );
        anti[8]= ( DevLav* ) Apparato->Get_Pointer ( "Ant9" );
        anti[9]= ( DevLav* ) Apparato->Get_Pointer ( "Ant10" );
        anti[10]= ( DevLav* ) Apparato->Get_Pointer ( "Ant11" );
        anti[11]= ( DevLav* ) Apparato->Get_Pointer ( "Ant12" );


        Cam1= ( DevMwc* )  Apparato->Get_Pointer ( "Cam1" );
        Cam2= ( DevMwc* )  Apparato->Get_Pointer ( "Cam2" );
        Cam3= ( DevMwc* )  Apparato->Get_Pointer ( "Cam3" );
        Cam4= ( DevMwc* )  Apparato->Get_Pointer ( "Cam4" );

        Rich= ( DevRich* )   Apparato->Get_Pointer ( "Rich" );

        St1x= ( DevStraw* )  Apparato->Get_Pointer ( "St1x" ); // la coordinata è parallela ai fili attenzione
        St1y= ( DevStraw* )  Apparato->Get_Pointer ( "St1y" );
        St1u= ( DevStraw* )  Apparato->Get_Pointer ( "St1u" );
        St1v= ( DevStraw* )  Apparato->Get_Pointer ( "St1v" );
        St2x= ( DevStraw* )  Apparato->Get_Pointer ( "St2x" );
        St2y= ( DevStraw* )  Apparato->Get_Pointer ( "St2y" );
        St2u= ( DevStraw* )  Apparato->Get_Pointer ( "St2u" );
        St2v= ( DevStraw* )  Apparato->Get_Pointer ( "St2v" );
        St3x= ( DevStraw* )  Apparato->Get_Pointer ( "St3x" );
        St3y= ( DevStraw* )  Apparato->Get_Pointer ( "St3y" );
        St3u= ( DevStraw* )  Apparato->Get_Pointer ( "St3u" );
        St3v= ( DevStraw* )  Apparato->Get_Pointer ( "St3v" );
        St4x= ( DevStraw* )  Apparato->Get_Pointer ( "St4x" );
        St4y= ( DevStraw* )  Apparato->Get_Pointer ( "St4y" );
        St4u= ( DevStraw* )  Apparato->Get_Pointer ( "St4u" );
        St4v= ( DevStraw* )  Apparato->Get_Pointer ( "St4v" );

        mnp1 = ( DevMagn* ) Apparato->Get_Pointer ( "Mnp1" );
        mnp2 = ( DevMagn* )  Apparato->Get_Pointer ( "Mnp2" );
        Mag48= ( DevMagn* )  Apparato->Get_Pointer ( "Magn" );
        Mnp33= ( DevMagn* )  Apparato->Get_Pointer ( "Mnp33" );

        hodo = ( DevChod* ) Apparato->Get_Pointer ( "Hodo" );
        hodox = ( DevChod* ) Apparato->Get_Pointer ( "Hodx" );
        hodoy = ( DevChod* ) Apparato->Get_Pointer ( "Hody" );

        hadc = ( DevHac* )  Apparato->Get_Pointer ( "HACM" );
        iron  =  Apparato->Get_Pointer ( "Ironwall" );
        muveto= ( DevMuv* ) Apparato->Get_Pointer ( "Muvt" );
        muvet1= ( DevMuv* ) Apparato->Get_Pointer ( "MUVET1" );
        muvet2= ( DevMuv* ) Apparato->Get_Pointer ( "MUVET2" );
        muvet3= ( DevMuv* ) Apparato->Get_Pointer ( "MUVET3" );
        monit = Apparato->Get_Pointer ( "Moni" );
        mamud=   Apparato->Get_Pointer ( "Mamu" );

        sacdev= ( DevSac* ) Apparato->Get_Pointer ( "Sac" );
        ircdev= ( DevIrc* ) Apparato->Get_Pointer ( "IRC" );
//    lkry=   (DevLkr*) Apparato->Get_Pointer ( "Lkry" );
//    if ( lkry==0 ) lkry= (DevLkr*)Apparato->Get_Pointer ( "Lkr" );


}
//------static Device *Walls,*lkry;
//  static Device *Coll, *Updet,*Veto,*Buco,*Walls,*lkry;

/**
Trova il pointer di un particolare device, dato il nome.<br>
Vedi anche  la procedura in overloading Get_Pointer ().
*/


Device * Device::Get_Pointer ( const  char *word )
{
        Device *dv,*dvok=0;
        dv=Apparato;
        while ( dv!=0 ) {
                if ( dv->escludi==0 &&
                                strcmp ( dv->nome,word ) ==0 ) dvok= dv;
                dv=dv->next;
        }
        Gout<<"\n Device "<< setw ( 9 ) <<word<<" pointer "<< ( dvok?" found":" === not ===  found or excluded" );
        return dvok;
}



//------------------------------------------
//  accedi=escludi=0;     trasformazioni lab -> dev -> lab
//--------------------------------------------
gvet& Device::Lab2Dev ( gvet &Vlab )
{
        // trasformazioni di vettori direzione lab-> dev.
        static gvet _Vdev;
        _Vdev.setvn ( Vlab%Nx,Vlab%Ny,Vlab%Nz );
        return _Vdev;
}

qvet& Device::Lab2Dev ( qvet &Plab )
{
        // trasformazioni di quadrivettori.  direzione lab-> dev.
        static qvet _Pdev;
        _Pdev.setvn ( Plab%Nx,Plab%Ny,Plab%Nz,Plab.m );
        return _Pdev;
}
//--------------------------------------------
gvet& Device::Lab2cDev ( gvet &Xin )
{
        // trasformazione di vettori al dev con traslazione...

        static gvet _Xdc,_Xdev;
        _Xdc=Xin-Centro;
        _Xdev.setvn ( _Xdc%Nx,_Xdc%Ny,_Xdc%Nz );
        return _Xdev;
}

//--------------------------------------------
gvet& Device::Dev2Lab ( gvet &Vdev )
{
        // trasformazioni di vettori direzione dev->lab.
        // calcola la norma.
        static gvet _Vlab;
        _Vlab=Nx*Vdev.x+ Ny*Vdev.y + Nz*Vdev.z;
        _Vlab.Norma();
        return _Vlab;
}

qvet& Device::Dev2Lab ( qvet &Pdev )
{
        // trasformazioni di quadrivettori. direzione dev->lab.
        // calcola la norma.
        static qvet _Plab;
        gvet Puls=Nx*Pdev.x + Ny*Pdev.y + Nz*Pdev.z;
        _Plab.setvn ( Puls,Pdev.m );
        return _Plab;
}

//--------------------------------------------
gvet& Device::Devc2Lab ( gvet &Xdev )
{
        // trasformazione di vettori dal Dev al lab con traslazione...
        // non calcola la norma...
        static gvet _Xlab;
        _Xlab=Nx*Xdev.x + Ny*Xdev.y + Nz*Xdev.z;
        _Xlab=_Xlab+Centro;
        return  _Xlab;
}

//***************************************************
//           Set_Rivela
//--------------------------------------------
void Set_Rivela_in_Dev ( Particella *pr )
{
        // start from the tree top...
        Device *pdev=Apparato;
        while ( pdev!=0 ) {
                pdev->Set_Rivela ( pr );
                pdev=pdev->next;
        }
}

//--------------------------------------------
void Device::Set_Rivela ( Particella *pr )
{
        /**
         <P>
        RIVELA:  <br>
        <ul>
        <li>-1  = pr non seen
        <li> 0  = pr seen
        <li> 1  = pr all energy memorized  but tracing goes on
        <li> 2  = pr seen and dump and  stop tracing
        <li> 3  = pr seen but not registered
        <li> 4  = pr dump but not registered
        </ul>
        </P>
        */
        int isee;
        const char *label;



        if ( escludi!=1 ) {
                // dev visibile
                accedi=0;
                rivela=-1;
                // test if the dev sees the particle ...first by name
                isee = nvede;
                while ( isee >= 0 ) {
                        label = vede[isee];
                        if ( label == pr->nome ) {
                                rivela=assorb[isee];
                                Idsee=isee;
                                break;
                        }
                        isee--;
                }
                // ... and now  by type
                if ( rivela==-1 )

                {
                        isee = nvede;
                        while ( isee >= 0 ) {
                                label = vede[isee];
                                if ( label == All ||label == pr->type ||
                                                ( label == Char && pr->charg!=0. ) ) {
                                        rivela=assorb[isee];
                                        Idsee=isee;
                                        break;
                                }
                                isee--;
                        }
                }
        }

// if(idev>1&&idev<4)
// Gout<<"\n Riv "<< nome <<" "<<pr->nome<<" " <<accedi << " " <<rivela;


}

