/***************************************************************************
                          device.h  -  description
                             -------------------
    begin                : Sat May 26 2001
    copyright            : (C) 2001 by Giuseppe Pierazzini
    email                : peppe@unipi.it
 ***************************************************************************/

#ifndef DEVICE_H
#define DEVICE_H
#include "gvet.h"
#include <string.h>
#include<cstring>

#if defined(MAINFLYO)
// Flags to define the device fuction....
const char *Veto = "veto", *Trig = "trig", *Trigh = "hard_t";  // Fun
const char *None = "none", *Dead = "dead", *Dump = "dump";  // Fun
const char *Magn = "magn" ,*Nodef ="nodef",*Bersaglio="bersag"; // Fun
const char *Detect = "det";
#endif
#if !defined(MAINFLYO)
extern const char *Veto, *Trig, *Trigh, *Magn;  // Fun
extern const char *None,*Dead, *Dump, *Nodef,*Bersaglio; // Fun
extern const char *Detect;
#endif


//class Particella;
#include "particella.h"


void Set_Rivela_in_Dev ( Particella * );
void Reset_Devices();


// struttura per la memorizzazione dei dati....
/**
Strutura dati hit particelle per i Buffer di Pileup;  Vedi device.
 *@author GmP
*/


struct Bufdev {
        Particella *Pr;
        int id;             //indice logico vedi Set:Id()
        int  ido;           // Indice della particella all'interno della reazione
        int  idm;           // Indice di materia, ovvero  tipo di particella rivelata
        int  idead;         // Tipo di morte, fato
        int  structinfo;    // Posizione della particela rivelata
        unsigned long long devstory; //storia dei dev attraversati... vedi meglio
        int Pileup;         // indice di pileup
        const char *pnome;        // nome della particella
        double last_path;   // cammino fatto nel dev
        double tempo;       // tempo di generazione  in ns.
        double e_rivela;    // energia della particella rivelata dal dev
        gvet Xlab,Xdev;     // cordinate nel lab e dev al punto di impatto
        qvet P,Pgen;           // quadrivettore impulso energia...attuale e come generato
        gvet Vers;          // direzione di volo della particelle nel punto di impatto


};

/**

   <b>device</b>  è la classe base di tutte la classi che descrivono i rivelatori. Per
   convenzione le classi dei rivelatori iniziano con "dev".
   In device appaiono:
<ul>
   <li> tutti i parametri geometrici e fisici  per definire il rivelatore di riferimento
   <li> i parametri che descrivono lo stato del rivelatore quando colpito da una particella.
   <li> le coordinate generate prima e dopo la simulazione degli errori (vedi sotto)
   <li> le strutture <b>Bufdev Hits[30]</b>  contengono le infornazioni delle ultime 30  particelle rivelate.
   <li> le strutture <b>Bufdev G_Hits[100]</b> costituiscono un  buffer circolare  dimensionato max 100 per
        ogni device ed è riempito copiando gli hits  da Hits. Gli hits  sono memorizzati
        in successione temporale. In realtà questo buffer circolare è attivato
        solo se si lavora in modo pileup (tipico per lo studio delle accidentali).
        Vedi anche la flag  di attivazione del pileup (-p) al momento della esecuzione di Flyo.
   <li> le strutture <b>Bufdev M_Hits[30]</b> costuituiscono un secondo buffer circolare in cui vengono memorizzati
        i dati elaborati dalle procedure di DataSmear e SimulaDev.
        Il buffer è completamento attivo se è on la flag di pileup, altrimenti si usa solo il primo elemento.
  </ul>

 @author Giuseppe Pierazzini

  */

// =================   D E V I C E  ==== base class =======
// =================   D E V I C E  ==== base class =======
// =================   D E V I C E  ==== base class =======

class Device
{
        friend class Particella;
        friend class TriggerBox;
        friend class  Radiografia;

protected:
        int devtype;
        std::string devclass;
        int idev;    // dev id  =  numero d'ordine  di creazione
        char nome[9];//  nome : identifica anche il blocco hbook nella ntupla
        char nickname[9]; //  nickname: defined for root device name. Usually shorter..but non important..

        const char *fun;           // device funzione
        int  nvede,assorb[30];       // -1,0,1,2,3 detection levels
        const char *vede[30];              // lista particelle visibili
        int registra;    //variabile globale di registrazione hits. default =1 (si)
        int extrawork;    // abilita altre definizioni... extra cluster(LKRY) o emissione di fotini cerenko(RICH)
        gvet Centro,Centrin,Cface;        // centro dev e centro del buco rispetto al c_dev
        gvet Nx, Ny,Nz;

// versori di orientamento
// Questi corrispondono ( a meno del segno) ai versori dei
// piani che definiscono il volume del dev.
// I versori dei piani vanno pensati orientati verso l'esterno.
// per un dev parallelepipedo di dimensioni: a,b,c

// piani           1    2    3    4    5    6
// perpendic. a    x    x    y    y    z    z
// in             -a    a   -b    b   -c    c
// versore       -Nx   Nx  -Ny   Ny  -Nz   Nz
//------------------------------------------------------

        gvet Lout,Lin,Sfera,Fortax;   // vettori semidimensioni device
        gvet X_dev,V_dev;
        double OpenSector,Devsize;

// Lout definisce le dimensioni esterne del dev riferite al centro device
// Lin quelle interne del buco
// Lout e Lin  per geom rett. sono definiti come (lx/2,ly/2,lz/2)
// corrispondenti alle coordinate dello spigolo in alto.
// per geom cilindriche con asse z lungo l'asse del cilindro
// come: (R,0.,lz/2)
// corrispondenti alla cordinata posta in alto del cilindro
// sullo  spigolo positivo.

// ---------------- Materiali di costruzione  ---------------------
// il dev e' costruito con un materiale indicato come materiale interno "mattin"
// il buco contine un'altro materiale di solito gas.. definito in "mattbuc"
// si indicano  la Z dei materiali "Zint,Zbuc"
// il numero atomico "Aint,Abuc"
// lunghezza di interazione interno e nel buco "liint,lbuc",
// la densita' "Roint,Robuc"
// la perdita di energia al minimo "Dedxint,Dedxbuc"
// lunghezze di radiazione: lradi,lradb
// variabili di comodo: lrad,Ro,Dedx.
//
//   char mattint[10] ,mattbuc[10];
        std::string mattint,mattbuc;
        int Zint,Zbuc,Ray_abs;
        double Aint,Abuc,liint,libuc;
        double Roint,Robuc,Dedxint,Dedxbuc;
        double lradi,lradb;
        double lrad,nrad,Ro,Dedx,lint;

// tutti dati sono definiti sia nelle schede di descrizione dei dev
// o meglio derivati dal data_base dei materiali definiti prima
// dei device nei file di descrizione .epc o  app.
//------------------------------------------------------------------------------
        double reckik;
        gvet Field;
        int campo;
        double Bref,Dbref;    //Beref Campo alla distanza dal centro Dbref
        gvet Field_sav;
        double time_now;
        double t_dead_mem;       // tempo morto del device

//------------------------------------------

//-------------D U M M I E S -----------------
//dummy variable to evaluate the multiple scattering...
//   gvet DX,Rx,Ry,Rz,S,Dplane;

//dummy variables
        double cost, sint, sinf, cosf, txplane,typlane;
        double tet0,xplane,yplane,z1,z2;
//    double  px,py,pz,p,   pt,     tratok;
        double Last_distq;
        double camm;
//    int posok,pos;
        int strutture_interne;
        int pos_corrente;
//-----------------------------------------
public:

        int   maxBrems,maxPair;
        int   Brem_done,Pair_done;
        int gnhit,ghcir,ghit_last;                  // numero di hits generati nel dev.
        int nhit,mhit;             // numero di hits nel dev.
        int Start_Hit;            // posizione  del primo hit nel buffer circolare
        double eDev;

        /* *******************************************************************
                  W O R K I N G  buffers


        ********************************************************************/
        /**
        I parametri memorizzati in Hits sono relativi agli hits generati nel rivelatore.
        <br>
        Type(0) variables: Hits  Dev hit at low level.... just generated..
        */
        Bufdev *Hits;

        /**
         G_Hits è un buffer circolare,dimensionato max 100 per ogni device, in cui si memorizzano, se pileup è acceso,  i dati che sono via, via generati ed hanno colpito il device. In pratica si copiano i dati da Hits in successione temporale.
         Se si verifica un trigger definito, per esempio in Trigger_Box, secondo un criterio di minima si accede a  ciascun buffer circolare e si prelevano, per ogni dev, gli hits da G_Hits  in una  finestra temporale definita  attorno al trigger_time e si memorizzano in  M_Hits[30].
        <br>
        Type(1) variables: G_Hits  Circular buffer for the hits if pipeline is on.
        */
        int working_buffer, circular_buffer;
        Bufdev *G_Hits;

        /**
        M_Hits contiene gli hits dei dev copiati nella opportuna finestra di trigger.
        Questi dati si elaborano con le procedure DataSmear() e  Simuladev() e quindi si
        trasferiscono sul file per analisi root Vedi outnt.cpp.
        <br>
        Type(2) variables: M_Hits  Dev hit info at Measured level (with errors)
        */
        Bufdev *M_Hits;



//*******************************************************************

//*******************************************************************
        /* R O O T    tipical default buffer
         Attenzione: le dimensioni possono dipendere dal dev
         Vedi "void  Device::Make_measur_buffer ( int dim )"
        */

//    int measur_buffer_dim == working_buffer;
        int hit_recorded;
//    int *index, *mid,*mdead;
        int *index,*ido, *mid,*structinfo,*mdead;
        unsigned long long *devstory;
        float emDev,dxynorma;
        float *mxd, *myd, *mzd,*mdd, *me, *mxl, *myl, *mzl, *gpx, *gpy, *gpz, *ge;
        float *pxd, *pyd, *pzd;
        float *pxl, *pyl, *pzl,*pel;
        double *dtemp;


//*******************************************************************


        Particella *pr_dev;   // particella attualmente nel dev
        int nowpos,prevpos,accedi,escludi,rivela,Idsee;
        int wrout;                   // se diverso da zero scrivo su out tape
        int Tot_seen, Part_hit[40];
        Device *up, *next;
        Device ();
        virtual ~Device();
        void Leggi_schede_dev();
        char *  Get_Nome() {
                return nome;
        };
        char *  Get_NickName() {
                return nickname;
        };
        int     Get_Idev() {
                return idev;
        }
        const char *  Get_Vede ( int i ) {
                return vede[i];
        };
        void  Make_working_buffer ( int );
        void  Make_measur_buffer ( int );


        int     Fill_Default();
        virtual void Reset_RootData() {};
        int     Get_DatiOut();
        double  Get_Zsize() {
                return Lout.z;
        };
        void    Change_sign_Field() {
                Field=-Field;
        };
        void    Set_Field ( double B ) {
                Field[1]=B;
        };
        void  Set_Rivela ( Particella * );
        void  Reset_Dev ();
        void  Reset_Pos ();
        void  Dev_summary();
        void  Spia_Camm ( std::string,Particella *,double,double, std::string );
        void Set_X_dev ( gvet gv ) {
                X_dev= gv;
        }
        gvet& Get_X_dev() {
                return X_dev;
        };
        gvet& Get_Centro() {
                return Centro;
        };
        gvet& Get_Cface() {
                return Cface;
        };


        gvet& Get_Lout ( ) {
                return Lout;
        };
        gvet& Get_Lin ( ) {
                return Lin;
        };

        double Get_Kik() {
                return reckik;
        };
        int Get_Extrawork() {
                return extrawork;
        }
        void   Get_Materiali();
        void GetWindowHits ( double );
        virtual int SimulaDev() {
                return 0;
        };
        virtual gvet& Lab2Dev ( gvet& );
        virtual gvet& Lab2cDev ( gvet& );
        virtual gvet& Dev2Lab ( gvet& );
        virtual gvet& Devc2Lab ( gvet& );

        virtual qvet& Lab2Dev ( qvet& );
        virtual qvet& Dev2Lab ( qvet& );



        virtual int Posizione() {
                return 0;
        };
        virtual double CamEster() {
                return 0.0;
        };
        virtual double CamInter() {
                return 0.0;
        };
        virtual double CamBuco() {
                return 0.0;
        };
        double Get_Cammino();


// procedure di ausilio per il tracing
// in funzione di ( gvet X_dev, gvet V_dev,gvet Lout);
        double  Cam_su_Rt ( gvet,gvet,gvet );  //verso un dev a forma esterna di parallepipedo
        double  Cam_da_Rt ( gvet,gvet,gvet );  //esce da un dev a forma interna di parallepipedo
        double  Cam_su_Cln ( gvet,gvet,gvet ); //verso un dev a forma esterna cilindrica
        double  Cam_da_Cln ( gvet,gvet,gvet ); //esce da un dev a forma interna cilindrica
        double  Cam_su_Sfera ( gvet,gvet,double ); //verso un dev a forma esterna sferica
        double  Cam_da_Sfera ( gvet,gvet,double ); //esce da un dev a forma interna sferoca
        int     Traccia_in_magn ( Particella *pr );
        int Traccia_in_materia ( Particella *pr );

        virtual void  ScrvColpi ( Particella * );
        virtual int Traccia ( Particella * );
        virtual gvet& Get_Field ( gvet& x ) {
                return Field;
        };
        virtual gvet& Get_Field() {
                return Field;
        };
        virtual int Mscatter ( Particella *, double ) ;
        virtual void DataSmear() {
                /*attenzione.. si chiama dopo Simuladev*/
        };
        virtual double Msperdita ( double );
        virtual double Get_Res() {
                return 0.;
        };
        virtual int Get_Structinfo() {
                return nowpos;
        };
        void  PrgeomId();
        virtual void  Prgeom();
        void     Get_Pointers();
        Device * Get_Pointer ( const char * );

        Device * Get_camm ( Particella * );

};
#if defined(MAIN)

Device  *Rivelatore[200];
int MemoryBuffer=40;

//double path_corrente=0;

Device *collo=0,*xch02=0,*spia0=0,*spia3,*vtappo=0;

Device *Apparato=0,*last_device=0,*Devloop=0,*Devloopend=0;
Device *dectubo=0;
Device *mgb1=0,*mgb2=0,*mgb3=0,*mgb4=0,*mgb5=0;
Device *iron=0,*monit=0;
Device *fluka=0;
Device *mamud=0;
/**********************
    Scelta della classe che descrive il rivelatore

Control word in epc   shape              class     device Type
(first 5 characters)
  no             no                      Device          0
rettangolo     parallelepipedo           DevRt           1
rettrett       parall. con buco rett.    DevRtRt         2
cilindro       cilindro                  DevCln          3
rettcili       parall. con buco cili.    DevRtCl         4
cilirett       cilin. con buco rett.     DevClRt         5
corona         corona cilindrica         DevCrn          6
ottagono       ottagonale                DevOt           7
ottagono       ott.con buco ottag.       DevOtOt         8
ottagono       ott. con buco cil.        DevOtCl         9
ciliott        cilind. con buco ott.     DevClOt        10
Lveto
magnete        magnete                   DevMagn         20
quadrupolo     quadrupolo magnetico      DevQuad         21
magnete        Mag.con buco per filtro   DevFlt          22
Gas            Dec.Reg. shape-beam-gas   DevGas          23

chamber        Mwire Chamber             DevMwc          50
lkr48          calorimetro(Na48)         DevLkr        51
chod        hodoscopio (NA62)         DevChod        52
hac48          Adron.calor(NA48)         DevHac          53
ytax           acromat collimator        DevYTax         54
xtax           acromat collimator        DevXTax         55
bluetube       Blu tube                  DevBlue         57

software       no shape                  DevTrk          100
basedev        Rivelatore di base        DevBase         500
**********************/
int  TypDevCedar= 61   ;
int  TypDevice  =  0   ;
int  TypDevRt   =  1   ;
int  TypDevRtRt =  2   ;
int  TypDevCln  =  3   ;
int  TypDevRtCl =  4   ;
int  TypDevClRt =  5   ;
int  TypDevCrn  =  6   ;
int  TypDevOt   =  7   ;
int  TypDevOtOt =  8   ;
int  TypDevOtCl =  9   ;
int  TypDevClOt =  10  ;
int  TypDevMagn =  20  ;
int  TypDevQuad =  21  ;
int  TypDevFlt  =  22  ;
int  TypDevGas  =  23  ;
int  TypDevLav=  24  ;
int  TypDevSveto=  25  ;
int  TypDevRich =  26  ;
int  TypDevSector= 31  ;
int  TypDevKbs  =  48  ;
int  TypDevMwc  =  50  ;
int  TypDevLkr  =  51  ;
int  TypDevChod =  52  ;
int  TypDevHac  =  53  ;
int  TypDevYTax =  54  ;
int  TypDevXTax =  55  ;
int  TypDevMuv =  56  ;
int  TypDevBlue =  57  ;
int  TypDevGig  =  58  ;
int  TypDevChanti=  60  ;
int  TypDevStraw=  59  ;
int  TypDevIrc=  62  ;
int  TypDevSac  =  61 ;
int  TypDevSolenoide =  64 ;
int  TypDevToro =  65 ;
int  TypDevSfera =  68 ;
int  TypDevMare =  70 ;

int  TypDevTrk  =  100 ;


#else
extern Device *Rivelatore[200];
extern int MemoryBuffer;
//extern double path_corrente;
extern Device *collo,*xch02,*spia0,*spia3,*vtappo;
extern Device *Apparato,*last_device,*Devloop,*Devloopend;
extern Device *dectubo;
extern Device *mgb1,*mgb2,*mgb3,*mgb4,*mgb5;
extern Device *iron,*monit;

extern Device *fluka;

//extern Device *Rich;



extern Device *mamud;

extern int  TypDevCedar  ;
extern int  TypDevice  ;
extern int  TypDevice  ;
extern int  TypDevRt   ;
extern int  TypDevRtRt ;
extern int  TypDevCln  ;
extern int  TypDevRtCl ;
extern int  TypDevClRt ;
extern int  TypDevCrn  ;
extern int  TypDevOt   ;
extern int  TypDevOtOt ;
extern int  TypDevOtCl ;
extern int  TypDevClOt ;
extern int  TypDevMagn ;
extern int  TypDevQuad ;
extern int  TypDevFlt  ;
extern int  TypDevGas  ;
extern int  TypDevLav;
extern int  TypDevSveto;
extern int  TypDevRich;
extern int  TypDevSector ;
extern int  TypDevKbs  ;
extern int  TypDevMwc  ;
extern int  TypDevLkr;
extern int  TypDevChod ;
extern int  TypDevHac  ;
extern int  TypDevYTax ;
extern int  TypDevXTax ;
extern int  TypDevMuv ;
extern int  TypDevBlue ;
extern int  TypDevGig  ;
extern int  TypDevChanti;
extern int  TypDevStraw;
extern int  TypDevIrc;
extern int  TypDevSolenoide;
extern int  TypDevToro;
extern int  TypDevSac;
extern int  TypDevTrk  ;
extern int  TypDevSfera ;
extern int  TypDevMare ;
#endif
#endif










