/*******   main utility routines *********/

#define   MAINFLYO
#include "flyoh.h"
#include  "flyline.h"
#include  "flytrk.h"


/** \file  mainuti.cpp
 * Contiene:
 * <ul>
 * <li>void flyoinit()  Procedura di inizializzazione di Flyo
 * <li>int flyoevento() Genera un evento
 * <li>void flyosimuladev()  Chiama int SimulaDev() e void DataSmear() per  simulare in dettaglio se richiesto il device.
 * <li>void flyoend()  Termina il processo di simulazione
 * </ul>
<br>
 *@author Giuseppe Pierazzini
 *
 *
*/

using namespace std;

Particella *Pr_avo,*pr,*pb,*pc;
//=======================================


/**
Inizializza tutta la procedura di simulazione. Definisce  le file di lettura e scrittura  (vedi File_Manager()) ed i parametri di controllo della simulazione (vedi carte.cpp e Leggi_input_File(cardin)).

<br> Per il momento è specializzato per gli esperimenti NA48 e NA62, ma può facilmente essere generalizzato ad altre geometrie.
*/

void flyoinit()
{

        //-------------------------------------------------------------
        cout<<"\n <><><><><><><><><><><><><><><><><><><><><><><><><>"<<endl;
        clockstart=clocklast = time ( &tck );
        cout<<"\n  Flyo starts  running "<< ctime ( &clockstart );
        Gout<<"\n  Flyo starts  running "<< ctime ( &clockstart );
        Gout<<"\n <><><><><><><><><><><><><><><><><><><><><><><><><>"<<endl;
        //------------------------------------------------------------------



        // Inizializzazione....
        TBurst=new Tempo();

// crea il data base delle particelle .
//Se manacano particelle aggiungere in:   Crea_part_db();
        Crea_part_db();
// crea il data base dei materiali
//Se manacano materiali aggiungere in:   Crea_materiali_db();
        Crea_materiali_db();

// si legge le file .epc e .app
        File_Manager();
        Sommario_Input_Data();


// first call... initialization

        if ( Reaction==0 ) {
                Gout<<"\n Reactions not defined.. exit!!";
                abort();
        }

        Reaction->MaxSpettro();
        Reaction->Norma_Rate(); // rate normalization
        Reaction->Simple_List();
        Gout<<"\n\n"<<endl ;

        // get dev pointers. Attention...adjust with dev name changes...
        Apparato->Get_Pointers();

        if ( Trigger_low_on==1 ) Trigger=new TriggerBox();


        if ( Wtrk==1 ) {
                // do tracks or trigger... at least four chamber...
                // for the kpivv experiment we need six chamber. of four planes each.....

                // NA48 experiment    4 chambers
                if ( Cam1!=0&&Cam2!=0&&Cam3!=0&&Cam4!=0&&Mag48!=0 ) {
                        // Na48/1, NA48/2   apparatus

                        // in principio si possono trovare piu' tracce ma
                        // ne ricostruiamo solo per 6 ....
                        Le_Tracce=new Flyline();
                        Tracc=new Flytrk[6];

                        Gout<<"\n Dev NA48  track  done \n";
                }
                // Kpvv   experimento... nuovo apparato per il calcolo della fascio dal gigatrack...
                if ( giga1!=0&&giga2!=0&&giga3 ) {
                        // NA62: Kpivv new apparatus with double spectrometer
//            Trkgiga=new TrackGiga();  // to test
//            Gout<<"\n Dev Trkgiga    done \n";
                }
        }
// here devstraw create...TriggerBox


        if ( FirstStraw!=0 ) {
// gmpKill

                DevStraw * poistraw=FirstStraw;

                ChambClus[0]=new ClusterStraw ( poistraw );
                poistraw=poistraw->nextstraw->nextstraw->nextstraw->nextstraw;
                ChambClus[1]=new ClusterStraw ( poistraw );
                poistraw=poistraw->nextstraw->nextstraw->nextstraw->nextstraw;
                ChambClus[2]=new ClusterStraw ( poistraw );
                poistraw=poistraw->nextstraw->nextstraw->nextstraw->nextstraw;
                ChambClus[3]=new ClusterStraw ( poistraw );
// generate the track straw pointer.
                if ( Wtrk==1 ) Trkstraw = new TrgmpStraw();
                Gout<<"\n Dev NA62  track (gmp)  done \n";


                // generate the track straw pointer.
                //        if( Wtrk==1 )Trkstraw = new TrackStraw();
                //        Gout<<"\n Dev NA62  track  done \n";

                //        ChambClus[0]=new ClusterStraw ( FirstStraw);//  attenzione per test gmp
        }

        Gout<<endl;


        // general random initialitiation:
        //     Correla =0 Inizializzazione fissa
        //     Correla =1 Inizializzazione dipendente dal run
        //     Correla =2 Inizializzazione dipendente dal run ed evento

        if ( Correla==0 );
        else Set_Run_Random();


        Mostra=evento_.Mxev/10;
        if ( Mostra< 10000 ) Mostra=10000;
        if ( Mostra>250000 ) Mostra=250000;


        clockstart = time ( &tck );
        Gout<<"\n\n Starting time: "<< tck<<" ===> "<<ctime ( &clockstart );
        cout<<"\n\n Starting time: "<< tck<<" ===> "<<ctime ( &clockstart ) <<endl;
}

//================== F l y o e v e n t  =============================
//============>>   M a i n    e v e n t  generator....     <<===========

/**
Genera le reazioni secondo le richieste dell'utente e  traccia tutte le particelle nell'apparato.
*/

int flyoevento()
{


        //   int colpo=0, visto = 0,debaggati=0;
        colpo=0;

//  se Correla=2 inizializzazione di random secondo l'evento e run
        if ( Correla==2 ) Set_Ev_Random();

        Debugon=1;
        if ( Eventi_Generati < evento_.Debg_first ) Debugon=0;
        if ( Eventi_Generati > evento_.Debg_last ) Debugon=0;

        if ( Eventi_Generati % Mostra == 0 ) {
                clockend = time ( &tck );
                double Elapsed=clockend-clockstart;
                double FromLast=clockend-clocklast;
                double pevsec=-1.,evsec =-1.;




                if ( Elapsed>0.0 ) evsec=Eventi_Generati/Elapsed;
                if ( FromLast>0.0 ) pevsec= ( Eventi_Generati-evento_.Evenlast ) /FromLast;

                std::ostringstream strs;
                strs << pevsec;
                std::string str = strs.str();
                if ( pevsec<0. ) str="inf";


                cout<<"\n  Eventi: gen. "<<setw ( 9 ) <<Eventi_Generati
                    <<" prod. "<<setw ( 8 ) <<Eventi_Prodotti
                    <<" scr. "<<setw ( 8 ) <<Eventi_Scritti
                    <<"  Ev_G/s "<<setw ( 8 ) << str.c_str()
                    <<"  Ev_G/s medio "<<setw ( 7 ) <<evsec
                    <<" elaps.  "<<Elapsed;


                cout<<flush;


                Gout<<"\n  Eventi: gen. "<<setw ( 9 ) <<Eventi_Generati
                    <<" prod. "<<setw ( 9 ) <<Eventi_Prodotti
                    <<" scr. "<<setw ( 9 ) <<Eventi_Scritti
                    <<"  Ev_G/s "<<setw ( 8 ) << str.c_str()
                    <<"  Ev_G/s medio "<<setw ( 8 ) <<evsec
                    <<"  elapsed  "<<Elapsed
                    <<" At "<<ctime ( &clockend );





                clocklast=clockend;
                evento_.Evenlast=Eventi_Generati;

        }



//==============    Select a new reaction   ===================

        if ( ( SelRea = Reaction->Select_reation () ) == 0 ) {
                cout<<"\n No Reaction ... exit"<<endl;
                Gout<<"\n NO Reaction ... exit"<<endl;
                abort();
        }
        SelRea->AddGen();

        if ( Debugon==1 ) {
                debaggati++;
                Gout<<"\n\nRun "
                    <<setprecision ( 0 ) <<fixed
                    <<setw ( 6 ) <<evento_.Gen.Run<<"  E v e n t o "
                    <<setw ( 6 ) <<Eventi_Generati<<"  Dbg="
                    <<setw ( 6 ) <<debaggati<<" "
                    <<setw ( 25 ) <<SelRea->Get_Titolo() <<endl;
        }
        // =================== reset param...=======================



        Pr_avo = SelRea->avo;
//-------------------

// get the beam  attenzioine qui si loop sull'esterno per scegluiere un'altra reazione
// vedre di fare meglio

        if ( SelRea->Get_Beam () <0 )  return ( -2 );


//--------------------------------------------

        visto=0;

//   trasport the first particle della reazione...
//     Pr_avo->Stampa_questa_particella();


        colpo = Pr_avo->Trasport();
        visto+= colpo;
        
        
// if the beam does not decay or has no gas intaractions...get out

        if ( Pr_avo->dw!=0 || Pr_avo->cls!=0 );
        else if ( SelRea->rea_type==-1 ) {
                visto=1;
                return ( 1 );
        }


        if ( colpo<0 ) {
                return ( colpo );
        }

//================================ l o o p  for the other particles =========//
//  decay loop ....dei decadimenti .
// main loop: iterative procedura to follow the reaction tree...

  //     cout<<"\n\n Flyevent Ev  "<<Eventi_Generati<<" precolpo  "<<colpo<<" visto "<<visto;

        if ( Pr_avo->dw!=0 || Pr_avo->cls!=0 ) { //non un urto: decay oppure gas
                // la particella padre decade comunque....
                // se è un proiettile che ha interagito non decade vedi PartBullet::Get_Bersaglio()
                Pr_avo->Set_Pbar();
                Pr_avo->Main_Loop();  //vedi qui sotto il sorgente

        }

  //      cout<<"\n Flyevent Ev  "<<Eventi_Generati<<" postcolpo "<<colpo<<" visto "<<visto;
        if(colpo<0) return colpo;



        if ( Pr_avo->Get_Type() ==Bullet ) {
                // la particella padre è il proiettile
                // il bersaglio è alla sua destra.....
                // ucms è il sistema virtuale (proiettile -bersaglio)
                // i cui parametri sono definiti in PartBullet::Get_Bersaglio()
//     Gout<<"\n "<<Eventi_Generati<<" è una Bullet....";
                Particella * ucms=Pr_avo->rg->dw;
                ucms->Set_Pbar();
                ucms->Main_Loop();
        }



// qui solo per debug
//  if(Eventi_Generati==151) SelRea->Simple_List();

        if ( Debugon==1 ) {
                Pr_avo->Stampa_particle ();
                Gout<<endl;
        }


//============================  main l o o p   end  ===================//

        //------------update special counters ---
        SelRea->Totpair+=   SelRea->Pair_done;
        SelRea->Totbrem+=  SelRea->Brem_done;
        SelRea->Totdltz+=   SelRea->Dalitz_done;
        evento_.Gen.Ipair=SelRea->Pair_done;
        evento_.Gen.Ibrem=SelRea->Brem_done;
        evento_.Gen.Idlz =SelRea->Dalitz_done;

        if ( Debugon==1 ) SelRea->avo->Debug_out ();

//  cout<<"\n Run-spia "<<evento_.Gen.Run  <<"  Ev. "<<setw ( 8 ) <<Eventi_Generati<<endl;

        return ( visto );
}





/**
Introduce gli errori sperimentali nei dati  geometrici e cinematici relativi a ciascun rivelatore (vedi virtual void DataSmear())
e ne simula in dettaglio il comportamento (procedura virtual int SimulaDev()) se richiesto.
*/
void flyosimuladev()
{
        Device *pv;
        pv=Apparato;
        while ( pv!=0 ) {
                if ( pv->escludi==0 ) {
                        pv->Reset_RootData();
// si simula in dettaglio il dev e poi si introducono gli errori
                        pv->SimulaDev();
                        if ( Errori==1 )  pv->DataSmear();
                }
                pv=pv->next;
        }
//     cout<<"\n cazz4";
//========================================================
        // ricostruisco le tracce per NA48.....

        if ( Le_Tracce!=0 ) {
                Le_Tracce->Reset();
                Le_Tracce->Get_Hits_from_Dev();
                Le_Tracce->Find_Tracce();  // creo le tracce:  max six ma si puo' aumentare
                if ( Debugon==1 ) {
                        if ( Tracc!=0 ) Tracc[0].Spia_Trk();
                }
        }

//========================================================

//  straw chamber track                                 GmpKill

        if ( Trkstraw !=0 )   {

                ChambClus[0]->Get_Cluster();
                ChambClus[1]->Get_Cluster();
                ChambClus[2]->Get_Cluster();
                ChambClus[3]->Get_Cluster();


                Trkstraw->Make_Trk();


        }






        // it is called at the  tracing end
        //------------update special counters ---

        SelRea->Tailsrt+=   SelRea->Tailsrt_done;
        SelRea->Tailong+=   SelRea->Tailong_done;
        evento_.Gen.Itsrt=SelRea->Tailsrt_done;
        evento_.Gen.Itlg= SelRea->Tailong_done;




}


//=================================================
/**
Termina  la simulazione,
Scrive il sommario sulla file di linsting di uscita.<br>
Da i tempi di elaborazione globali.
*/


void flyoend()
{
        Gout<<"\n------------ Device hit summary   ----------";
        Apparato->Dev_summary();
        Gout<<"\n------------         E n d        ----------";
        Gout<<"\n---                                     ----\n";
        if ( Le_Tracce!=0 )
                Gout<<"\n Charged tracks info.. "<<Tracc[0].tottrk<<" "<<Tracc[1].tottrk<<" "<<Tracc[2].tottrk<<endl;
        Reaction->Fine_run ();

        clockend = time ( &tck );
        double dft=clockend-clockstart;
        double evsec =Eventi_Generati/dft;

        Gout<<"\n T start "<<clockstart<<"  e stop "<<clockend<<"  e diff  "<<dft<<" Sec."<<endl;
        Gout<<"\n Epsi Event/sec = "<<setw ( 10 ) <<evsec<<endl;

        Gout<<"\n Epsi main loop start at "<<setw ( 25 ) <<ctime ( &clockstart ) <<endl;
        Gout<<"\n Epsi main loop end   at "<<setw ( 25 ) <<ctime ( &clockend ) <<endl;


}

//========================================================================

int Particella::Main_Loop()
{
        Particella  *pb=0, *pc=0;

// decay and particle transport....
// iterative procedure to follow the reaction tree...



        /*      Gout<<"\n Vold1_Ev "<<Eventi_Generati<<" "<<setw(6)<<ido
                <<" "<<setw(6)<<id
                <<" "<<setw(6) <<nome<<flush;
        */

//--------- decay
        if ( colpo<0 ) return 0; // close any tracing if "colpo" has been set to negative, ( as by veto system?)




        if ( dw != 0 ) {
                pb = dw;
                if ( pb->rg!=0 ) pc = pb->rg;
                else if ( pb->lf!=0 ) pc = pb->lf;
                if ( pc==0 ) {
                        Gout<<"\n Voldec Chaining error!! Ev "<<evento_.Gen.Event;
                        exit ( 0 );
                }




                if ( Voldec ( pb, pc ) >0 ) { //trasport only if return > 0! Check the program wy!


//      trasport  per la prima particella
//            pb->Stampa_questa_particella();
                        if ( ( colpo = pb->Trasport() ) <0 ) return 0;
                        visto+= colpo;

//      seconda particella
//            pc->Stampa_questa_particella();
                        if ( ( colpo = pc->Trasport () ) <0 ) return 0;
                        visto+= colpo;
                        
                        
//                         Gout<<"\n\n Ev " <<Eventi_Generati<<"  Dopo trasport visto "<<visto<< flush;
//                         pb->Stampa_questa_particella();
//                         pc->Stampa_questa_particella();
//                    
// 
//                         Gout<<" =dw=> "<<setw ( 6 ) <<pb->id<<" "<<setw ( 6 ) <<pb->nome;
//                         if ( pb->dw!=0 ) Gout << " =dw=> "<<pb->dw->nome;
//                         else Gout << "    -- ";
//                         Gout<<" =rg=> "<<setw ( 6 ) <<pc->id<<" "<<setw ( 6 ) <<pc->nome;
//                         if ( pc->dw!=0 ) Gout << " =dw=> "<<pc->dw->nome;
//                         else Gout << "    -- ";


                        if ( pb->dw!=0||pb->cls!=0 ) pb->Main_Loop();  //itera....
                        if ( pc->dw!=0||pc->cls!=0 ) pc->Main_Loop();  //itera....
                }
        }


//---------------only tracing (particles from cluster)
        if ( cls != 0 ) {
                if ( ( colpo = cls->Trasport () ) <0 )  return 0; //just in the case of veto
                visto+= colpo;
                /*
                   Gout<<" =cls=>  "<<setw(6)<<cls->id<<" "<<setw(6)<<cls->nome;
                   if(cls->dw!=0) Gout << " =dw=> "<<cls->dw->nome; else Gout << "    -- ";*/

                if ( cls->dw!=0||cls->cls!=0 ) cls->Main_Loop();
        }

        return 0;

}



