/***************************************************************************
                             -------------------
    copyright            : (C) 2012 by Giuseppe M Pierazzini
    email                : pierazzini@unipi.it
 ***************************************************************************/

#include "flyoh.h"
#include "flyopart.h"
#include "evento.h"
#include "reazione.h"
#include "matrice.h"
#include "pip0ee.h"

using namespace GmpConst;


Pip0ee::Pip0ee ( Particella *pr )
{
        nome=string ( "pip0ee" );
        pdec= ( Partres4* ) pr;
        norma_integral=-1.0;
        Gout<<"\n Generata la matrice pip0ee per il decadimento K->pip p0 e+ e- ";
        pdec->Stampa_questa_particella();
}

double Pip0ee::M_Valore()
{
        // decdimento del K+=> pip pi0   e+ e-  (questa è la sequenza con cui si deve scrivere la sheda...)
//static double Mmax=525388493.45;
//static double Mmax=0.0;

//*****************************
// NOTE we use the reduced matrix element...without factor ( pole?)    double K1 =2./ ( Q2*Q2 );
// which is evalueted before with integration method...
//*******************************



//=====================
// si calcolano i parametri del decadimento in funzione dei parametri scelti dallo spazio delle fasi
        if ( pdec->Define_CMvalues ( 1 ) ==0 ) {
                return 0.0;    //Calcolo dei valori cinematici di decadimento...
        }




//***********D'ambrosio paper variables
//    DATA MKCH /0.493677/
        qvet QG = pdec->P3cm - pdec->P4cm; //Q in paper
        qvet QP = pdec->P3cm + pdec->P4cm; //q in paper
        qvet P1 = pdec->P1cm;
        qvet P2 = pdec->P2cm;


//        IB MATRIX ELEMENT FROM FORMULA 18 G. D'Ambrosio with F1 F2 coming from 21
        // DATA XELMAS /0.000511/

        double Q2 = pdec->mresq34;



        //Do not use complex numbers just multiply the i in the final formula
        //Removed the terms in EQ in the F1 and F2 and in K1 removed Mp+p0
        double FIB1,FIB2;
        double T11,T12,T22;

        double a11=QP*P1;
        double a21=QG*P1;
        double a12=QP*P2;
        double a22=QG*P2;
        double e34mk=pdec->eres34*pdec->mpadre;

        // Q4*PK =  (pep+pem)*Pk = eres34*mk ???
        // P1*P2 = (mqres12-mpiq-mp0q)/2.;


        FIB1 =  4.*a12 / ( ( 2.*a11+Q2 ) * ( 2.*e34mk-Q2 ) );
        FIB2 = -2.             / ( 2*e34mk-Q2 );

        T11 = a11*a11 -a21*a21 - Q2*pdec->mchd1q;
        T22 = a12*a12 -a22*a22 - Q2*pdec->mchd2q;
        T12 = a11*a12 -a21*a22 - Q2* ( P1*P2 );

//    double  M=K1* ( fabs ( FIB1*FIB1 ) *T11+ fabs ( FIB2*FIB2 ) *T22 +2.*FIB1*FIB2*T12 );
//   double  FF= FIB1*FIB1 *T11+  FIB2*FIB2*T22 +2.*FIB1*FIB2*T12 ;

//***************************************************
// NOTA: in the real matrix  the factor
//   double K1 =2./ ( Q2*Q2 ); is removed
//  The complte element is:
//   double  M=K1*( FIB1*FIB1 *T11+  FIB2*FIB2*T22 +2.*FIB1*FIB2*T12 );
//  here we evaluate the M without K1 factor!!   done outside:
//**********************************************


        double  M= ( FIB1*FIB1 *T11+  FIB2*FIB2*T22 +2.*FIB1*FIB2*T12 );


        if ( M<0.0 ) {
                M=0.0;    // check
        }
        return M;


}

void Pip0ee::Get_FaseMass()
{
        // qui estrae la massa quadra ee con il metodo integrale
        // Si valuta poi l'elemento di matrice ridotto...

        if ( norma_integral<=0.0 ) {

                norma_integral= 1.0/pdec->mminq34 - 1./pdec->mmaxq34;

        }
        pdec->mres34=sqrt ( pdec->mresq34=1.0/ ( 1.0/pdec->mminq34-norma_integral*Pran() ) );
//========================================================================
//
//   standard: uniform between  min e max
        pdec->mres12    =sqrt ( ( pdec-> mresq12 =pdec->mminq12+Pran() *pdec->dmq12 ) );

}


double Pip0ee::M_ValoreMax()
{
        // prima di procedere verifico se la lista particella ha la corretta sequenza....
        static int check=1;
        if ( check==1 ) {
                Check_Lista();
                check=-1;

        }

// Gout<<"\n Partres4 ev !"<<Eventi_Generati<<"  Valormax ";
// si itera solo sulle masse ... fare meglio
        return M_Valore();

}

void  Pip0ee::Check_Lista()
{
// check lista delle particelle
// se non va bene si termina il programma.
// check the correct order  of particles ( pip pi0 e+ e-)

        if ( pdec->p1->Get_Idm() !=8 ||
                        pdec->p2->Get_Idm() !=9 ||
                        pdec->p3->Get_Idm() !=4 ||
                        pdec->p4->Get_Idm() !=3 ) {
                Gout<<"\n ERROR .... Incorrect particles order... has to be (pip pi0 elep elec)  instead of: "
                    <<pdec->p1->Get_Nome() <<" "<< pdec->p2->Get_Nome() <<" "<<pdec->p3->Get_Nome() <<" "<<pdec->p4->Get_Nome()
                    << "  EXIT "<<endl;
                cout<<"\n ERROR .... Incorrect particles order... has to be (pip pi0 elep elec)  instead of: "
                    <<pdec->p1->Get_Nome() <<" "<<pdec->p2->Get_Nome() <<" "<<pdec->p3->Get_Nome() <<" "<<pdec->p4->Get_Nome()
                    << "  EXIT "<<endl;
                exit ( 1 );

        }



}
