/***************************************************************************
                          partfase.cpp  -  description
                             -------------------
    begin                : Mon Dec 25 2000
    copyright            : (C) 2000 by Giuseppe M Pierazzini
    email                : pierazzini@unipi.it
 ***************************************************************************/

#define ESTERNO  1
#define INTERNO  0
#define INBUCO   -1

#define EPSR     1.0E-200
#include "flyoh.h"
#include "fasemax.h"
using namespace GmpConst;

int dbgpartfase=0;

//using namespace std;


//-------------------------------------------------

//--------------------------------------------------------------
//             F s p a c e
//--------------------------------------------------------------
//  Fspace    routine

double
Fspace ( double res, double rm1, double rm2 )
{
    double dm,sm;
    sm = ( rm1 + rm2 ) ;
    if ( sm==0.0 ) {
        return 1.0;
    }
    if ( sm>res ) {
        return 0.0;
    }
    if ( sm<0.0 ) {
        return 0.;
    }
    if ( res<=0.0 ) {
        return 0.0;
    }
    sm/=res;
    dm = ( rm1 - rm2 ) /res ;
    return  sqrt ( ( 1.0 - dm * dm ) * ( 1.0 - sm*sm ) );
}
//-----------------------------------------------------
//-----------------------------------------------------
PartFase::PartFase()
{
    type="Dec3";

    mpadre=0.;
    p1=p2=p3=p4=p5=0;

    if ( evento_.Gen.Event< 1 ) {
        Gout<<"\n Classe PartFase  : "<<nome<<" done!"<<flush;
    }

}

//--------------------------------------------------------------*/
//                  D e c P a r m                               */
//--------------------------------------------------------------*/
// Get the resonance mass   and decay angles                     Get_mass       */
//

void PartFase::
DecParm()
{
// procedura per il decadimento in tre particelle
// esempio  K ->p1+p2+p3
// K -> mres12 p3
//      mres12 ->   p1 p2
// solo spazio fasi..... o con matrice di decadimento se fornita...
//this->Stampa_questa_particella();

    if ( pr1==0 ) { // inizialization... only first time
        Get_PhaseParm();
    }
// -------------------------------
//Gout<<"\nGet_Parm ev "<<Eventi_Generati<<" begin";
// qui sempre...
// nota: normalmente in Get_PhaseSpace ci sono 1...6 iterazioni!
    Get_PhaseSpace(); //  Attenzione!  qui ora hai dipsonibili e1,e2,e3..... mres12,mres12q,cst
//   the matrix element is automatically into  Get_PhaseSpace if needed Mtx !=0..


//-----------------------------------------------------

// define the mass and the quadrimomentum (qvet) of the decay particle.(mres)...
    pr1->Set_Massa ( mres12 );
    pr1->P.Riposo ( mres12 );  //Definizione di P a riposo nel cm di mres

    Define_CMvalues ( 0 );//  Attenzione!  qui ora hai dipsonibili e1,e2,e3...
 //   Gout<<"\nGet_Parm end  energie: "<<e1<<" "<<e2<<" "<<e3<<endl;
}

// ---------------------------------

void  PartFase::Get_PhaseSpace()
{
  
// attenzione nel caso di un urto con produzione di tre corpi, per esempio
// cms -> Kl res2 ; con res2 ->pip pip
// la massa del centro di massa può variare (anche se di poco.. dipende dallo spettro del beam)
// quindi il limte superiore della risonanza   res2 va ricalcolata!
// attenzione  wfase_max, calcolato inizialmente per un valore  della massa invariante di cms,viene via via aggiornato.....

    if ( dw->Get_Nome() ==Cms ) { //C'è stato un urto!   ???RIfare
        mpadre=up->Get_Massa();
        mmax12=mpadre-mfrate;
        mmaxq12 =mmax12*mmax12;
        dmq12 =mmaxq12-mminq12;
    }
//---
    iterfs=0;


    while ( iterfs++<5000 ) {
        // resonance mass selection...
        
         
        if ( Mtx!=0 ) {
 // Calcola le masse delle risonanze (12,...) tenendo conto dell'elemento di matrice 
            Mtx->Get_FaseMass();  
        }
       else {    
           mres12=sqrt ( ( mresq12 =mminq12+Pran() *dmq12 ) );
        }

        if ( mres12+mfrate>mpadre ) {
            continue;
        }
        double w= Fspace ( mres12, mchd1, mchd2 ) * Fspace ( mpadre, mfrate, mres12 );

//**** segue l'elemento di matrice:
// se è definito  sempre "1.0" (default) o Mtx ==0  si genera lo spazio delle fasi puro....
//     Gout<<"\nGet_PhaseSpace ev "<<Eventi_Generati<<"  iter "<< iterfs;

        if ( w<=0.0 ) {
            continue;
        }
        if ( Mtx!=0 ) {
            w*=Mtx->M_Valore();
        }
//        Gout<<"  e .. "<<e1<<" "<<e2<<" "<<e3;
        //elemento di matrice, default = 1.
//******

        if ( w >wfase_max ) {
            Gout<<setprecision ( 5 ) <<"\nEv "<<setw ( 9 ) <<evento_.Gen.Event<<" iter="<<iterfs<<" W-Ph.Sp. off: prb="<<w
                <<" prbmax="<<wfase_max<<" "<<SelRea->Get_Titolo();
            wfase_max=w+0.000001;
            // aggiorna  wfase_max memorizzato nella lista
            int p_id[6]= {Get_Idm(),p1->Get_Idm(),p2->Get_Idm(),p3->Get_Idm(), 0,0};
            Fasemx->Set_Wmax ( p_id, wfase_max );
        }
        if ( Pran() <w/wfase_max ) {
            break;
        }
    }
    if ( iterfs>4990 ) {
        Gout<<"\n Ev "<<Eventi_Generati<<"  F.Sp. warning: Too many iters...="<<iterfs<<" Res2  "<<mres12;
    }
    //     cout<<"\n Ev "<<Eventi_Generati<< " iter "<<iterfs<<" Res2  "<<mres12;




}
//---------------------------------------------------

void PartFase::Get_PhaseParm()
{
    /*-------------------------------------------
     Three bodies decay  M -> p1 p2 p3    in phase space..

    -----
     the chain is M-> res12   p3
                      res12 ->p1 p2
    -----
    M è la particella padre! p1,p2,3 sono i fligli

     Phase space weight
     w = F(M,res12,p1)*F(res12,p2,p3)
    ------------
    --------------------------------------------------*/
    if ( pr1!=0 ) {
        return;    // procedura called only the first time....
    }


    pr1=dw;   //il padre è this: pr1 punta a mres12 (p1,p2)
    p3=pr1->rg;
    p1=pr1->dw;
    p2=pr1->dw->rg;

    Gout<<"\n in Get_PhaseParm: "<<nome<<" -->  "<<pr1->Get_Nome();
    Gout<<" "<<p3->Get_Nome();
    Gout<<" -> "<<p1->Get_Nome();
    Gout<<" "<<p2->Get_Nome();
    Gout<<" "<<p3->Get_Nome() <<flush;

// particle parameters
    mpadre=Get_Massa();
    mpadreq=mpadre*mpadre;

    mchd1=p1->Get_Massa();
    mchd1q=mchd1*mchd1;
    mchd2 =p2->Get_Massa();
    mchd2q=mchd2*mchd2;
    mchd3 =mfrate=p3->Get_Massa();
    mchd3q=mfrateq=mchd3*mchd3;

// massimi e minimi valori della risonanza  Res2
    mmin12     =mchd1+mchd2;
    mminq12 =mmin12*mmin12;
    mmax12=mpadre-mfrate;
    mmaxq12 =mmax12*mmax12;
    dmq12 =mmaxq12-mminq12;
//  Gout<<setprecision (4)<<" min12 "<<mmin12<< " mmax12 " <<mmax12<<endl;
// controlla se wfase_max è già stato calcolato
    int p_id[6]= {Get_Idm(),p1->Get_Idm(),p2->Get_Idm(),p3->Get_Idm(), 0,0};

    if ( Fasemx!=0 ) {
        double  wmx=Fasemx->Get_Wmax ( p_id );
        if ( wmx>0. ) {
            wfase_max=wmx;
//      Gout<<"\n Ev "<<Eventi_Generati<<" F.S. trovato "<<up->Get_Nome() <<" "<<nome<<"=> "<<p1->Get_Nome()
//      <<" "<<p2->Get_Nome() <<" "<<p3->Get_Nome();
            return;
        }
    }

    double pass2=dmq12/100000.;
//     double w1;
    // define the wfase_max for p.s.  normalizazion
    wfase_max=0.0;
    for ( mresq12=mminq12; mresq12<mmaxq12; mresq12+=pass2 ) {
        mres12=sqrt ( mresq12 );
        double w=Fspace ( mpadre,mres12,mfrate ) *Fspace ( mres12,mchd1,mchd2 );
        // *****
//  cout<<"\n m12 "<<setw(8)<<mres12<<" "<<setw(8) <<w;

        if ( w<=0. ) {
            continue;
        }
        if ( Mtx!=0 ) {
            w*=Mtx->M_ValoreMax();    // valore massimo dell'elemento compatibile con mre12
        }
        // *****
//    cout<<" "<<setw(8) <<w;


        if ( w>wfase_max ) {
            wfase_max=w;
        }

    }
    Gout<<"\n\n Inizializzo  phase space ";
    Gout<<setprecision ( 4 );
    Gout<<"\n PhaseParm: w_max="<<wfase_max<<" done for " <<Get_Nome() <<" ==> "<<p1->Get_Nome() <<" "<<p2->Get_Nome()
        <<"  "<<p3->Get_Nome() <<"  Masse range: "<< mmin12 <<" " <<mmax12;
    this->Stampa_questa_particella();
    Gout<<endl;

    new FaseMax ( p_id,wfase_max );

}
//==================================
int PartFase::Define_CMvalues ( int genera )
{

    // calcolo in anteprima  la direzione di decadimento dei figli.
//  nel cm res2  (qui isotropo)
//  si calcolano gli angoli se non sono già definiti!!! o genera è 1


    static qvet Stesso;
    // energie di res12  e impulso di res12  in K(padre)  cm

    if ( mresq12==0.0 ) {
        return 0;
    }
    e3=0.5* ( mpadreq+mfrateq-mresq12 ) /mpadre;
    eres12=0.5* ( mpadreq-mfrateq+mresq12 ) /mpadre;


    double pp12q= eres12*eres12-mresq12;
    if ( pp12q<0.0 ) {
        pp12q=0.0;    // attention check limits....
    }
    double pp12=sqrt ( pp12q );
    qvet Pres12 ( 0.,0.,pp12,mres12 ); // immaginato lungo z (arbitrario)

// decadimento di mres12
// attenzione se genera è 1 si ridefiniscono gli angoli; tipicamente nella loop del phasespace


    if ( genera==1||p1->cstar>1. ) {

        if(dbgpartfase>0) Gout<<"\n ev "<<Eventi_Generati<< "  genera "<< genera<< "  "<<p1->cstar;
        double p1cst = 1.0 -  2.0 * Pran();
        double p1fi = DuePiGreco * Pran();
        p1->cstar=p1cst;
        p1->fistar=p1fi;
        p2->cstar=-p1cst;
        p1fi+=PiGreco;
        if ( p1fi>DuePiGreco ) {
            p1fi-=DuePiGreco;
        }
        p2->fistar=p1fi;

    }

    // energie delle particelle e1 e e2 in mres12 cm
    double ecm1=0.5* ( mresq12+mchd1q-mchd2q ) /mres12;
//   double ecm2=mres12-ecm1;
    double pcm=sqrt ( ecm1*ecm1-mchd1q );

    // quadrimpulsi 1 e 2 in mres12
    double cs,st,cf,sf;

    cs=p1->cstar;
    st=sqrt ( 1.-cs*cs );
    cf=cos ( p1->fistar );
    sf=sin ( p1->fistar );
    qvet P1v ( st*cf*pcm, st*sf*pcm,cs*pcm,p1->Get_Massa() );
    qvet P2v ( -P1v.x,-P1v.y,-P1v.z,p2->Get_Massa() );

// calcolo degli impulsi in k cm

    P1cm=P1v.Lburst ( Pres12 );
    P2cm=P2v.Lburst ( Stesso );
    e1=P1cm.e;
    e2=P2cm.e;



//    Gout<<"\n Ev_cm "<<Eventi_Generati<<" e "<<setw(8)<<e1<<" "<<setw(8)<<e2<<" "<<setw(8)<<e3
//    <<" e "<<setw(8)<<eres12<<" "<<setw(8)<<pp12<<" "<<setw(8)<<pp1;
    s1=SelRea->ss0=mpadreq+mchd1q-2.*mpadre*e1;
    s2=SelRea->ss1=mpadreq+mchd2q-2.*mpadre*e2;
    s3=SelRea->ss2=mpadreq+mchd3q-2.*mpadre*e3;

 //cout<<"\n ev "<<Eventi_Generati<< "   sss "<< s1<<" "<<s2<<" "<<s3;
// attenzione i parametri per root si definoscono solo con genera=0
    if ( genera!=0 ) {
        return 1;
    }

// per root
// root....continue
    evento_.Gen.mres12=mres12;
    evento_.Gen.eres12=eres12;
    evento_.Gen.ge1=e1;
    evento_.Gen.ge2=e2;
    evento_.Gen.ge3=e3;
    evento_.Gen.gss1=s1;
    evento_.Gen.gss2=s2;
    evento_.Gen.gss3=s3;
    evento_.Gen.iterfase=iterfs;
    return 1;
}




