/***************************************************************************
 *   Copyright (C) 2005 by Giuseppe Pierazzini          Pisa 20.02.05      *
 *   pierazzini@unipi.it                                                   *
 *                                     F l y o                             *
 *   Epsi/NA48                                                             *
 ***************************************************************************/
#include "flyoh.h"

int Debug_brems=0;
using namespace std;
using namespace GmpConst;

/**
PartLepton
Controlla la produzione di bremsstrhalung  dinamica, vedi anche Partgam per la pair production.
La generazione di elettroni (e muoni) è attivata dall parola chiave in .epc:
"bremsstrahlung   "  non commentata!
Nella definizione del device occorre inserire una linea con
" brems   xxx " dove xxx è il numero massimo di gamma generati..
Attenzione nel caso si voglia attivare la produzione di sciami elettromagnetici il numero
indicato può essere alto come 100 o addirittura 500. In questo caso deve essere attivata anche la pair production da gamma con:
 "pairprod  yyy " con  yyyy  simile a quello previsto per il bremsstrhalung.
 Nella produzione si creano automaticamente gamma e ed elettroni marcati per la produzioni di  pair o brems. Il numero massimo
 di particelle (leptoni e gamma) creati è legato  ai numeri su discussi e al valore della soglia (Ecut) definito in PartLepton().
 Per esempio in un radiatore compatto e abbastanza lungo in lunghezze di radiazioni, si possono produrre numerose particelle, mille o più...
30.12.2009 gmp
<br>
*@author Giuseppe Pierazzini
*/


PartLepton::PartLepton()

{
    type=Lepstar;

    Ecut=0.250;
    // attenzione questo taglio determina l'energia minima ( E_critica)  di soglia per la produzion del Brems.  
    // dovrebbe dipendere da materiale attraversato....tuttavia gamma di più bassa energia non sono visti dall'apparato...
    // Nella distribuzione della energia di ionizzazione rilasciata dagli elettrini  nel radiatore appare un picco
    // alla stessa energia di soglia su scelta.
    // Questa soglia determina anche il numero delle particelle (gamma. elettroni e positroni)
    // generati nella simulazione per ciascun evento.
    brems_factor=1.0;
    double mele= ptr_Elec->Get_Massa();
    if(massa> mele){
    double mrap= mele/massa;   
    brems_factor=mrap*mrap;    // per i mu si definisce un fattore di scala
    }
   
    if (Abilita_Brems) Brems_Production=1;
    else Brems_Production=0;

    if (evento_.Gen.Event<1000)
    {
        Gout<<"\n Classe PartLepton  : "<<nome;
        if (Abilita_Brems) Gout<<"*  done!  with Brems_factor: "<<setprecision(6)<<brems_factor;
        else Gout<<"  done! ";
    }

}


PartLepton::~PartLepton()
{
}

//********************************************
// bremssthralung section...
int   PartLepton::
Get_Brems(Device *dev,double &path,double rad)
{
// questa procedura e' chiamata da Device::traccia()
// se siamo qui....c' e' possibilita' di bremsstrhalung
// nell'interno o nel buco del dev..
//-----------------------------------------------
// vedi l'articolo di Y. S. Tsai, Pair Production and Bremsstrahlung of Charged Leptons,
// Rev. Moder. Phys. 46,1974

//
// procedura:
// si accettano solo particelle con energia superiore a Ecut
// see the blue book or the article of Tsai del 1974 Rev.Moder. Phys. 46,4
//-------------------------------------------------
   

    double rad_length=rad;

    if (path/rad_length<.00001) return 0;   //ritorna subito per cammini insignificanti
    if (brems_factor<0.9) rad_length/=brems_factor;  // per particelle di massa superiore all'elettrone(mu) ....
    
    double e=P.e;
     e-=dev->Msperdita(path); 
    if (e<Ecut) return 0;  // attenzione a questo cut... vedi il costruttore Partlepton()
    ebrem=mbrem=0.0;

    double Emax=e-massa;
    double Ebgam= (Emax-Ecut)*Pran()+Ecut;
    double rape= Ebgam/e;
    double Bprob= (path/rad_length)*(1.3333-1.3333*rape+ rape*rape)/Ebgam;
  
    
    if(Pran()<Bprob)
    {
        ebrem=Ebgam;
   
        Brems_Production=2;  // brems fatto....
        fato=Decay;// si fa per dire....si fa decare?!
        Ifato=-13;
        path=path*Pran();     //cammino libero ridefinito....prima di decadere: random lungo il potenziale cammino....
        SelRea->Brem_done++;
        dev->Brem_done++;
       
        if (Debug_brems==1)
        {
            P.print( nome);
            X.print(nome);
            Gout<<"\n Brlm "<<evento_.Gen.Event<<" "
            <<fato<< " dw "<<dw
            << " dev " << dev->Get_Nome()<<" Brem_done "<<dev->Brem_done
            << "  path " <<path;
        }
     
       // generate the new partcles: un nuovo leptone ed un gamma
// max defined into device cards...
        Particella *pb,*pc;
// electron o muon   test se  possono fare ancora brems...
        Partname=nome;
        if (Abilita_Brems>0&&dev->Brem_done<=dev->maxBrems) {
            pb = new PartLepton();
        }
        else pb= new Particella();  //no brems....

// gamma di brems...test se  può fare pair production ...
        Partname=Gam;
        if (Abilita_Pair>0&& dev->Pair_done<=dev->maxPair)   {
            pc = new PartGam();  // no pair production
        }
        else pc= new Particella();
// links: attention here the link is broken to the older particle... but see Restore_link
// attenzione non alterare la sequenza dei link....altrimenti cambiano gli Id
        dw=pb;
        pb->up=this;
        pb->rg=pc;
        pc->lf=pb;
        pc->up=this;
        pb->Set_Id();
        pc->Set_Id();

        return -13;  //brems ok
    
    }  
        return 0;  // no brems
}
//-------------------------------------------------
//     B r e m s s t r h a l u n g
//     produzione di gamma da elettroni
//--------------------------------------------------------------
// Get the brems mass  (attenzione al decadimento)

void  PartLepton::
DecParm_Brems()
{
// Attenzione questa routine vale solo per la produzione
// di gamma da bremsstrahlung
// Procedura
// Si ridefiniscono i parametri della particella che fa bresstrhalung
// immaginandola una particella con massa virtuale che decade in lepton piu gamma
// di bremsstrhalung di energia ebrem definata in Get_Brems().
//-------------
// (1)
// calcolo la massa invariante del sistema e-gam con forma esponenziale..
// partendo da un valore limite minimo che tiene conto dell'energia del gamma emesso
// (2)
// ridefinizione del quadrimpulso della particella padre
// si aggiorna la massa (virtuale) e si ricalcola l'energia
// (3)
// nota l'energia del gamma nel lab, calcolo il coseno* di decadimento
// compatibile con l'energia (ebrem) del gamma nel lab.
    if (Debugon==1)
    {
        P.print( "Qpatre");
        Gout<<" Brem_ev "<<evento_.Gen.Event;
    }

    if (fato!=Decay || dw==0) return ;
    double Et,Ele,pe,masmin;
    Et= Qpatre.e;
    Ele=Et-ebrem;

// caso perdita energia per multiple scattering aggiusta...
    if (Ele<Ecut)
    {
//        Gout<<"\n *** ERRORE *** Ev "<<evento_.Gen.Event<<setprecision(4)<<" ebrem "<< ebrem<<"  Ele "<<Ele<<"Q.e  "<< Qpatre.e;
        Ele=Ecut;
        ebrem=Et-Ecut;
    }


    pe=sqrt(Ele*Ele-massa*massa);
    masmin = sqrt( massa*massa + 2.*(Ele -pe)*ebrem);

//   Gout<<"\nDecBrem:  Ev "<<evento_.Event<<"  Ele "<<Ele<<" ebrem="<<ebrem<<" ms="<<masmin;

// genero la massa a random esponenzialmente partendo dalla minima... ...piccola
//    mbrem= -0.001*log(Pran()+EPSI)+masmin;  //larghezza ... mev
    mbrem= -0.01*log(Pran()+EPSI)+masmin;  //larghezza ... mev
    mbremq= mbrem*mbrem;
    SelRea->mbrem=mbrem;  //ricorda l'energia in reazione...
    Qpatre.m=mbrem;
    Qpatre.mq=mbremq;
// piccola correzione (rinormalizzazione)  dell 'impulso... della particella padre
// a energia totale costante....

    Qpatre.Impulso();


// calcolo il coseno* di decadimento
    double  gamma = Qpatre.e/Qpatre.m;
    double  beta  = Qpatre.norma/Qpatre.e;

    double    pstar = 0.5*(mbremq-massa*massa)/mbrem;   // = Estar del gamma

//    double cost = (Ele-ebrem-gamma*massa*massa/mbrem)/(2.*beta*gamma*pstar);
    double cost = -(ebrem-gamma*pstar)/(beta*gamma*pstar);

//      Gout<<"\n ebrem "<< ebrem<<"  mbrem "<<mbrem<<"  pstar "<<pstar<<"  cos "<<cost;
    /*
    //if(evento_.Event>83500)
    {
       char sbuf[128];
       sprintf(sbuf,"\nDecBrem: Ev %6d id  %2d ET %10.6f Ele %10.6f pe %10.6f Min  %8.6f  mb %8.6f  eb %10.6f cs  %8.6f z %8.1f ",
                       evento_.Event,id,Et,Ele,pe,masmin,mbrem,ebrem,cost,X.z);
       Gout<<sbuf;
       P.print(evento_.Event,nome,id,"DecBrem");
       Qpatre.print(evento_.Event,nome,id,"qPatre");Gout<<endl;
    }
    */

// Ridefinisco Pbar....Attenzione se si dimentica si sbaglia lorentz....
    Pbar=Qpatre;
//----------------------------------------------------------------
// memorizzo la direzione di decadimento dei figli.
    double   fi = DuePiGreco * Pran();
    dw->cstar=cost;
    dw->fistar=fi ;             //fi  elettrone
    dw->rg->cstar=-cost;
    dw->rg->fistar=fi+PiGreco; //fi+pigrec  gam
}



