/***************************************************************************
                          partres4.cpp  -  description
                             -------------------
    begin                : Mon Jun 18 2001
    copyright            : (C) 2001 by Giuseppe Pierazzini
    email                : peppe@unipi.it
  ***************************************************************************
  *                                                                         *
  *   NA48  simulation program.                                             *
  *                                                                         *
  ***************************************************************************/


#include "flyoh.h"
#include "partres4.h"
#include "fasemax.h"
#include "pip0ee.h"

using namespace GmpConst;
using namespace std;
/**
Four bodies decay like  k -> p1 p2 p3 p4 in phase space..
the decay is via the res2 and vres2  resonance
generated in the decay chain automatically..
 *@author Giuseppe Pierazzini
 */
Partres4::Partres4()
{
    p1=p2=p3=p4=p5=pr1=pr2=0;
    type="Dec4";
    Gout<<"\n Class Partres4  : "<<nome<<" done!";
}


void Partres4::DecParm()
{
// skip.. if  pr1 allready defined....
//    Gout<<"\n Ev "<<Eventi_Generati<< " in decparm "<<nome <<flush;

    if ( pr1==0 ) {
        // first time...
// define the pointers to the  particles;
// and the decay parameters
// evaluate the max weight of phase space (X matrix element if defined )

        Get_PhaseParm();
    }
//
//Gout<<"\nGet_Parm 4 ev "<<Eventi_Generati<<" begin";
// get the decay parameters taking into account phasespace and matrix element if Mtx!=0;
    Get_PhaseSpace();

// define the resonace particle parameters..masses...
// res 1
    pr1->Set_Massa ( mres12 );
    pr1->P.Riposo ( mres12 );
// res 2
    pr2->Set_Massa ( mres34 );
    pr2->P.Riposo ( mres34 );
// si definiscono i parametri di decadimento e si memorizzano per root
    Define_CMvalues ( 0 );
// Gout<<"\nGet_Parm end  energie: "<<e1<<" "<<e2<<" "<<e3<<" "<<e4<<endl;
}


void Partres4::Get_PhaseParm()
{
    /*-------------------------------------------
       four bodies decay  (notare la sequenza da scrivere nella scheda....)
         // M è il padre,
    M => p1  p2  p3 p4   generata secondo la catena
    M =>  mres12 mres34  generata secondo la catena
    mres12 =>   P1 p2
    mres34 =>   p3  p4

    Per esempio K+=> pip pi0   e+ e- va in
     mres12= pip pi0
     mres34 = e+ e-

      -----
       Phase space weight
       w = F(padre,mres12,mres34)*F(mres12,p1,p2)*F(mres34,p3,p4)

      --------------------------------------------------*/

//Gout<<"\n Ev "<<Eventi_Generati<<" partfase4 parm per "<<nome<<flush;

    if ( pr1!=0 ) {
        return;
    }

    pr1=this->dw;
    pr2=this->dw->rg;
    p1=pr1->dw;
    p2=p1->rg;
    p3=pr2->dw;
    p4=p3->rg;

// particle parameters
    mpadre=Get_Massa();
    mpadreq=mpadre*mpadre;
    mchd1=mfrate=p1->Get_Massa();
    mchd1q=mfrateq=mfrate*mfrate;
    mchd2 =p2->Get_Massa();
    mchd2q=mchd2*mchd2;
    mchd3 =p3->Get_Massa();
    mchd3q=mchd3*mchd3;
    mchd4 =p4->Get_Massa();
    mchd4q=mchd4*mchd4;



// massimi e minimi valori delle risonanze
    mmin12=mchd1+mchd2;
    mminq12 =mmin12*mmin12;
    mmin34=mchd3+mchd4;

//===============
// if you prefer a mass cut put here.....
    mmin34=0.005;
//===============
    
    mminq34=mmin34*mmin34;
    mmax12=mpadre-mmin34;
    mmaxq12=mmax12*mmax12;
    mmax34=mpadre-mmin12;
    mmaxq34=mmax34*mmax34;
    dmq12 =mmaxq12-mminq12;
    dmq34=mmaxq34-mminq34;





// nota Fase

    int p_id[6]= {Get_Idm(),p1->Get_Idm(),p2->Get_Idm(),p3->Get_Idm(),p4->Get_Idm(),0};
    if ( Fasemx!=0 ) {
        double  wmx=Fasemx->Get_Wmax ( p_id );
        if ( wmx>0. ) {
            wfase_max=wmx;
//      Gout<<"\n Ev "<<Eventi_Generati<<" F.S. trovato "<<up->Get_Nome() <<" "<<nome<<"=> "<<p1->Get_Nome()
//       <<" "<<p2->Get_Nome() <<" "<<p3->Get_Nome()<<" "<<p4->Get_Nome();
            return;
        }
    }


//allora calcolo il valore massimo

    double pass12=dmq12/200.;
    double pass34=dmq34/200.;

//   Gout<<"\n Passi "<<scientific<<setprecision(6)<<pass12<<" "<<pass34;
    double w1,w2,w3,w;
//   double wbutta=0.;
    double wmb=0.0;
    // define the wfase_max for p.s.  normalizazion
    wfase_max=0.0;
    for ( mresq12=mminq12; mresq12<mmaxq12; mresq12+=pass12 ) {
        mres12=sqrt ( mresq12 );

        w1=Fspace ( mres12,mchd1,mchd2 );


        if ( w1<=0.0 ) {
            continue;
        }
        for ( mresq34=mminq34; mresq34<mmaxq34; mresq34+=pass34 ) {
            mres34=sqrt ( mresq34 );
            if ( mres12+mres34>mpadre ) {
                continue;
            }
            w2=Fspace ( mpadre,mres12,mres34 );
            w3=Fspace ( mres34,mchd3,mchd4 );
            w=w1*w2*w3;
            //wbutta=w;
            //**** segue l'elemento di matrice:
// se è definito  sempre "1.0" (default) o Mtx ==0  si genera lo spazio delle fasi puro....
            if ( w<=0.0 ) {
                continue;
            }
            if ( Mtx!=0 ) {
                double vmax=0.0;
                for ( int i=0; i<10; i++ ) {  // provo più configurazioni angolari
                    wmb=Mtx->M_ValoreMax();
                    if ( wmb>vmax ) {
                        vmax=wmb;
                    }
                }
                wmb=vmax;
                w*=wmb;


            }


            //   Gout<<"\n loopx "<<setw(8)<<mres12<<" "<<setw(8)<<mres34<<" wmb "<<setw(8)<<wmb<<" w "<<setw(8)<<w;

            if ( w>wfase_max ) {
                wfase_max=w;

                ///          Gout<<"\n Ricerca Wmax "<<wfase_max <<" wmb "<<wmb <<setprecision(4)<<" masse "<< mres12 <<" "<<mres34;
            }

//        Gout<<"\n w "<<setw(8)<<w1<<" "<<setw(8)<<w2<<" "<<setw(8)<<w3<<" "<<setw(8)<<w<<" "<<setw(8)<<wfase_max<<flush;

        }
    }

    Gout<<"\n Phase-Space4: w_max ="<<wfase_max<<"   done for " <<Get_Nome() <<" ==> "<<p1->Get_Nome() <<" "<<p2->Get_Nome()
        <<"  "<<p3->Get_Nome() <<" "<<p4->Get_Nome();

    new FaseMax ( p_id,wfase_max );

}

void  Partres4::Get_PhaseSpace()
{
    iterfs=0;
    
    while ( iterfs++<10000 ) {
        // resonance mass selection...

        if ( Mtx!=0 ) {
            Mtx->Get_FaseMass();    // vedi in pip0ee.cpp
        } else {
            mres12    =sqrt ( ( mresq12 =mminq12+Pran() *dmq12 ) );
            mres34    =sqrt ( ( mresq34 =mminq34+Pran() *dmq34 ) );
        }
        
        if ( mres12+mres34>mpadre ) {
            continue;
        }

        double w=Fspace ( mpadre,mres12,mres34 ) *Fspace ( mres12,mchd1,mchd2 ) *Fspace ( mres34,mchd3,mchd4 );

        //   Gout<<"\nGet_PhaseSpace4 ev "<<Eventi_Generati<<"  iter "<< iterfs;

        //**** segue l'elemento di matrice:
// se è definito  sempre "1.0" (default) o Mtx ==0  si genera lo spazio delle fasi puro....


        if ( w<=0.0 ) {
            continue;
        }
        if ( Mtx!=0 ) {
            w*=Mtx->M_Valore();

//           Gout<<"\n ev "<<setw(8)<<Eventi_Generati<<"  iterfs "<<setw(4)<<iterfs<<"  mee "<<setw(8)<<mres34
//           <<setprecision(5)<<"  w "<<setw(12)<<w  <<" wfase_max "<<setw(12)<<wfase_max<<" w/wmx "<<setw(12)<<w/wfase_max;

        }

        // Gout<<"  e .. "<<e1<<" "<<e2<<" "<<e3<<" "<<e4;


        if ( w >wfase_max ) {
            Gout<<"\n Wfas off: Ev "<<evento_.Gen.Event<<" iter="<<setw ( 6 ) <<iterfs
                <<setprecision ( 5 )
                <<" Risonaze: m12="<<setw ( 12 ) <<mres12<<" m34="<<setw ( 12 ) <<mres34<<setprecision ( 2 )
                <<" w "<<setw ( 12 ) <<w <<" wfase_max"<<setw ( 12 ) <<wfase_max<<"   "<<SelRea->Get_Titolo();
            wfase_max=w+0.000001;

            Gout<<"   new "<<wfase_max;
// aggiorna  wfase_max memorizzato nella lista
            int p_id[6]= {Get_Idm(),p1->Get_Idm(),p2->Get_Idm(),p3->Get_Idm(), p4->Get_Idm(),0};
            Fasemx->Set_Wmax ( p_id, wfase_max );
        }
        if ( Pran() <w/wfase_max ) {

            //     Gout<<"\n ev "<<setw(8)<<Eventi_Generati<<"  iterfs "<<setw(4)<<iterfs<<"  mee "<<setprecision(4)<<setw(8)<<mres34;

            break;
        }

//    Gout<<"\n PhaseSpace out   "<<mres12<<" "<<mres34<<flush;
        if ( iterfs>9998 ) {
            Gout<<"\nEv "<<setw ( 8 ) <<Eventi_Generati<<" PartRes4 overflow  iter="<<iterfs<<setprecision(4)<<" Risonaze: m12="<<mres12<<" m34="<<mres34;
            break;
        }
    }

}


int Partres4::Define_CMvalues ( int genera )
{

    // calcolo in anteprima  la direzione di decadimento dei figli.
//  nel cm res2  (qui isotropo)
//cout<<"\n Cm ev "<<Eventi_Generati;
    static qvet Stesso;

    //  Gout<<"\n Partres4 Define_CMvalues...ev "<<Eventi_Generati<<" nome "<<nome<< " genera "<<genera;
// energie di res12  e impulso di res12  in K(padre)  cm
    eres12=0.5* ( mpadreq+mresq12-mresq34 ) /mpadre;
    double pp12=sqrt ( eres12*eres12-mresq12 );
    qvet Pres12 ( 0.,0.,pp12,mres12 ); // immaginato lungo z (arbitrario)

// decadimento di mres12

    if ( genera==1||p1->cstar>1. ) {
        double p1cst = 1.0 -  2.0 * Pran();
        double p1fi = DuePiGreco * Pran();
        p1->cstar=p1cst;
        p1->fistar=p1fi;
        p2->cstar=-p1cst;
        p1fi+=PiGreco;
        if ( p1fi>DuePiGreco ) {
            p1fi-=DuePiGreco;
        }
        p2->fistar=p1fi;

    }

    // energie delle particelle e1 e e2 in mres12 cm
    double ecm1=0.5* ( mresq12+mchd1q-mchd2q ) /mres12;
//   double ecm2=mres12-ecm1;
    double pcm=sqrt ( ecm1*ecm1-mchd1q );

    // quadrimpulsi 1 e 2 in mres12
    double cs,st,cf,sf;

    cs=p1->cstar;
    st=sqrt ( 1.-cs*cs );
    cf=cos ( p1->fistar );
    sf=sin ( p1->fistar );
    qvet P1v ( st*cf*pcm, st*sf*pcm,cs*pcm,p1->Get_Massa() );
    qvet P2v ( -P1v.x,-P1v.y,-P1v.z,p2->Get_Massa() );

// calcolo degli impulsi in k cm

    P1cm=P1v.Lburst ( Pres12 );
    P2cm=P2v.Lburst ( Stesso );
    e1=P1cm.e;
    e2=P2cm.e;


    // energie di res34  in K cm
    eres34=0.5* ( mpadreq-mresq12+mresq34 ) /mpadre;
    qvet Pres34 ( 0.,0.,-pp12,mres34 ); // immaginato lungo -z (arbitrario)

    if ( genera==1||p3->cstar>1. ) {
        double p3cst = 1.0 -  2.0 * Pran();
        double p3fi = DuePiGreco * Pran();
        p3->cstar=p3cst;
        p3->fistar=p3fi;
        p4->cstar=-p3cst;
        p4->fistar=p3fi+PiGreco;
        p3fi+=PiGreco;
        if ( p3fi>DuePiGreco ) {
            p3fi-=DuePiGreco;
        }
        p4->fistar=p3fi;
    }
    double ecm3=0.5* ( mresq34+mchd3q-mchd3q ) /mres34;
//   double ecm4=mres34-ecm3;
    pcm=sqrt ( ecm3*ecm3-mchd3q );
    // quadrimpulsi 3 e 4 in mres34
    cs=p3->cstar;
    st=sqrt ( 1.-cs*cs );
    cf=cos ( p3->fistar );
    sf=sin ( p3->fistar );
    qvet P3v ( st*cf*pcm, st*sf*pcm,cs*pcm,p3->Get_Massa() );
    qvet P4v ( -P3v.x,-P3v.y,-P3v.z,p4->Get_Massa() );

// calcolo degli impulsi in k cm
    P3cm=P3v.Lburst ( Pres34 );
    P4cm=P4v.Lburst ( Stesso );
    e3=P3cm.e;
    e4=P4cm.e;

//
// attenzione i parametri per root si definoscono solo con genera=0
    if ( genera!=0 ) {
        return 1;
    }
    
    // SE NELL'EVENTO CI SONO PIÙ RISONANZE QUESTI DATI SONO SOVRASCRITTI...!!!!!
    s1=SelRea->ss0=mpadreq+mchd1q-2.*mpadre*e1;
    s2=SelRea->ss1=mpadreq+mchd2q-2.*mpadre*e2;
    s3=SelRea->ss2=mpadreq+mresq34-2.*mpadre*eres34;
// root....
    evento_.Gen.mres12=mres12;
    evento_.Gen.eres12=eres12;
    evento_.Gen.mres34=mres34;
    evento_.Gen.eres34=eres34;

    evento_.Gen.ge1=e1;
    evento_.Gen.ge2=e2;
    evento_.Gen.ge3=e3;
    evento_.Gen.ge4=e4;
    evento_.Gen.gss1=s1;
    evento_.Gen.gss2=s2;
    evento_.Gen.gss3=s3;
    evento_.Gen.iterfase=iterfs;

//     if(genera==0) {
//      Gout<<"\n SF ev "<<Eventi_Generati<<" p12 "<<setw(8)<<p1->cstar<<" "<<setw(8)<<e1<<" "<<setw(8)<<e2<<" masse "<<mres12<<" "<<mres34
//      <<" e "<<ecm1<<" "<<ecm2<<" "<<ecm3<<" "<<ecm4;
//      Gout<<"\n SF ev "<<Eventi_Generati<<" p34 "<<setw(8)<<p3->cstar<<" "<<setw(8)<<e3<<" "<<setw(8)<<e4<<" energ "<<eres12<<" "<<eres34<<" "<<eres12+eres34;;
//     }

    return 1;
}
