/***************************************************************************
                         particella.h  -  description
                             -------------------
    begin                : Sun Nov 26 2000
    copyright            : (C) 2000 by Giuseppe m Pierazzini
    email                : pierazzini@unipi.it
 ***************************************************************************
 *                                                                         *
 *   NA48  titles for simulation program.                                  *
 *                                                                         *
 ***************************************************************************/
// Pisa   28.10.99
//------------------------------------------------------------
//      classe          P a r t i c e l l a

#ifndef PARTICELLA_H
#define PARTICELLA_H
#include "gvet.h"

class Reazione;
class Part_db;
class Device;
class Matrice;
/**
 Class Particella: classe base per la descrizione delle particelle 
@author Giuseppe Pierazzini
*/

class Particella
{
    friend class Device;
    friend class DevStraw;
    friend class Part_db;
    friend class Bufdev;
    friend class Reagas;
    friend class Illumina;
    friend class Matrice;
// inserire altri nomi se necessario....

protected:
    const char *nome;      //nome della particella
    const char *fato;      // fato di end  fly
    const char *type;      // Beam,carico,beam......
    unsigned long long devstory; // 64 bit code to memorize hit devices...see scrivi colpi
    int devhit;      // l'ultimo device colpito
    int ido;         // indice progressivo della particella nella reaz.
    int id;         // indice logico  --attenzione Vedi Set_Id() per la corretta definizione...
    int idm;         // indice della part. nel database part_db
    int see;         // visibilita' nei rivelatori
    int nata;        // indice di nascita...non zero se generata nell'evento.
    int Ifato;       // integer fato flag Vedi Fato sopra.


    int Pair_Production,Brems_Production; // =1 puo' fare pair o brems.. = 2 ha fatto....
    double charg,massa,width,ctau; // parametri fisici
    gvet Gx ;           // vett. pos. inziale
    qvet Gp ;           // quadrivettore iniziale
    qvet Pbar;          // quadrivettore
    qvet Pburst;        // quadrivettore
    qvet Qpatre;        // quadrivettore del padre generatore

    double tmp;         // tempo di produzione

    double prandom;// to restore the link and parametrs... in reset.....
    Part_db *pointer_db;
    // puntatori alle particelle di  : sinistra,destra,sopra,sotto, urto 
    Particella *lfsav,*rgsav,*upsav,*dwsav;

public:
//    int Irdm_count,Gauss_count,Irdm_last_count,Gauss_last_count;
    Reazione *rea_madre;
    Matrice *Mtx;
    
    gvet X, Vers;
    qvet P;
    double cstar,fistar; // costet e fi rispetto alla line di volo del padre e nel padre cm ...
    double pathok,path_max,path_done,last_path;  //tracing parameters....
 //
 // puntatori alle particelle di  : sinistra,destra,sopra,sotto,cluster, urto, prossima. 
 // esempio A => B  C 
 //  A.dw punta a B
 //  B.up punta a A
 //  B.rg punta a C
 //  C.up punta a A
 //  C.lf punta a B
 //  poi se si genera un cluster come nelle interazionigas A.cls punta all prima particella del gas_cluster
 //  se A deve urtare con un bersaglio D  A.urt punta a D ( vedi implementazione in ....)
 //
   
 
    Particella *lf, *rg, *up, *dw,*cls, *before,*next;
    Particella ();

    virtual ~Particella () {};
//                                Procedure
    const char   *Get_Nome() {
        return nome;
    }
    const char   *Get_Fato() {
        return fato;
    }
    const char   *Get_Type() {
        return type;
    }
    int     Last_Devhit() {
        return devhit;
    }
    int     Get_Ifato() {
        return Ifato;
    }
    unsigned long long Get_DevStory() {
        return devstory;
    };

    int     Get_Idm() {
        return idm;
    }
    int     Get_Id() {
        return id;
    }
    int     Get_Ido() {
        return ido;
    }
    int     Get_See() {
        return see;
    }
    int     Get_Nata() {
        return nata;
    }
    double  Get_Massa() {
        return massa;
    }
    double  Get_Massq() {
        return P.mq;
    }
    double  Get_Width() {
        return width;
    }
    double  Get_Ctau() {
        return ctau;
    }
    double  Get_Charg() {
        return charg;
    }
    gvet &  Get_Gx() {
        return Gx;
    }
    qvet &  Get_Gp() {
        return Gp;
    };
    double  Get_Gpx() {
        return Gp[0];
    }
    double  Get_Gpy() {
        return Gp[1];
    }
    double  Get_Gpz() {
        return Gp[2];
    }
    double  Get_Gpn() {
        return Gp.norma;
    }
    void Set_Ido(int num){ido=num;}
    void    Set_Id();
    void    Set_Type(const char *tip) {
        type=tip;
    };
//     void    Set_undef (){fato = Undef; Ifato=0;};

    void    Set_Nata() {
        nata=1;
    }
    void    Set_Fato ( const  char *w,int ift ) {
        fato=w;
        Ifato=ift;
    }
    void    Set_Massa ( double m ) {
        massa=m;
    }
    void    Set_Pbar() {
        Pbar=P;
    }
    void    Save_Link();
    void    Restore_Link();
    void    Update_Devstory(int );
    void    P2Gp() {
        Gp=P;
    }
    void    X2Gx() {
        Gx=X;
    }
    void    Move ( double );
    void    Debug_out ();
    void    Spia ( char *, Device *,Device *,int,int );
    void    Reset_Stato();
    void    Stampa_questa_particella();
    void    Stampa_particle ( );

    Particella * Decay_in_2( const char *,const char * );
    Particella * Decay_in_3( const char *,const char*,const char *);
    Particella * Decay_in_4( const char *,const char*,const char *,const char *);
    Particella * Decay_in_5( const char *,const char*,const char *,const char *,const char *);
    void    Aggiorna_link ( Particella *, Particella *, Particella *,
                            Particella * );
    void    Aggiorna_link_Bullet ( Particella *);		    
    void    Aggiusta_link ( Particella *, Particella *, Particella *,
                            Particella * );

    void Sostituisci_part( Particella *pr);
    
    virtual void DecParm() {};  //parametri massa e angoli di decadimento:
    virtual void DecParm_Pair() {};
    virtual void DecParm_Brems() {};
    virtual double Get_Ecut() {
        return 1.0;
    }; // dummy per PartBrem;
    virtual int Get_Brems ( Device *,double &,double  ) {
        return 0;
    };
//    virtual double Get_Brems() {return 0.0;}
    virtual void Get_Pair ( Device *,double&,double ) {};
    
//  virtual bersaglio     
    virtual void Get_Bersaglio ( Device *, double  ) {};
     
     
     
    virtual int Svalues() {
        return 0;
    };
    virtual double Get_e1() {
        return 0.0;
    };
    virtual double Get_e2() {
        return 0.0;
    };
    virtual double Get_e3() {
        return 0.0;
    };
    virtual double Get_S1() {
        return 0.0;
    };
    virtual double Get_S2() {
        return 0.0;
    };
    virtual double Get_S3() {
        return 0.0;
    };
    virtual int Get_ss_done() {
        return  -1;
    };
    
    virtual int Define_CMvalues(int){return 1;};
    int Main_Loop();
    int Voldec ( Particella *, Particella * );
    int Trasport ();
    Particella *Get_Pointer ( char * );
    Particella *Ptr_Particle ( const  char * );

};

#if defined(MAIN)
//  Particle flags  .

const char *Undef = "undef", *Lost = "lost",*Nata="nata";
const char *Decay= "decay",*Decfar="decfar",*Gasint="gasint";
const char *Bucin= "bucin",*Inbuc="inbuc";
const char *Distr= "distr";
const char *Novede = "Unseen";
#endif

#if !defined(MAIN)
//  Particle flags  .
extern const char *Undef, *Lost,*Nata,*Decay,*Decfar,*Gasint;
extern const char *Bucin,*Inbuc;
extern const char *Distr;
extern const char *Novede;
#endif

#if defined(MAIN)
int   Partflag;
const char *Partname;
#endif
#if !defined(MAIN)
extern  int   Partflag;
extern const char *Partname;
#endif









#endif

