/***************************************************************************
   *    Ftta by gmp  10.06.2012
  ***************************************************************************/

#include "parm.h"
#include "gmatrix.h"
#include "gvet.h"
#include "gfitta.h"
// gfitta.h contiene la struttura gemnerica per l'interfaccia con i dati

using namespace std;

int dbg_fitta=-1;
/**
<p>Procedura di fittting del tutto generale.
Calcola le derivate numericamente.</p>
<br>L'utente deve definire:
<ul>
<li>Il numero dei parametri Y misurati.
<li>Il numero dei parametri X da stimare ( se esistono).
<li>Il numero dei vincoli.
<li>Deve fornire la procedura Fvincoli(int) per il calcolo del vincolo
</ul>
*/


Gfitta::Gfitta()
{



    Vprecis=1.0;    // ridefinire secondo il problema
    Derstepx=0.00001;
    Derstepx2=2.*Derstepx;
    Derstepy=0.00001;
    Derstepy2=2.*Derstepx;

}

void Gfitta::Set_dim(int nyy,int nvv,int nxx,int nvp)
{
    ny=nyy;
    nv=nvv;
    nx=nxx;
    nvec=nvp;

    YM.Set_Size(ny,1);   // misure
    Y.Set_Size(ny,1);   // teoriche
    X.Set_Size(nx,1);   // parametri da stimare
//    G.Set_Size(ny,ny);   // matrice degli errori; si può ridurre ad un vettore!
    F.Set_Size(nv,1);    // vincoli
    AF.Set_Size(nv,nx);  // derivata dei vincoli rispetto alle variabili da stimare
    BF.Set_Size(nv,ny);  // derivata rispetto a Y; forse si può ridurre
    R.Set_Size(nv,1);    // residuoi
}

gmatrix Gfitta::Fvincoli(int index )
{
// procedura che dipende dal problema..... con overload
// qui un esempio come tamplate
    gmatrix Fnow(nv,1);
// ora non fa nulla

    return Fnow;
}

//
gmatrix Gfitta::AFderiva()
{
// calcolo della matrice AF con derivate numeriche
// si può applicare  l'overloading

    gmatrix Fup(nv,1),Fdown(nv,1),Faux(nv,1);
    gmatrix Ax(nv,nx);

    for (int i=0;i<nx;i++)
    {
        double saveprm= X(i,0);
        X(i,0)=saveprm+Derstepx;
        Fup=Fvincoli(3);
        X(i,0)=saveprm-Derstepx;
        Fdown =Fvincoli(4);
//  Fdown.Print("Fdown");
        Faux=(Fup-Fdown)/Derstepx2;
        X(i,0)=saveprm;
// copia in
        for (int j=0;j<nv;j++)
            Ax(j,i)=Faux(j,0);
    }
    return Ax;
}
//
gmatrix Gfitta::BFderiva()
{
// calcolo dellamatrice  BF con derivate numeriche
// si può applicare  l'overloading

    gmatrix Fup(nv,1),Fdown(nv,1),Faux(nv,1);
    gmatrix BF(nv,ny);

    for (int i=0;i<ny;i++)
    {
        double saveprm= Y(i,0);
        Y(i,0)=saveprm+Derstepy;
        Fup=Fvincoli(1);
        Y(i,0)=saveprm-Derstepy;
        Fdown =Fvincoli(2);
//  Fdown.Print("Fdown");
        Faux=(Fup-Fdown)/Derstepy2;
        Y(i,0)=saveprm;
// copia in
        for (int j=0;j<nv;j++)
            BF(j,i)=Faux(j,0);

    }
    return BF;
}
//     ========================     FITTING ===================
//============================

double Gfitta::Fitting()
{
// procedura generale di fitting
//  Moltiplicatori di lagrange. Funzine da minimizzare.
//  M= chi2 +lambda* F  o scritta sviluppando la F in serie...
//  M= Dy.G.Dy * 2.Lambda.(Ax.Dx + By.Dy + R)  deve essere minimo....

    double chiq=-1;
    gmatrix DY(ny,1);
//    gmatrix G(ny,ny),V(ny,ny);
    gmatrix BGB(nv,nv);
    gmatrix lambda(nv,1);
    gmatrix DX(nx,1);
    gmatrix ABGBA(nx,nx);
//
// fouri loop perchè costanti....

// get errors  ( da rivedere specie per errori variabili);
//  for (int i=0;i<ny;i++) {
//     G(i,i)=1.;
//       V(i,i)=1.;
    //  }

// errori se sono diagonali uso un metoido ridotto....
    gmatrix  Verror(ny,1);
    for (int i=0;i<ny;i++) Verror(i,0)=4.; // +- 2 ns

 //for (int i=0;i<ny;i++) Verror(i,0)=4.*peso[i]; // +- 2 ns

// copio le misure e i parametri iniziali
    for (int i=0;i<ny;i++) Y(i,0)=YM(i,0);





    if (dbg_fitta==1)X.Print("X");
    if (dbg_fitta==1)Y.Print("Y");

// LOOP
    for ( iter=0;iter<100;iter++)
    {
       if(iter%10==0) cout<<"\n Iterazionenum. "<<iter<<flush;
        
        F=Fvincoli(0);  // calcolo i vincoli

     if (dbg_fitta==1) 
        {
            if (iter<2) {
                Gout<<"\n loop iter "<<iter;
                F.Print("F");
                X.Print("X");
            }
        }

// test di fine corsa!!!
        double Fabs=0.0;
        int fcheck=0;
        for (int iv=0;iv<nv;iv++)
        {
            double fass= fabs(F(iv,0));
            Fabs+=fass;
            if (fass>Vprecis) fcheck=1;
        }
        //  if(Fabs/double(nv)<0.1)fcheck=0;



        if (dbg_fitta==1) {
            Gout<<"\n X "<<setw(8)<<X(0,0)<<" "<<setw(8)<<X(1,0)<<" "<<setw(8)<<X(2,0)<<" "<<setw(8)<<X(3,0)<<" "<<setw(8)<<X(4,0)
            <<"FSt:"<<Fabs/double(nv)<<" Iter "<<iter;

            //      cout<<"\n X "<<setw(8)<<X(0,0)<<" "<<setw(8)<<X(1,0)<<" "<<setw(8)<<X(2,0)<<" "<<setw(8)<<X(3,0)<<" "<<setw(8)<<X(4,0)
            //       <<"FSt:"<<Fabs/double(nv)<<" Iter "<<iter;

        }

        if (fcheck==0) {
            if (dbg_fitta==1)Gout<<"\n ============  Corsa finita. ========= iter "<<iter;
            if (dbg_fitta==1)cout<<"\n ============  Corsa finita. ========= iter "<<iter;
            break;
        }

        BF=BFderiva();
        if (dbg_fitta==3)BF.Print("BF");


////metodo ridotto
        gmatrix GBF(nv,ny);
        for (int iv=0;iv<nv;iv++)
        {
            for (int iy=0;iy<ny;iy++)
            {
                GBF(iv,iy)= BF(iv,iy)*Verror(iy,0);
            }
        }


//      BGB= BF*V*(~BF);
        BGB= BF*(~GBF);

        if (dbg_fitta==1)Gout<<"\n BGB determinante "<<BGB.Det();

        if (dbg_fitta==3)BGB.Print("BGB");
        BGB =!BGB;
        
        if(BGB.okflag==-1) { //Errore di inversione 

         Gout<<"\n Errore di inversione BGB exit con chiq=-1.";
          chiq=-1.;
          return chiq;
        }
          
          
          
        if (dbg_fitta==3) BGB.Print("BGB-1");





        R=F-BF*DY;
        if (dbg_fitta==2)  R.Print("R");

        AF=AFderiva();
        if (dbg_fitta==1) AF.Print("AF");

        ABGBA= (~AF)*BGB*AF;
        if (dbg_fitta==1)Gout<<"\n ABGBA determinante "<<ABGBA.Det();
        if (dbg_fitta==1) ABGBA.Print("ABGBA");

//attenzione ABGBA non è invertivbile se non ci sono sufficienti vincoli; almeno
// pari alle incognite X

        ABGBA=!ABGBA;
        if (dbg_fitta==2) ABGBA.Print("ABGBA-1");
        DX = -ABGBA*(~AF)*BGB*R;
        if (dbg_fitta==2) DX.Print("DX");
        lambda = BGB*(AF*DX+R);
//       DY=-G*(~BF)*lambda;
        DY=-(~BF)*lambda;
        for (int iy=0;iy<ny;iy++) DY(iy,0)/=Verror(iy,0);


        X+=DX;

        if (dbg_fitta==1)X.Print("X");

        Y+=DY;
        gmatrix ddy(ny,1);
        ddy=YM-Y;

//         gmatrix cc(1,1);
//         cc=(~ddy)*G*ddy;  // caso generale con errorti non diagonali..
//         chiq=cc(0,0);

        chiq=0.0;
        for (int iy=0;iy<ny;iy++)
        {
            chiq+=ddy(iy,0)*ddy(iy,0)/Verror(iy,0);
        }
//





        if (dbg_fitta==1)Gout<<"\n  ===========>    Chiq  "<<chiq;

    }
    return chiq;
}



//=============

gvet  Gfitta::Autovalore(gvet *Xp)

{
// ============================================

// METODO DEGLI AUTOVETTORI  (solo per matrici 3x3)

    gvet Xmed;
    double totale=0.0;
    gmatrix A(3,3);
    for (int i=0;i<nvec;i++) {
        Xmed+=Xp[i]; totale+=peso[i];
    }
   double fnvec=double(nvec);
    Xmed/=fnvec;
    if (dbg_fitta==1)Xmed.print("Xbar");


    for (int it=0;it<nvec;it++)
    {
        gvet Xnow=Xp[it]-Xmed;
        A(0,0)+=Xnow.x*Xnow.x;
        A(1,1)+=Xnow.y*Xnow.y;
        A(2,2)+=Xnow.z*Xnow.z;
        A(0,1)+=Xnow.x*Xnow.y;
        A(0,2)+=Xnow.x*Xnow.z;
        A(1,2)+=Xnow.y*Xnow.z;
    }
    A(1,0)= A(0,1);
    A(2,0)= A(0,2);
    A(2,1)= A(1,2);
    for (int i=0;i<3;i++)
    {
        for (int j=0;j<3;j++)
        {
            A(i,j)/=fnvec;
        }
    }




    double determina= A.Det();
    if (dbg_fitta==1)A.Print("A");

    double lmax= (A(0,0)+A(1,1)+A(2,2))*1.1;
    double lmin=0.;

   if (dbg_fitta==1)
    Gout<<"\n  Drt "<<determina<<"  lmin "<<lmin<<"  lmax "<<lmax<<"\n\n"<<endl;


    l0=A.Autoval3(-10.0);
    l2=A.Autoval3(lmax*1.2);
    l1=determina/(l0*l2);

      if (dbg_fitta==1)
    Gout<<"\n  Autovalori   l0 "<<l0<<"  l1 "<<l1<<"  l2 "<<l2<<"\n\n"<<endl;

    gvet V0=A.Autovet3(l0);
    gvet V1=A.Autovet3(l1);
    gvet V2=A.Autovet3(l2);

  if (dbg_fitta==1)
    {
        V0.print("V0");
        Gout<<" l0 "<<l0;
        V1.print("V1");
        Gout<<" l1 "<<l1;
        V2.print("V2");
        Gout<<" l2 "<<l2;
        Gout<<"\n\n scalari "<<setw(8)<<(V1%V2)<< " " <<setw(8)<<(V2%V0)<<" " <<setw(8)<<(V0%V1)<<"\n\n"<<endl;
        V2.print("V2");
        Gout<<" Ricostruito ";
    }


    return V2;
}


