/***************************************************************************
 *   Copyright (C) 2005 by Giuseppe Pierazzini          Pisa 20.02.05      *
 *   pierazzini@unipi.it                                                   *
 *                                     F l y o                             *
 *   Epsi/NA48                                                             *
 ***************************************************************************/
/* Authors G.Pierazzini - G.Ruggiero */

#include <parm.h>
#include "apparato.h"
#include "gmatrix.h"
#include "evento.h"
#include "reazione.h"
#include "clusgmp.h"
#include "clusgmp.h"

using namespace GmpConst;  

using namespace std;

int Strawdebug=-1;

int Singelpiano=0;

HitClus Hitp[4];




////////////////////
// CONSTRUCTOR & DESTRUCTOR //
////////////////////
TrgmpStraw::TrgmpStraw()
{

    ntrk=-1;
    ncls0=ncls1=ncls2=ncls3=0;

    if (Mnp33!=0) z_centro_mag= Mnp33->Get_Centro().z;
    else  {
        cout<<"\n TrgmpStraw: Error... Mnp33 not defined ....exit "<<endl;
        Gout<<"\n TrgmpStraw: Error... Mnp33 not defined ....exit "<<endl;
        exit(0);
    }

    Gout<<"\n\n Class TrgmpStraw done with  Z_mag "<< z_centro_mag;


    // Pointers to the cluster in each group of chamber
    pcl0=ChambClus[0];
    pcl1=ChambClus[1];
    pcl2=ChambClus[2];
    pcl3=ChambClus[3];
}
TrgmpStraw::~TrgmpStraw()
{}



//////////////////////////////////////////
// MAKE TRACK                                                //
// NOTA
//////////////////////////////////////////
void TrgmpStraw::Make_Trk()
{



    static int codemask[16]={0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4};

    int c0ok,c1ok,c2ok,c3ok, nchamb,nsingle;
    int c0flag[30],c1flag[30],c2flag[30],c3flag[30];     // flags to kill the used cluster
    for (int i=0;i<30;i++)c0flag[i]=c1flag[i]=c2flag[i]=c3flag[i]=1;// flags set

    double chicut=1000.0;
    ntrk=-1;


// Find the  Cluster number  in straw chambers..
    ncls0=ncls1=ncls2=ncls3=0;
    if ( pcl0!=0 ) ncls0=pcl0->nclust+1;
    if ( pcl1!=0 ) ncls1=pcl1->nclust+1;
    if ( pcl2!=0 ) ncls2=pcl2->nclust+1;
    if ( pcl3!=0 ) ncls3=pcl3->nclust+1;
    
    
//test per debug    
// if(Eventi_Generati>6586&&Eventi_Generati<6589  )Strawdebug=10;
//   else Strawdebug=-1;
    
    

    if (Strawdebug==1) {
        Gout<<"\n\n\n Inizio Debug in  Trgmpstraw  =====================  Tr  Tr ===============";
        
        for(int i=0;i<Mnp33->mhit;i++)Mnp33->M_Hits[i].P.print(Mnp33->M_Hits[i].pnome);
         for(int i=0;i<Mnp33->mhit;i++)Mnp33->M_Hits[i].Xlab.print(Mnp33->M_Hits[i].pnome);
        
        Gout<<"\nEv "<<Eventi_Generati<<" N_cluster per straw "
        << setw(2)<<ncls0<<" "<< setw(2)<<ncls1<<" "<< setw(2)<<ncls2<<" "<< setw(2)<<ncls3;

        for (int i=0;i<ncls0;i++)pcl0->Clus[i].Cluster_Print();
        Gout<<endl;
        for (int i=0;i<ncls1;i++)pcl1->Clus[i].Cluster_Print();
        Gout<<endl;
        for (int i=0;i<ncls2;i++)pcl2->Clus[i].Cluster_Print();
        Gout<<endl;
        for (int i=0;i<ncls3;i++)pcl3->Clus[i].Cluster_Print();
        Gout<<endl<<endl;
    }




    if (pcl0>0) {
        for (int i=0;i<ncls0;i++)pcl0->clsusato[i]=0;
    }
    if (pcl1>0) {
        for (int i=0;i<ncls1;i++)pcl1->clsusato[i]=0;
    }
    if (pcl2>0) {
        for (int i=0;i<ncls2;i++)pcl2->clsusato[i]=0;
    }
    if (pcl3>0) {
        for (int i=0;i<ncls3;i++)pcl3->clsusato[i]=0;
    }


    int maxcamere=0;// camere coinvolte.
    if (ncls0 >-1) maxcamere++;
    if (ncls1 >-1) maxcamere++;
    if (ncls2 >-1) maxcamere++;
    if (ncls3 >-1) maxcamere++;
    if (maxcamere <3) return; // minimum 3 chambers
    int Single_esiste=pcl0->single_plans+pcl1->single_plans+pcl2->single_plans+pcl3->single_plans;
    if (Strawdebug==1) Gout<<" Camere convolte "<<maxcamere<< " con single "<<Single_esiste;    

    for (int VisCam=maxcamere; VisCam>2;VisCam--)
    {
        for (nsingle=0;nsingle<VisCam-1;nsingle++)   // single plan loop
        {
            //      if (Single_esiste==0&&nsingle>0) continue;
            if (nsingle>Single_esiste)continue;

            int Istart=0;
            if (VisCam <4)Istart=-1;


//======================================================

            for ( int ic0=Istart;ic0<ncls0;ic0++)
            {
                c0ok=0;
                if (ic0>-1&&c0flag[ic0]>0)
                {
                    Hitp[0]= pcl0->Clus[ic0];
                    c0ok=1;// prima camera
                }
                else     Hitp[0].mask=Hitp[0].planes=0;

                for ( int ic1=Istart;ic1<ncls1;ic1++)
                {
                    c1ok=0;
                    if (ic1>-1&&c1flag[ic1]>0)  {
                        Hitp[1]= pcl1->Clus[ic1];
                        c1ok=2;  // seconda camera
                    }
                    else   Hitp[1].mask=Hitp[1].planes=0;

                    for ( int ic2=Istart;ic2<ncls2;ic2++)
                    {
                        c2ok=0;
                        if (ic2>-1&&c2flag[ic2]>0)  {
                            Hitp[2]= pcl2->Clus[ic2];
                            c2ok=4; // terza camera
                        }
                        else    Hitp[2].mask=Hitp[2].planes=0;

                        for ( int ic3=Istart;ic3<ncls3;ic3++)
                        {
                            c3ok=0;
                            if (ic3>-1&&c3flag[ic3]>0)  {
                                Hitp[3]= pcl3->Clus[ic3];
                                c3ok=8;
                            }
                            else     Hitp[3].mask=Hitp[3].planes=0;
// verify again the flags  because some hit may change flag values if used

                            if (ic0>-1&&c0flag[ic0]==0) continue;
                            if (ic1>-1&&c1flag[ic1]==0) continue;
                            if (ic2>-1&&c2flag[ic2]==0) continue;
                            if (ic3>-1&&c3flag[ic3]==0) continue;




// here find track,,===========================



                            short int mask_cam;
                            mask_cam=c0ok+c1ok+c2ok+c3ok;
                            nchamb=codemask[mask_cam];
                            if (nchamb!=VisCam)  continue; // no  enough active chamber

//verifica della configurazione per trattare i cluster con un piano
                            TrkNow.camere=VisCam;
                            TrkNow.used_plans=0;

                            if (Strawdebug==1) {
                                Gout<<"\n Ev "<< Eventi_Generati
                                <<" camere "<< VisCam
                                <<" Sgle_e "<<Single_esiste <<"  "<<setw(2)<<nsingle
                                <<" Ic: "<<setw(2)<<ic0<<" "<<setw(2)<<ic1<<" "<<setw(2)<<ic2<<" "<<setw(2)<<ic3<<" Flag ";
                                if (ic0>-1)Gout<<c0flag[ic0];
                                else Gout<<"?";
                                if (ic1>-1)Gout<<c1flag[ic1];
                                else Gout<<"?";
                                if (ic2>-1)Gout<<c2flag[ic2];
                                else Gout<<"?";
                                if (ic3>-1)Gout<<c3flag[ic3];
                                else Gout<<"?";
                            }

                            int vero=0;
                            if (Single_esiste>0)
                                if ((vero=Verify(mask_cam,nsingle))<0) {
                                    if (Strawdebug==10) Gout<<" Verify "<<mask_cam<<" single "<<nsingle<<" code "<<vero;
                                    continue;
                                }
// fitting  data
                            Singelpiano=nsingle;


                            double chiq= Fitting(mask_cam);
                            
                    //    if(chiq<0.0) Gout<<"\n  Tr ev " <<Eventi_Generati<<" error "<< chiq
                    //        <<" Ic: "<<setw(2)<<ic0<<" "<<setw(2)<<ic1<<" "<<setw(2)<<ic2<<" "<<setw(2)<<ic3;

                            TrkNow.ic0=ic0;
                            TrkNow.ic1=ic1;
                            TrkNow.ic2=ic2;
                            TrkNow.ic3=ic3;


                            if (Strawdebug==1)
                            {
                                if (chiq>0)TrkNow.Print_Trk();
                                else Gout<<" Rj chq "<<chiq;
                            }

                            if (ntrk>8) {
                                Gout<<"\n Too many tracks..."<<setw(3)<<ic0<<" "<<setw(3)<<ic1<<" "<<setw(3)<<ic2<<" "<<setw(3)<<ic3
                                <<" chi "<<chiq;
                                break;
                            }


//save track
                            if (chiq>-0.001&&chiq<chicut)
                            {

                                ntrk++;
                                if (Strawdebug==1)Gout<<"\n Accettata ntrk "<<ntrk;
                                // save traccia

                                TrkNow.clstgx=-99.0;
                                TrkNow.clstgy=-99.0;
                                if (ic0>-1&&ic1>-1)
                                {
                                    double x0= Hitp[0].Xcls.x;
                                    double x1= Hitp[1].Xcls.x;
                                    double y0= Hitp[0].Xcls.y;
                                    double y1= Hitp[1].Xcls.y;
                                    double z0= Hitp[0].Xcls.z;
                                    double z1= Hitp[1].Xcls.z;
                                    TrkNow.clstgx=(x1-x0)/(z1-z0);
                                    TrkNow.clstgy=(y1-y0)/(z1-z0);
                                }

                                TrkNow.tgxfit=TrkNow.P_in.x/TrkNow.P_in.z;
                                TrkNow.tgyfit=TrkNow.P_in.y/TrkNow.P_in.z;

                                Traccia[ntrk]=TrkNow;
                                Traccia[ntrk].id=ntrk;
                                Traccia[ntrk].camere=nchamb;
                                Traccia[ntrk].single=nsingle;
                                Traccia[ntrk].ic0=ic0;
                                Traccia[ntrk].ic1=ic1;
                                Traccia[ntrk].ic2=ic2;
                                Traccia[ntrk].ic3=ic3;

                                if (ic0>-1) {
                                    c0flag[ic0]=0;
                                    pcl0->clsusato[ic0]=1;

                                }
                                if (ic1>-1) {
                                    c1flag[ic1]=0;
                                    pcl1->clsusato[ic1]=1;

                                }
                                if (ic2>-1) {
                                    c2flag[ic2]=0;
                                    pcl2->clsusato[ic2]=1;

                                }
                                if (ic3>-1) {
                                    c3flag[ic3]=0;
                                    pcl3->clsusato[ic3]=1;

                                }

                            }
                        }
                    }
                }
            }
        }
    }

    if (Strawdebug==1)Gout<<"\n N tracce: "<<ntrk+1;

    charge_all=0;

    for (int i=0;i<ntrk+1;i++)
    {
        charge_all+=Traccia[i].charge;

        //  Traccia[i].Print_Trk();

    }

    Track2Root();


}

//=====================================================================
///=====================================================================
// error routine
gmatrix Get_Error(  double P);


//================================F I T T I N G =======================


double TrgmpStraw::Fitting(short int mask_cam)   //,HitClus *//Hitp)
{

//    Gout<<"\n\n Ev "<< Eventi_Generati<<" Fitting  mask " << mask;


    
    
    int gradi=0;
    int pcam=0;
    unsigned short mskc=1;
    gmatrix AF(8,5),Y(8,1),FC(8,1);


    
    int idc=0;
    for (int icm=0;icm<4;icm++)
    {
        if ( (mask_cam&mskc)>0)  {  // la camera  esiste...
            pcam++;
//            idc=icm*2;
            idc=icm;
            double dzmg=Hitp[icm].Xcls.z-z_centro_mag;
            int piani=Hitp[icm].planes;


            // code to fill F and Y  when there is more than 1 plan
            if (piani>1) {
                Y(idc,0)  =Hitp[icm].Xcls.x;
                Y(idc+4,0)=Hitp[icm].Xcls.y;
                AF(idc,2)=AF(idc+4,0)=1.;
// calcolo degli errori 
 //               G(idc,idc)=varsigmq; // Si pesano di più i clusters delle prime due camere.
 //               G(idc+4,idc+4)=varsigmq;  // Si pesano di più i clusters delle prime due camere.
 
 
                gradi+=2;
                if (icm<2) {
                    AF(idc,3)=AF(idc+4,1)=dzmg;                }
                else {
                    AF(idc,4)=AF(idc+4,1)=dzmg;


                }
            }

            // code to fill F and Y when there is only one plan
            else
            {
                short int mask_plan=Hitp[icm].mask;

                if (mask_plan&1) {     //piano u   solo
                    AF(idc,0)=SINCOS45;
                    AF(idc,1)=dzmg*SINCOS45;
                    AF(idc,2)=SINCOS45;
   //                 G(idc,idc)=varsigmq;
                    gradi++;
                    if (icm<2) {
                        AF(idc,3)=dzmg*SINCOS45;            
                    }
                    else AF(idc,4)=dzmg*SINCOS45;
                    Y(idc,0)  =Hitp[icm].Ufil;
                    FC(idc,0)=(Hitp[icm].Centro.x+Hitp[icm].Centro.y)*SINCOS45;


                }
                else if (mask_plan&2) {  //piano v   solo
                    AF(idc+4,0)=-SINCOS45;
                    AF(idc+4,1)=-dzmg*SINCOS45;
                    AF(idc+4,2)=SINCOS45;
//                     G(idc+4,idc+4)=varsigmq;
                     gradi++;
                    if (icm<2) {
                        AF(idc+4,3)=dzmg*SINCOS45;                     
                    }
                    else AF(idc+4,4)=dzmg*SINCOS45;
                    // Y(idc+4,0)  =Hitp[icm].Filo+(Hitp[icm].Centro.x-Hitp[icm].Centro.y)*SINCOS45 ;
                    Y(idc+4,0)  =Hitp[icm].Vfil;
                    FC(idc+4,0) =(Hitp[icm].Centro.x-Hitp[icm].Centro.y)*SINCOS45 ;
                }
                else if (mask_plan&4) {    //piano x   solo
                    AF(idc,2)=1.;
 //                       G(idc,idc)=varsigmq; 
                        gradi++;
                    
                    if (icm<2) {
                        AF(idc,3)=dzmg;
                    }
                    else AF(idc,4)=dzmg;
                    Y(idc,0)=Hitp[icm].Xcls.x;


                }
                else if (mask_plan&8) {     //piano y solo
                    AF(idc+4,0)=1.;
                    AF(idc+4,1)=dzmg;
                    Y(idc+4,0)=Hitp[icm].Xcls.y;
 //                 G(idc+4,idc+4)=varsigmq;
                    gradi++;
                }

            }
            if (Strawdebug==10) Gout<<setprecision(2)<<" Ev "<<Eventi_Generati 
              <<fixed<<"["<<setw(8)<<Y(idc,0)<<" "<<setw(8)<<Y(idc+4,0)<<"]";
        }
        mskc*=2;
                
    }
    gradi=gradi-5;
    if(gradi<1)gradi=1;
 
// F and Y  matrix done

    if (pcam<3) return -1.0; // at least tree chambers...

//=============== calcolo dei Y-values

gmatrix M(5,5), Noto(5,1), X(5,1),FS(8,1),G(8,8);

double pinc = -1.;
double ymg,tgyz,xmg,tgxz_before,tgxz_after,carica;
 gvet Versor_in;
 
for(int itera=0;itera<2;itera++)
{
// inizializzazioni matrici degli errori 
// si fanno due iterazioni... 
// la prima  (pinc negativo) con errori solo di misura e si calcola l'impulso 
// nella seconda (pinc positivo), noto l'impulso, si tiene conto del multiple scattering
  G=Get_Error(pinc);   

    M=(~AF)*(G*AF);   // 5x5

    M.Det();


    if (Strawdebug==10) {
        G.Print("G");
        AF.Print("AF");
        M.Print("M");// debug =====
    }


    if (fabs(M.detrm)<100000.) {
        Gout<<"\n Fitting mtx ERROR Ev Deteminante nullo!!!! "<<Eventi_Generati<<" mask_cam "<<mask_cam
        <<" msk "<< Hitp[0].mask  <<" "<< Hitp[1].mask  <<" "<< Hitp[2].mask  <<" "<< Hitp[3].mask;
        return -50;
    }
    FS=Y+FC;
    Noto=(~AF)*(G*FS);
    X=  (!M)*Noto;

//====== get the X-parameters by name
  
    ymg   =X(0,0);  // at magnet
    xmg   =X(2,0);  // at magnet
    tgyz  =X(1,0);
    tgxz_before=X(3,0);
    tgxz_after =X(4,0);
    
 // ==================== calcolo del momento ===========
    Versor_in.setv(tgxz_before,tgyz,1.);
    Versor_in=Versor_in.NVerso();
    double cdip=sqrt(Versor_in.x*Versor_in.x+Versor_in.z*Versor_in.z);
    double kik=Mnp33->Get_Kik();
    pinc= (kik/ (atan(tgxz_after)-atan(tgxz_before )) )/cdip;
    carica=1.;
    if (pinc<0.0)carica=-1.;
    pinc=abs(pinc);  
    
}
    
    
    
    
//
    double chiq=0;
    gmatrix DY(8,1),sumq(1,1);
    DY=(AF*X)-FS;

    sumq= (~DY)*G*DY;   // chiq...
    chiq=sumq(0,0);
    chiq/=double(gradi);
  
    
    
// per debug
    if (Strawdebug==10&&chiq<5.&& mask_cam==15&&Singelpiano==1)

    {
        Gout<<"\n Ev "<<Eventi_Generati<<" mask_cam "<<mask_cam
        <<" mskpl "<< Hitp[0].mask  <<" "<< Hitp[1].mask  <<" "<< Hitp[2].mask  <<" "<< Hitp[3].mask
        <<"  chiq "<<chiq;
        Noto.Print("Noto");
        X.Print("X");
        Y.Print("Y");
        DY.Print("DY");
    }

// debug =====
    if (Strawdebug==10&&chiq<200.)
    {

        Gout<<"\n\n Fitting Ev "<<Eventi_Generati<<" mask_cam "<<mask_cam
        <<" msk "<< Hitp[0].mask  <<" "<< Hitp[1].mask  <<" "<< Hitp[2].mask  <<" "<< Hitp[3].mask;
        Noto.Print("Noto");
        X.Print("X");
        Y.Print("Y");
        //   F.Print("F");

    }

    if (Strawdebug==10&&chiq<200.) {
        DY.Print("DY");
        Gout<<"\n Ev "<<Eventi_Generati<<" chiq "<<chiq;
    }
// end debug =====


    if (chiq> 1000.) {
        if (pcam==4) return -2.;    // -2
        else return-3;
    }

    // test di convergenza in y verso la zona di decadime
    double zby_y=0.;
    if(abs(tgyz)>0.0005)
    {
    zby_y= z_centro_mag-ymg/tgyz;
    if (Strawdebug==10)Gout<<"\n ym "<<ymg<<" z_y "<<zby_y<<" tgyz "<<tgyz;
    if (zby_y<6000.||zby_y>20000.)return  -10.;
}
    
//     { 
//       Gout<<"\n  Tr ev " <<Eventi_Generati<<" error "<< chiq;                    
//       Gout<<" msk "<<mask_cam<<" ym "<<ymg<<" z_y "<<zby_y<<" tgyz "<<tgyz;return -10.;}

// test di convergenza in x verso la zona di decadimento
// attenzione il beam può essere inclinato??? aggiustare secondo il caso
    double zby_x=0.;
    double tgxb=+0.00124;
    double tgxx=tgxz_before-tgxb;
    if(fabs(tgxx)>0.0005)
    {
    double xb=+11.;   // il beam passa per x =+12. al magnete
    zby_x= z_centro_mag+(xb-xmg)/tgxx;
    if (Strawdebug==10)Gout<<"\n xm "<<xmg<<" z_x "<<zby_x<<" tgxz "<<tgxz_before;
    if (zby_x>20000.)return -11.;
    }
    

// // ==================== calcolo del momento ===========
//     gvet Versor_in(tgxz_before,tgyz,1.);
//     Versor_in=Versor_in.NVerso();
//     double cdip=sqrt(Versor_in.x*Versor_in.x+Versor_in.z*Versor_in.z);
//     double kik=Mnp33->Get_Kik();
//     double pinc= (kik/ (atan(tgxz_after)-atan(tgxz_before )) )/cdip;
//     double charge=1.;
//     if (pinc<0.0)charge=-1.;
//     pinc=abs(pinc);
    
      if (Strawdebug==10&&chiq<200.)
        Gout<<"\n End fitting: Impulso "<<pinc<< " vero "<<Mnp33->M_Hits[0].P.mom<<endl;




    TrkNow.mask=mask_cam;
    TrkNow.msk0= Hitp[0].mask;
    TrkNow.msk1= Hitp[1].mask;
    TrkNow.msk2= Hitp[2].mask;
    TrkNow.msk3= Hitp[3].mask;
    TrkNow.zby_y=zby_y;
    TrkNow.zby_x=zby_x;
    TrkNow.chiq_fit= chiq;

    TrkNow.dx0=DY(0,0);
    TrkNow.dy0=DY(1,0);
    TrkNow.dx1=DY(2,0);
    TrkNow.dy1=DY(3,0);
    TrkNow.dx2=DY(4,0);
    TrkNow.dy2=DY(5,0);
    TrkNow.dx3=DY(6,0);
    TrkNow.dy3=DY(7,0);




    TrkNow.Xmg.setv(xmg,ymg,z_centro_mag);
    TrkNow.P_in=Versor_in*pinc;
    gvet Versor_out(tgxz_after,tgyz,1.);
    Versor_out=Versor_out.NVerso();
    TrkNow.P_out=Versor_out*pinc;
    TrkNow.charge=carica;

// estrapolazione traccia all'odoscopio X
    double dkam= (hodox->Get_Cface().z-z_centro_mag)/Versor_out.z;
    TrkNow.Xhodx=TrkNow.Xmg+dkam*Versor_out;

    // estrapolazione traccia al calorimetro
    double dlkr= (lkry->Get_Cface().z-z_centro_mag)/Versor_out.z;
    TrkNow.Xlkr=TrkNow.Xmg+dlkr*Versor_out;





    return chiq;  // return chiq;

}
//====================================================================================

int TrgmpStraw::Verify(short int mask_cam,int single)

{

    //test the number of single plans accepted
    // for 4 active chambers  we accept up to two different single planes, with at least one no   Y (8)
    // for 3 active chambers we accept up to two different single planes but both no  Y (8)

    unsigned short mskc=1;
    unsigned short msklast=0;
    unsigned short mskplan=0;
    TrkNow.used_plans=0;
    int singlepiani=0;
    for (int icm=0;icm<4;icm++)
    {
        if ( (mask_cam&mskc)>0)  {  // la camera esiste...
            short int mskpl= Hitp[icm].mask;
            if (single==1&&mskpl==8) return -1; // the y plan is not usefull in only trhee plans
            if (mskpl==1||mskpl==2||mskpl==4||mskpl==8)
            {
                if (msklast==8 && mskpl==8) return -2;
                singlepiani++;
                msklast=mskpl;
                mskplan=(mskplan|mskpl);
            }
            if (singlepiani>single) return -3;
        }
        mskc*=2;
    }
    TrkNow.used_plans=mskplan;  // da rivedere........



    return 1;

}



//=================
void Track::Print_Trk()
{
    Gout<<"\n Ev "<<Eventi_Generati<<" id "<<id<<" ch "<<charge<<" msk "<<mask<<" cam "<<camere<<" SINGL "<<single
    <<" msk: "<<msk0<<" "<<msk1<<" "<<msk2<<" "<<msk3
    <<setprecision(4)<<" chiq "<<chiq_fit;
    Xmg.print("Xmg");
    Gout<<" Ic.. "<<ic0<<" "<<ic1 <<" "<<ic2 <<" "<<ic3 ;
    P_in.print("P_in");
    P_out.print("P_out");
    Gout<<" Kik "<<(P_out-P_in).px;

}
// ================

void TrgmpStraw::Track2Root()
{
//int ntrk,mask[MAXTRACK],camere[MAXTRACK],ic0[MAXTRACK],ic1[MAXTRACK],ic2[MAXTRACK],ic3[MAXTRACK];
//  float chiq[MAXTRACK], charge[MAXTRACK], xmg[MAXTRACK], ymg[MAXTRACK], px[MAXTRACK],py[MAXTRACK],pz[MAXTRACK];
    ntrk_done=ntrk+1;

    for (int i=0;i<ntrk_done;i++)
    {
        Trk2Root.id[i]=  Traccia[i].id;
        Trk2Root.mask[i]=  Traccia[i].mask;
        Trk2Root.camere[i]=Traccia[i].camere;
        Trk2Root.single[i]=Traccia[i].single;
        Trk2Root.usepl[i]=Traccia[i].used_plans;
        Trk2Root.ic0[i]=   Traccia[i].ic0;
        Trk2Root.ic1[i]=   Traccia[i].ic1;
        Trk2Root.ic2[i]=   Traccia[i].ic2;
        Trk2Root.ic3[i]=   Traccia[i].ic3;
        Trk2Root.zbyy[i]=  Traccia[i].zby_y;
        Trk2Root.zbyx[i]=  Traccia[i].zby_x;
        Trk2Root.chiq[i]=  Traccia[i].chiq_fit;
        Trk2Root.charge[i]=Traccia[i].charge;

        Trk2Root.dx0[i]=Traccia[i].dx0;
        Trk2Root.dx1[i]=Traccia[i].dx1;
        Trk2Root.dx2[i]=Traccia[i].dx2;
        Trk2Root.dx3[i]=Traccia[i].dx3;
        Trk2Root.dy0[i]=Traccia[i].dy0;
        Trk2Root.dy1[i]=Traccia[i].dy1;
        Trk2Root.dy2[i]=Traccia[i].dy2;
        Trk2Root.dy3[i]=Traccia[i].dy3;


        Trk2Root.xmg[i]=   Traccia[i].Xmg.x;
        Trk2Root.ymg[i]=   Traccia[i].Xmg.y;
        Trk2Root.p[i]=     Traccia[i].P_in.mom;
        Trk2Root.py[i]=    Traccia[i].P_in.py;
        Trk2Root.pxin[i]=  Traccia[i].P_in.px;
        Trk2Root.pzin[i]=  Traccia[i].P_in.pz;
        Trk2Root.pxout[i]= Traccia[i].P_out.px;
        Trk2Root.pzout[i]= Traccia[i].P_out.pz;
        Trk2Root.xhodx[i]=   Traccia[i].Xhodx.x;
        Trk2Root.yhodx[i]=   Traccia[i].Xhodx.y;
        Trk2Root.xlkr[i]=   Traccia[i].Xlkr.x;
        Trk2Root.ylkr[i]=   Traccia[i].Xlkr.y;
        Trk2Root.clstgx[i]=   Traccia[i].clstgx;
        Trk2Root.clstgy[i]=   Traccia[i].clstgy;
        Trk2Root.tgxfit[i]=   Traccia[i].tgxfit;
        Trk2Root.tgyfit[i]=   Traccia[i].tgyfit;





    }


}
//   auxiliary 
// routine to evaluate the errors with nd without  multiple scattering

// error routine
gmatrix  Get_Error( double P)
{
 // per la documentazione vedi il file rettafitMag.pdf
 
    static double simq=0.000225;  // si suppone un errore di .15 mm (aggiustare?)
    static double vsimq=1./simq; 
    
    
    
    static int First_time=0;  
    static gmatrix Vbase(3,3);
    static gmatrix Gbase(8,vsimq);
    
 //  Gout<<"\n ********* volta "<<First_time<<" p="<<P;
    
    if(First_time==0)
    {
// calcolo dei parametri fissi..... solo la prima volta 
// rivedere le formule delle correlazioni.
    double dz0=1056; // distanza camera 1-0 
    double dz1=2095; // distanza camera 2-0 
    double dz2=3538; // distanza camera 3-0 
    double sigtetq=225.e-8; // dteta quadratico medio per una camera (quattro viste). ad un gev... 
      Vbase(0,0)=dz0*dz0*sigtetq;
      Vbase(1,1)=Vbase(0,0)+dz1*dz1*sigtetq;
      Vbase(2,2)=Vbase(1,1)+dz2*dz2*sigtetq;
      Vbase(0,1)=Vbase(1,0)= dz0*dz1*sigtetq;
      Vbase(0,2)=Vbase(2,0)= dz0*dz2*sigtetq;
      Vbase(1,2)=Vbase(2,1)= dz1*dz2*sigtetq;
      Vbase.Det();
//   Vbase.Print("Vbase");
//    
//   errori senza multpile scattering
//   Gbase.Print("Gbase");
 First_time=10;
 }
 
 
 

if(P<0.0001) return Gbase;
double mscq=1./(P*P);
// now with multiple scattering
// errori senza multpile scattering

// sotomatrice 3x3 per  errori 
// in X e in y con correlazione ( le due matrici sono identiche)  
 gmatrix V(3,3),Gv(3,3);
 V=Vbase;
 V*=mscq;// normalizzo gli errori di scttering multiplo
// aggiungo gli errori di misura  

  V(0,0)+=simq;
   V(1,1)+=simq;
   V(2,2)+=simq;
    
   V.Det();
//    V.Print("V1");  
   Gv=!V;    // inverto ed ottengo la matrice Gv ridotta 
//   Gv.Print("Gv");
   
 gmatrix G(8,8); 
 G(0,0)=G(4,4)=vsimq;
 for (int i=0;i<3;i++)   
 {
  for (int j=0;j<3;j++)     
  {
  G(i+1,j+1)= G(i+5,j+5)=Gv(i,j);
}}

//G.Print("G");

return G;
}
